#ifndef _LCL_APP_CONTROL_H_
#define _LCL_APP_CONTROL_H_

#include <lcl.h>

class LCL_EXPORT LApplicationControl_Base;

#include <lcl/lobject.h>
#include <lcl/lsocket.h>
#include <lcl/lthread.h>
#include <lcl/lbuffer.h>
#include <lcl/lsem.h>
#include <lcl/lhash.h>
//#include <lcl/liobuffer.h>

class LCL_EXPORT LApplicationControl_Class;

class LCL_EXPORT LApplicationControl_Base : public LObject {
 public:
  class LCL_EXPORT Packet;
  class LCL_EXPORT BasicRequest;
  class LCL_EXPORT Request;
  class LCL_EXPORT Reply;

 public:

  class LCL_EXPORT Packet {
  protected:
    
    unsigned long serial;
    unsigned long replySerial;
    LString command;
    LHash<LString,LString> vars, target;

  public:

    static Packet fromData ( LBuffer const & data );

    Packet ();

    LBuffer getData ();

    void setReplySerial ( unsigned long );
    unsigned long getReplySerial () const;

    void setSerial ( unsigned long );
    unsigned long getSerial () const;

    void setCommand ( LString const & _command );
    LString getCommand () const;

    LHash<LString,LString> & getTarget ();
    LHash<LString,LString> const & getTarget () const;

    LHash<LString,LString> & getVars ();
    LHash<LString,LString> const & getVars() const;
  };

  class LCL_EXPORT BasicRequest {
    friend class LCL_EXPORT LApplicationControl_Base;
  protected:
    static int C_timeout;
    int timeout;
    Packet packet;
    unsigned long sendTime;

    void setSendTime( unsigned long );
    unsigned long getSendTime() const;

  public:

    BasicRequest ();
    bool timedOut () const;

    int getTimeout() const;
    void setTimeout ( int );

    Packet & getPacket ();
    Packet const & getPacket () const;

    LEmitter0 eventCompleted;
    LEmitter0 eventTimedOut;
  };

  class LCL_EXPORT Request : public BasicRequest {
    friend class LCL_EXPORT LApplicationControl_Base;
  protected:
    bool reply;
    LManagedPointer<Reply> replyData;
    LManagedPointer<LSemaphore> semaphore;

    void flagEvent ();

  public:
    Request ();
    bool gotReply () const;
    LManagedPointer<Reply> getReplyData () const;

    void wait ();

    LEmitter1 < LManagedPointer < Request > > requestCompleted;
    LEmitter1 < LManagedPointer < Request > > requestTimedOut;
  };

  class LCL_EXPORT Reply : public BasicRequest  {
  protected:
  public:
    Reply ();
    Reply ( Request const & );
    Reply ( LManagedPointer<Request> );
  };

 protected:

  const unsigned long C_timeout;

  LSet < LManagedPointer<Request> > requestList;
  LSet < LManagedPointer<Reply> > replyList;
  LHash < LString, LApplicationControl_Class * > classHash;

  LSocket4Inet socket;

  LBuffer writeBuffer;
  LBuffer readBuffer;

  unsigned long lastData;
  unsigned long serial;

  virtual void connectionLost () = 0;

  void setLastDataNow ();
  bool isConnectionTimedOut () const;

  void processInputData ( void );
  void processRequest ( LManagedPointer<Request> packet );
  void processReply ( LManagedPointer<Reply> reply );

  virtual void marshallClass ( LManagedPointer<Request> packet );
  virtual void gotPing ( LManagedPointer<Request> request );

  void sendData ( LBuffer const & data );

  void processLists ();

  void timeoutRequest ( LManagedPointer<Request> request );
  void timeoutReply ( LManagedPointer<Reply> reply );

  virtual void eventCompleteRequest ( LManagedPointer<Request> request );
  virtual void eventTimeoutRequest ( LManagedPointer<Request> request );

 public:

  LEmitter1 < LManagedPointer < Request > > requestTimeout;
  LEmitter1 < LManagedPointer < Reply > > replyTimeout;

  LApplicationControl_Base ( LSocket4Inet const & socket );

  void writeEvent ( LFileDesc * fd );
  void readEvent ( LFileDesc * fd );

  void setEventFlags ();

  void sendRequest ( Request const & request );
  void sendRequest ( LManagedPointer<Request> request );
  void sendReply ( Reply const & reply );
  void sendReply ( LManagedPointer<Reply> reply );

  void sendPing ( void );

  void registerClass ( LString const &, LApplicationControl_Class * );
  void unregisterClass ( LString const & );

  virtual LManagedPointer<Request> sendEvent ( LString const & channel, LHash<LString,LString> const & data, LString const & app = "", LString const & node = "" );
};

class LCL_EXPORT LApplicationControl : public LApplicationControl_Base {
 protected:

  bool connected;
  LSockAddr4Inet addr;
  unsigned long tix;

  LMethodThreadFn<LApplicationControl> runnerFn;

  virtual void connectionLost ();

 public:
  LApplicationControl ();

  void timeoutEvent ( void );
  void runner ( void );

};

template < class Serializer >
LCL_EXPORT Serializer & operator << ( Serializer & ser, LApplicationControl_Base::Packet const & packet );
template < class Serializer >
LCL_EXPORT Serializer & operator >> ( Serializer & ser, LApplicationControl_Base::Packet & packet );

template < class Serializer >
LCL_EXPORT Serializer & operator << ( Serializer & ser, LApplicationControl_Base::Packet const & packet )
{
  ser << packet.getSerial();
  ser << packet.getReplySerial();
  ser << packet.getCommand();
  ser << packet.getVars();
  ser << packet.getTarget();

  return ser;
}

template < class Serializer >
LCL_EXPORT Serializer & operator >> ( Serializer & ser, LApplicationControl_Base::Packet & packet )
{
  unsigned long tmp1, tmp2;
  LString tmp3;

  ser >> tmp1 >> tmp2 >> tmp3;

  packet.setSerial ( tmp1 );
  packet.setReplySerial ( tmp2 );
  packet.setCommand ( tmp3 );
  ser >> packet.getVars();
  ser >> packet.getTarget();

  return ser;
}


#include <lcl/lappcontrol_class.h>

#endif
