#ifndef _LCL_APP_EVENT_H_
#define _LCL_APP_EVENT_H_

#include <lcl.h>

#include <lcl_common.h>
#include <lcl/lqueue.h>
#include <lcl/lappcontrol_class.h>

class LCL_EXPORT LApplicationEvent;
class LCL_EXPORT LApplicationEventQueue;
class LCL_EXPORT LApplicationEventManager;

class LCL_EXPORT LApplicationEvent {
  friend class LCL_EXPORT LApplicationEventQueue;
 protected:
  LString channel;
  LHash<LString,LString> data;
 public:
  void setChannel ( LString const & _channel );
  LString getChannel () const;
  void setData ( LHash<LString,LString> const & _data );
  LHash<LString,LString> const & getData () const;
};


class LCL_EXPORT LApplicationEventManager {
  friend class LCL_EXPORT LApplicationEventQueue;
  friend class LCL_EXPORT LApplicationEvent_Class;
 private:
  LGuardMutex controlLock;
  LSet<LApplicationEventQueue*> m_queues;
 protected:
  void registerQueue ( LApplicationEventQueue * queue );
  void unregisterQueue ( LApplicationEventQueue * queue );
  void push ( LApplicationEvent const & e );
 public:
};


class LCL_EXPORT LApplicationEventQueue {
  friend class LCL_EXPORT LApplicationEventManager;
 private:
  LGuardMutex controlLock;
  LApplicationEventManager & m_mgr;
  LSet<LString> m_channelList;
  LQueueSender<LApplicationEvent> m_sender;
 protected:
  void inject ( LApplicationEvent const & );
  bool match ( LString const & );
 public:
  LApplicationEventQueue ( LApplicationEventManager & mgr );
  LApplicationEventQueue ( LApplicationEventQueue const & p1 );
  ~LApplicationEventQueue ();
  
  void setQueueReceiver ( LQueueReceiver<LApplicationEvent> & receiver );
  void addChannel ( LString const & );
  void removeChannel ( LString const & );
};

class LCL_EXPORT LApplicationEvent_Class : public LApplicationControl_Class {
 protected:
  LApplicationEvent_Class ();
  static LApplicationEvent_Class * theInstance;
 public:
  static LApplicationEvent_Class * getInstance();

 protected:
  LApplicationEventManager m_mgr;

 public:
  virtual void runner ( void );
  virtual ~LApplicationEvent_Class ();

  LApplicationEventManager & getManager ();
};

#endif
