#ifndef _LCL_ASCII_PROTOCOL_
#define _LCL_ASCII_PROTOCOL_

#include <lcl.h>

#include <lcl_common.h>
#include <lcl/lsockaddr.4inet.h>
#include <lcl/lsocket.h>

class LCL_EXPORT LASCIIProtocolHandler;
class LCL_EXPORT LASCIIProtocolFactory;
class LCL_EXPORT LASCIIProtocolConnection;

class LCL_EXPORT LASCIIProtocolHandler {
 protected:
  LASCIIProtocolConnection * m_connection;
 public:
  LASCIIProtocolHandler ();
  virtual ~LASCIIProtocolHandler ();
  virtual void gotLine ( LString const & line ) = 0;
  virtual void setConnection ( LASCIIProtocolConnection * connection );
  virtual luint32 maxMessageSize () const = 0;
};

class LCL_EXPORT LASCIIProtocolFactory {
 protected:
 public:
  LASCIIProtocolFactory ();
  virtual ~LASCIIProtocolFactory ();
  virtual LASCIIProtocolHandler * createHandler () = 0;
};

class LCL_EXPORT LASCIIProtocolConnection {
 protected:  
  LASCIIProtocolHandler * m_handler;
  LFileDesc m_fileDesc;
  LString m_sendBuffer;
  LString m_readBuffer;
  bool m_connected;
  bool m_closeMarker;
  luint32 m_lastData;
  luint32 m_timeout;
  LSlotConnection m_readSlot, m_writeSlot, m_timeoutSlot;
  bool m_skipLine;

  LGuardMutex m_sendLock;
  LGuardMutex m_readLock;

  void checkEventMasks ( void );

 public:

  LASCIIProtocolConnection ( LASCIIProtocolHandler * handler, LFileDesc const & fileDesc );
  virtual ~LASCIIProtocolConnection ();

  virtual void sendLine ( LString const & line );

  virtual void eventWrite ( LFileDesc * fd );
  virtual void eventRead ( LFileDesc * fd );

  virtual void requestClose ( void );
  virtual void close ( void );

  virtual bool isConnected () const;
  virtual void setTimeout ( luint32 timeout );
  virtual void tickLastData ();
  virtual bool isTimedOut () const;
  virtual void eventTimeout ();
  virtual void eventSelectTimeout ();
};

class LCL_EXPORT LASCIIProtocolTCPClient : public LASCIIProtocolConnection {
 protected:
 public:
  LASCIIProtocolTCPClient ( LASCIIProtocolFactory * );
  void connect ( LSockAddr4Inet const & addr );
};

class LCL_EXPORT LASCIIProtocolTCPServer {
 protected:
  LGuardMutex m_lock;
  LManagedPointer<LASCIIProtocolFactory> m_factory;
  LHeap < LManagedPointer<LASCIIProtocolConnection> > m_connList;
  LSlotConnection m_tickerSlot;
 public:
  LASCIIProtocolTCPServer ( LManagedPointer<LASCIIProtocolFactory> factory );
  virtual ~LASCIIProtocolTCPServer ();

  void newConnection ( LSocket4Inet & connection );

  void removeDead ( void );
};

#endif
