/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#ifndef _LCONFIGMODULE_H_
#define _LCONFIGMODULE_H_

#include <lcl.h>

class LCL_EXPORT LConfigReader;
class LCL_EXPORT LConfigWriter;

#include <lcl/liostream.h>
#include <lcl/lconfsys.h>
#include <lcl/lfile.h>
#include <lcl/lpointer.h>

class LCL_EXPORT LConfigReader {
 protected:
 public:
  virtual void read ( LConfigSystem & config ) const = 0;
  virtual ~LConfigReader ();
};

class LCL_EXPORT LConfigWriter {
 protected:
 public:
  virtual void write ( LConfigSystem & config ) const = 0;
  virtual ~LConfigWriter ();
};

class LCL_EXPORT LConfigFileReader : public LConfigReader {
protected:
  LIStream * m_stream;
  LManagedPointer<LFile> m_file;
public:
  LConfigFileReader ( LIStream & instream );
  LConfigFileReader ( LString const & fname );
  virtual ~LConfigFileReader ();

  virtual void read ( LConfigSystem & config ) const;
  virtual void read_stream ( LIStream & instream, LConfigSystem & config ) const = 0;
};

class LCL_EXPORT LConfigFileWriter : public LConfigWriter {
protected:
  LOStream * m_stream;
  LManagedPointer<LFile> m_file;
public:
  LConfigFileWriter ( LOStream & outstream );
  LConfigFileWriter ( LString const & fname, int oa = LFDManager::write | LFDManager::truncate | LFDManager::create , int om = LFDManager::all_r  | LFDManager::user_w );
  virtual ~LConfigFileWriter ();
  
  virtual void write ( LConfigSystem & config ) const;
  virtual void write_stream ( LOStream & outstream, LConfigSystem & config ) const = 0;
};

class LCL_EXPORT LConfigReader_default : public LConfigFileReader {
protected:
public:
  LConfigReader_default ( LIStream & );
  LConfigReader_default ( LString const & );

  virtual ~LConfigReader_default ();
  virtual void read_stream ( LIStream & instream, LConfigSystem & config ) const;
};

class LCL_EXPORT LConfigWriter_default : public LConfigFileWriter {
 protected:
 public:
  LConfigWriter_default ( LOStream & );
  LConfigWriter_default ( LString const & fname, int oa = LFDManager::write | LFDManager::truncate | LFDManager::create , int om = LFDManager::all_r  | LFDManager::user_w );

  virtual ~LConfigWriter_default ();
  virtual void write_stream ( LOStream  & stream, LConfigSystem & config ) const;
};

#endif
