#ifndef _LCRYPT_H_
#define _LCRYPT_H_

#include <lcl.h>
#include <lcl/lbuffer.h>
#include <lcl/lvcs.h>

/*!
 * \defgroup Cryptography Cryptography
 */

/*!
 * \class LCryptKey
 * \brief A base crypt key class
 * \ingroup Cryptography
 */

class LCL_EXPORT LCryptKey {
  LVCS_DECLARE;
 protected:
  //! Stores the key value
  LBuffer key;

 public:

  LCryptKey ( );                                   //!< Default constructor
  LCryptKey ( LCryptKey const & p1 );              //!< Copy constructor

  void set ( LBuffer const & value );              //!< Set the value of a key
  LBuffer get ( ) const;                           //!< Get the value of a key
};

/*!
 * \class LCryptSecretKey
 * \brief A secret key class used in symetric cryptography
 * \ingroup Cryptography
 */

class LCL_EXPORT LCryptSecretKey : public LCryptKey {
  LVCS_DECLARE;
 protected:
 public:
};

/*!
 * \class LCryptPublicKey
 * \brief A public key class used in asymetric cryptography
 * \ingroup Cryptography
 */

class LCL_EXPORT LCryptPublicKey : public LCryptKey{
  LVCS_DECLARE;
 protected:
 public:
  virtual bool isValid () const;
};

/*!
 * \class LCryptPrivateKey
 * \brief A private key class used in asymetric cryptography
 * \ingroup Cryptography
 */

class LCL_EXPORT LCryptPrivateKey : public LCryptKey {
  LVCS_DECLARE;
 protected:
 public:
  virtual bool isValid () const;
};

/*!
 * \class LCryptModule
 * \brief A base class for cryptography modules
 * \ingroup Cryptography
 */

class LCL_EXPORT LCryptModule {
  LVCS_DECLARE;
 protected:
 public:
  virtual void encode ( LBuffer & data ) = 0;
  virtual void decode ( LBuffer & data ) = 0;
};

/*!
 * \class LCryptAsymModule
 * \brief A base class for asymmetric cryptography modules
 * \ingroup Cryptography
 */

class LCL_EXPORT LCryptAsymModule : public LCryptModule {
  LVCS_DECLARE;
 protected:
  //! the public key
  LCryptPublicKey publicKey;
  //! the private key
  LCryptPrivateKey privateKey;

 public:
  LCryptAsymModule ( LCryptPublicKey const & _key );
  LCryptAsymModule ( LCryptPrivateKey const & _key );
};

/*!
 * \class LCryptSymModule
 * \brief A base class for symmetric cryptography modules
 * \ingroup Cryptography
 */

class LCL_EXPORT LCryptSymModule : public LCryptModule {
  LVCS_DECLARE;
 protected:
  //! the secret key
  LCryptSecretKey key;

 public:
  LCryptSymModule ( LCryptSecretKey const & _key );
};

#endif
