#ifndef _LCRYPT_XOR_H_
#define _LCRYPT_XOR_H_

#include <lcl/lcrypt.h>

#include <lcl/lvcs.h>

/*!
 * \class LCryptModule_XOR
 * \brief A XOR crypto module
 * \author Jens Reimann
 * \ingroup Cryptography
 *
 * This crypto module crypts using a very simple XOR algorithm.
 *
 * It will combine the data and the key using XOR. En/de-crypting it twice will give you the
 * original data.
 *
 * m_e = m_d xor key
 *
 * but ( m_d xor key ) xor key = m_d
 */


class LCL_EXPORT LCryptModule_XOR : public LCryptSymModule {
  LVCS_DECLARE;
 protected:

  //! does the actual en/de-cryption
  void apply ( LBuffer & data );

 public:
  LCryptModule_XOR ( LCryptSecretKey const & _key );

  //! encode data (calls apply)
  virtual void encode ( LBuffer & data );

  //! decode data (calls apply)
  virtual void decode ( LBuffer & data );
};

#endif
