#ifndef _LFD_LAYER_POSIX_H_
#define _LFD_LAYER_POSIX_H_

#include <lcl.h>
#include <lcl/lvcs.h>
#include <lcl/lxcpt.h>
#include <lcl/lset.h>
#include <lcl/lmutex.h>

class LCL_EXPORT LFDManager {
  LVCS_DECLARE;
 protected:

  static LFDManager * theInstance;   // Common
  static LMutex instanceLock;
  LFDManager ();

 public:

  enum OpenFlags {  read = 0x01, write = 0x02, append = 0x4, create = 0x08, failExists = 0x10, truncate = 0x20 };
  enum Permissions { 
    user_r = S_IRUSR, user_w = S_IWUSR, user_x = S_IXUSR, user_rwx = (user_r|user_w|user_x),
    group_r = S_IRGRP, group_w = S_IWGRP, group_x = S_IXGRP, group_rwx = (group_r|group_w|group_x),
    other_r = S_IROTH, other_w = S_IWOTH, other_x = S_IXOTH, other_rwx = (other_r|other_w|other_x),
    set_user = S_ISUID, set_group = S_ISGID,
    all_r = (user_r|group_r|other_r),
    all_w = (user_w|group_w|other_w),
    all_x = (user_x|group_x|other_x)
  };

  static LFDManager * getInstance ();  // Common

  bool is_valid ( LFileDescriptor fd ) const;
  bool is_socket ( LFileDescriptor fd ) const;

  int sys_select ( int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout );
  
  int sys_setsockopt ( LFileDescriptor fd, int level, int optname, void const * optval, unsigned long optlen );
  int sys_getsockopt ( LFileDescriptor fd, int level, int optname, void * optval, unsigned long *optlen );

  int sys_open ( char const * name, int of, luint32 om );

  int sys_socket ( int af, int type, int proto );
  int sys_bind ( LFileDescriptor fd, struct sockaddr const * addr, unsigned long addr_len );
  int sys_connect ( LFileDescriptor fd, struct sockaddr const * addr, unsigned long addr_len );
  int sys_listen ( LFileDescriptor fd, int backlog );
  LFileDescriptor sys_accept ( LFileDescriptor fd, struct sockaddr * addr, unsigned long * addr_len );
  int sys_getsockname ( LFileDescriptor fd, struct sockaddr *name, unsigned long *namelen );
  int sys_getpeername ( LFileDescriptor fd, struct sockaddr *name, unsigned long *namelen );

  int sys_sendto ( LFileDescriptor fd, void const * data, unsigned long len, int flags, struct sockaddr const *to, unsigned long tolen );
  int sys_recvfrom ( LFileDescriptor fd, void * data, unsigned long len, int flags, struct sockaddr *from, unsigned long * fromlen );

  LFileDescriptor sys_dup ( LFileDescriptor fd );
  int sys_read ( LFileDescriptor fd, void * ptr, unsigned long size );
  int sys_write ( LFileDescriptor fd, void const * ptr, unsigned long size );
  int sys_close ( LFileDescriptor fd );

  bool sys_getblocking ( LFileDescriptor fd );
  void sys_setblocking ( LFileDescriptor fd, bool state );

  bool sys_select ( LSet<LFileDescriptor> const & read,
		    LSet<LFileDescriptor> const & write,
		    LSet<LFileDescriptor> const & xcpt,
		    LSet<LFileDescriptor> & read_res,
		    LSet<LFileDescriptor> & write_res,
		    LSet<LFileDescriptor> & xcpt_res,
		    unsigned long msecs );
};

#endif
