/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#ifndef _LFIFO_H_
#define _LFIFO_H_

template <class Type>
class LFIFO;

#include <lcl/lmutex.h>

/*!
 * \class LFIFO
 * \author Jens Reimann <ctron@dentrassi.de>
 * \brief a FIFO (first in first out) implementation
 *
 * This is a simple implementation of a FIFO
 */

template <class Type>
class LCL_EXPORT LFIFO {
 protected:
  Type * array;
  long size, nums, head, tail;

  LMutex mutex;

 public:
  LFIFO ( long num );           //!< Create a new instance with num elements
  LFIFO ( LFIFO const & );      //!< Copy a FIFO from an existing one

  ~LFIFO ();

  void clear ();                //!< Clear the FIFO

  bool add ( Type const & );    //!< Add an element to the FIFO
  long entries () const;        //!< Query the number of elements
  long spaceLeft () const;      //!< Query the number of free slots
  Type fetch ();                //!< Fetch an element from the FIFO

};

template <class Type>
LFIFO<Type>::LFIFO ( long _size )
{
  size = _size;
  nums = 0;
  tail = head = 0;
  

  array = new Type[size];
}

template <class Type>
LFIFO<Type>::LFIFO ( LFIFO const & p1 )
{
  size = p1.size;
  nums = p1.nums;
  head = p1.head;
  tail = p1.tail;

  array = new Type[size];
  *array = *p1.array;
}

template <class Type>
LFIFO<Type>::~LFIFO ()
{
  delete [size] array;
}

template <class Type>
void LFIFO<Type>::clear ()
{
  mutex.lock ();

  head = tail = 0;  
  nums = 0;

  mutex.unlock ();
}

template <class Type>
bool LFIFO<Type>::add ( Type const & item )
{

  mutex.lock ();

  if ( entries() >= size ) {
    mutex.unlock ();
    return false;
  }

  array[head] = item;
  head++;

  if ( head >= size ) {
    head = 0;
  }

  nums++;

  mutex.unlock ();
  return true;
}

template <class Type>
long LFIFO<Type>::entries () const
{
  return nums;
}

template <class Type>
long LFIFO<Type>::spaceLeft () const
{
  return size-nums;
}

template <class Type>
Type LFIFO<Type>::fetch ()
{
  mutex.lock ();
  long my_pos = tail;

  if ( entries() == 0 ) {
    mutex.unlock ();
    return Type();
  }

  tail++;
  if ( tail >= size ) {
    tail = 0;
  }

  nums--;

  Type tmp = array[my_pos];

  mutex.unlock ();
  return tmp;
}

#endif
