#ifndef _LFILEWATCHER_H_
#define _LFILEWATCHER_H_

#include <lcl.h>

#include <lstring.h>
#include <lemit.h>
#include <lmap.h>
#include <lmutex.h>
#include <lthread.h>

/*!
 * \defgroup Filesystem Filesystem
 */

/*!
 * \class LFileWatcher
 * \brief A file status watcher
 * \author Jens Reimann <reim@pe-muc.de>
 * \date 2002-03-20
 * \ingroup Filesystem
 *
 */

class LCL_EXPORT LFileWatcher {
protected:
  class FileInfo;

  friend LCL_EXPORT LString key ( LFileWatcher::FileInfo const & );
 protected:

  LMethodThreadFn<LFileWatcher> runnerFn;

  class FileInfo {
  public:

    bool checked;

    off_t last_size;
    time_t last_mtime, last_atime, last_ctime;
    bool last_valid;

    LString fileName;
  };

  LMap<FileInfo, LString> fileList;
  LMutex lock;

  void checkFile ( FileInfo & fi );

 public:

  enum EventType { size, atime, mtime, ctime, valid };

  LFileWatcher ();

  void checkOnce ( void );         //!< Run the check once
  void runner ( void );            //!< Run a neverending loop which does the checks

  void addFile ( LString const & ); //!< Add a new file to be checked
  void removeFile ( LString const & ); //!< Remove a file from the listed of been checked

  LEmitter2 < LString const &, EventType > changed; //!< Notifier for change events
  LEmitter1 < LString const & > changedStatus;      //!< Notifier for status change events
  LEmitter1 < LString const & > changedSize;        //!< Notifier for size change events
  LEmitter1 < LString const & > changedMTime;       //!< Notifier for modification timestamp change events
 
  void start ( void );   //!< Starts a new thread which does all the checking automatically
};

LCL_EXPORT LString key ( LFileWatcher::FileInfo const & );

#endif
