/* ****************************************************************************
   |      File: literator.h
   |      Contains: base class definition for LIterator
   |      Created: 03.09.1998
   |      Last Changed: 03.09.1998
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |
   ************************************************************************* */

#ifndef _L_ITERATOR_H_
#define _L_ITERATOR_H_


/*!
 * \class LIterator
 * \brief Generic iterator class
 * \author Jens Reimann <ctron@dentrassi.de>
 * \version 0.1
 * \date 2000
 * \bug Should be more or less bug free.
 *
 * This is a generic iterator class which may be used by many other classes which allow
 * iteration through a set of data.
 *
 * This iterator is used by the list classes LHeap, LSet and LMap. They will iterate automatically through
 * all elements in the list.
 *
 * If you want to iterate through the list step by step you should use cursors (LCursor).
 *
 * To create an iterator simply derive your iterator from LIterator and define the method applyTo which will
 * be called for each item.
 */

template <class Type>
class LCL_EXPORT LIterator {
 protected:
 public:
  virtual void applyTo ( Type & item ) = 0;        //!< Method which will be called
};

/**
 * \class LConstIterator
 * \brief Generic const iterator class
 * \author Jens Reimann <ctron@dentrassi.de>
 * \version 0.1
 * \date 2000
 * \bug Should be more or less bug free.
 *
 * This is like the LIterator class but allowes to use const objects.
 *
 * For more information of iterators see LIterator
 */

template <class Type>
class LCL_EXPORT LConstIterator {
 protected:
 public:
  virtual void applyTo ( Type const & item ) const = 0;  //!< Method which will be called
};


#endif
