#ifndef _LLIBRARY_H_
#define _LLIBRARY_H_

#include <lcl.h>

#include <lcl/lvcs.h>
#include <lcl/lstring.h>
#include <lcl/lobject.h>

/*!
 * \class LLibrary
 * \brief A simple dynamic shared library wrapper
 * \author Jens Reimann <ctron@dentrassi.de>
 * \bugs Using default argument on UNIX
 * \ingroup PlugIns
 * 
 * This class loads a shared library and fetches symbol addresses from it.
 *
 * It is mainly used by LGenericPlugIn but may be used for anything else.
 */

class LCL_EXPORT LLibrary : public LObject {
  LVCS_DECLARE;
 protected:
  LLibraryHandle handle;

 public:
  LLibrary ();                                //!< Dummy default constructor
  LLibrary ( LString const & file );          //!< Constructor which loads the library
  ~LLibrary ();

  bool isValid() const;                       //!< Check if the library is valid
  void * getProc ( LString const & name );    //!< Get a proc address from the library

  void load ( LString const & );              //!< Load a library
  void unload ( void );                       //!< Unload a library
};

#endif
