#ifndef _LLOGHANDLER_H_
#define _LLOGHANDLER_H_

#include <lcl.h>
#include <lcl/lstring.h>
#include <lcl/liostream.h>
#include <lcl/lstreamproxy.h>

typedef struct {
  unsigned long pid;
  unsigned long time;
  long pri;
  LString module;  
  LString message;
} LLogEvent;

class LLogHandler;

#include <lcl/llogsystem.h>

class LCL_EXPORT LLogHandler {
 protected:
  LString name;
  bool active;

  void setName ( LString const & );

 public:

  LLogHandler ();
  virtual void logEvent ( LLogSystem &, LLogEvent & ) = 0;

  LString getName () const;
  bool getActive () const;
  void setActive ( bool = true );
};

#ifndef PURE_WIN32
class LCL_EXPORT LLogHandler_Syslog : public LLogHandler {
 protected:
  static bool initstate;

 public:

  LLogHandler_Syslog ();
  virtual ~LLogHandler_Syslog ();

  virtual void logEvent ( LLogSystem &, LLogEvent & );

};
#endif

class LCL_EXPORT LLogHandler_Console : public LLogHandler {
 protected:
  LOStream & out;
  long level;

 public:
  LLogHandler_Console ( long _level = 0x7FFFFFFF, LOStream & = lout );
  virtual ~LLogHandler_Console ();

  virtual void logEvent ( LLogSystem &, LLogEvent & );
};

#ifdef _WIN32

#include <windows.h>

class LCL_EXPORT LLogHandler_WinNT : public LLogHandler {  
 protected:
  LString server;
  HANDLE handle;

 public:
  LLogHandler_WinNT ( LString const & server = "" );  
  virtual ~LLogHandler_WinNT ();

  virtual void logEvent ( LLogSystem &, LLogEvent & );
};
#endif

#endif
