#ifndef _LLOGSYSTEM_H_
#define _LLOGSYSTEM_H_

#include <lcl.h>
#include <lcl/lvcs.h>
#include <lcl/lstring.h>
#include <lcl/lset.h>
#include <lcl/lmutex.h>

class LCL_EXPORT LLogSystem;

#include <lcl/lloghandler.h>

/*!
 * \def LLOG(p,s)
 * \relates LLogSystem
 * 
 * Send out a log event of priority \a p and with message \a s .
 *
 * \param p The priority of the log event (one of BULK, INFO, ...)
 * \param s The log message itself
 */
#define LLOG(p,s) LLogSystem::getInstance()->log ( (p), (s) );


/*!
 * \defgroup Logging Logging
 */


/*!
 * \class LLogSystem
 * \author Jens Reimann <ctron@dentrassi.de>
 * \brief The Log System Management class
 * \ingroup Logging
 * \relates LLOG
 *
 * This class manages the logging system of LCL.
 *
 * The log system is normally activated by the method LApplication::processLCLParameters in each application.
 *
 * Also refer to page \ref lcl_log_page_1 and the macro LLOG() for more information.
 */
class LCL_EXPORT LLogSystem {
  LVCS_DECLARE;

 protected:
  LLogSystem ();
  static LLogSystem * theInstance;

  LString module;
  LSet<LLogHandler*> handlerSet;
  LMutex syncLock;

 public:

  enum LogPriority { 
    BULK = 20, //<! A bulk message (lowest priority)
    INFO = 3, //<! An informational message
    WARNING = 2, //<! An event occured which is important but will not cause the application to malfunction
    ERR = 1, //<! An event occured which might (but not for sure) cause the application to malfunction or will cause a malfunction which can be recovered
    ERRO = 1,
    FATAL = 0 //<! An event occured which will cause the application to malfunction
  };

  static void createLogHandler ( LString const & );
  static LLogSystem * getInstance ( );

  void addLogHandler ( LLogHandler * );
  void removeLogHandler ( LLogHandler * );

  void setModule ( LString const & );
  LString getModule ( ) const;

  void log ( LogPriority, LString const & event );

  void dump ();
};

#endif
