
#ifndef _LPLUGIN_LOADER_H_
#define _LPLUGIN_LOADER_H_

#include <lcl/lstringtokenizer.h>
#include <lcl/ldirectory.h>

/*!
 * \class LPlugInLoader
 * \brief Virtual base class PlugIn Loader Factory
 */

template < class PlugIn > 
class LPlugInLoader {
 protected:

  void scanPlugInPath ( LString const & path );

 public:
  LPlugInLoader ();

  virtual void foundPlugIn ( PlugIn * plugin ) = 0;

  void scan ();
};

template < class PlugIn >
LPlugInLoader<PlugIn>::LPlugInLoader ()
{
}

template < class PlugIn >
void LPlugInLoader<PlugIn>::scanPlugInPath ( LString const & path )
{
  LTRACE ( "LPlugInLoader", "Scanning: " + path );

  LHeap<LString> list = LDirectory::getFileList( path );
  LManagedPointer<LCursor> cursor = list.createCursor();

  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() )
    {
      LTRACE ( "LPlugInLoader", "Loading PlugIn " + path + "/" + list.elementAt ( cursor ) );
      PlugIn * plugin = new PlugIn ( path + "/" + list.elementAt ( cursor ) );
      if ( plugin->isValid() )
	{
	  LTRACE ( "LPlugInLoader", "Loaded PlugIn " + plugin->getName() + " -> " + plugin->getDescription() );
	  foundPlugIn ( plugin );
	}
      else
	{
	  LTRACE ( "LPlugInLoader", "PlugIn " + path + "/" + list.elementAt ( cursor ) + " is not a valid plug-in or not one of the selected type" );
	  delete plugin;
	}
    }
}

template < class PlugIn >
void LPlugInLoader<PlugIn>::scan ()
{
  LString envPath = LString::fromEnv ( "LCL_PLUGIN_DIR" );
  LString defPath = "/usr/share/lcl:/usr/local/share/lcl:~/lcl";

  LString totalPath = envPath + ":" + defPath;

  LStringTokenizer tok ( ":" );
  tok.tokenize ( totalPath );

  for ( int i = 0; i<tok.numWords(); i++ )
    {
      scanPlugInPath ( tok.word(i) );
    }
}

#endif

