/* ****************************************************************************
   |      File: lpointer.h
   |      Contains: class definition for LPointer, ...
   |      Created: 31.08.1998
   |      Last Changed: 31.08.1998
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |
   ************************************************************************* */

#ifndef _L_POINTER_H_
#define _L_POINTER_H_

template <class Type> class LManagedPointer;

#include <lcl.h>
#include <lcl/lobject.h>
#include <lcl/lxcpt.h>

template <class Type>
class LCL_EXPORT LAutoPointer {
 protected:
  Type * ptr;

 public:
  LAutoPointer ()
    {
      ptr = NULL;
    }
  LAutoPointer ( Type * _ptr )
    {
      ptr = _ptr;
    }
  LAutoPointer ( LAutoPointer & p1 )
    {
      ptr = p1.ptr;
      p1.ptr = NULL;
    }
  ~LAutoPointer ( )
    {
      delete ptr;
    }
  LAutoPointer & operator = ( Type * _ptr )
    {
      if ( ptr )
	delete ptr;
      ptr = _ptr;
      return *this;
    }
  Type const & operator = ( Type const & value )
    {
      *ptr = value;
      return *ptr;
    }
  bool operator == ( LAutoPointer const & p1 ) const
    {
      return ptr == p1.ptr;
    }
  bool isValid () const
    {
      return ptr != NULL;
    }
  Type & operator * () const
    {
      return *ptr;
    }
  Type * operator-> () const 
    {
      return ptr;
    }
  operator Type * () const
    {
      return ptr;
    }
  operator Type & () const
    {
      return *ptr;
    }  
  operator Type const & () const
    {
      return *ptr;
    }
};

template <class Type>
class LCL_EXPORT LManagedPointer {
 protected:
  Type * ptr;
  unsigned long * numRef;
  bool dontDestroy;

 public:
  LManagedPointer ( ) :
    ptr ( NULL ),
    numRef ( NULL ),
    dontDestroy ( false )
    {
    }

  LManagedPointer ( Type * _ptr ) :
    ptr ( NULL ),
    numRef ( NULL ),
    dontDestroy ( false )
    {
      attach ( _ptr );
    }

  LManagedPointer ( LManagedPointer const & p1 ) :
    ptr ( NULL ),
    numRef ( NULL ),
    dontDestroy ( false )
    {
      attach ( p1 );
    }

  ~LManagedPointer ()
    {
      detach ();
    }

  void attach ( LManagedPointer const & p1 )
    {
      //LTRACE_FUNC ( "LManagedPointer", "attach (LManagedPointer const &)" );

      detach ();

      dontDestroy = p1.dontDestroy;
      numRef = p1.numRef;
      if ( numRef )
	(*numRef)++;
      ptr = p1.ptr;

      if ( (numRef == NULL) && (ptr!=NULL) )
	{
	  NAMESPACE_STD(cerr) << "LManagedPointer: WARNING!! Invalid reference counter but valid data pointer!" << NAMESPACE_STD(endl) << NAMESPACE_STD(flush);
	}
    }

  void attach ( Type * _ptr )
    {
      detach ();

      numRef = new unsigned long int;
      (*numRef) = 1;
      ptr = _ptr;
    }

  void detach ()
    {

      if ( numRef ) {
	//LTRACE ( "LManagedPointer", "Detaching. " + LString((*numRef)-1) + " references left!" );

	if ( (*numRef) )
	  {
	    (*numRef)--;
	  }
	else
	  {
	    NAMESPACE_STD(cerr) << "LManagedPointer: WARNING!! numRef would go below zero!" << NAMESPACE_STD(endl) << NAMESPACE_STD(flush);
	  }

	if ( (*numRef) == 0 ) {
	  //LTRACE ( "LManagedPointer", "Destroying object!" );
	  delete numRef;

	  if ( !dontDestroy )
	    delete ptr;

	}
	ptr = NULL;
	numRef = NULL;
      }
    }

  LManagedPointer & operator = ( Type * _ptr )
    {
      //      detach ();
      attach ( _ptr );
      return *this;
    }

  LManagedPointer & operator = ( LManagedPointer const & p1 )
    {
      //LTRACE_FUNC ( "LManagedPointer", "operator= (LManagedPointer const &)" );
      //      detach ();
      attach ( p1 );

      /*
      if ( p1.numRef ) {
	attach ( p1 );
      } else {
	numRef = NULL;
	ptr = NULL;
      }
      */

      return *this;
    }

  bool operator == ( LManagedPointer const & p1 ) const
    {
      return ptr == p1.ptr;
    }

  Type const & getConst() const
    {
      if (!ptr)	throw LNullPointerException ( __FILE__, __LINE__ );
      return *ptr;
    }

  bool isValid () const
    {
      return ptr != NULL;
    }
  Type & operator * ()
    {
      if (!ptr)	throw LNullPointerException ( __FILE__, __LINE__ );
      return *ptr;
    }
  Type * operator-> () const 
    {
      if ( ptr == NULL ) {
	NAMESPACE_STD(cerr) << "Requesting for NULL pointer!! Expect segfault ... (" << __FILE__ << ":" << __LINE__ << ")\n" << NAMESPACE_STD(endl) << NAMESPACE_STD(flush);
	throw LNullPointerException ( __FILE__, __LINE__ );
      }
      return ptr;
    }

  operator Type * () const
    {
      return ptr;
    }

  Type * getPtr ( void )
    {
      return ptr;
    }

  LManagedPointer & disableDestroy ()
    {
      dontDestroy = true;
      return *this;
    }
};

//! A special hash function for LManagedPointer
/*
template < class Type >
LCL_EXPORT unsigned long hash ( LManagedPointer<Type> const & item, unsigned long hash_size )
{
  void const * ptr = item.getConstPtr();
  return hash ( &ptr, sizeof(ptr), hash_size );
}

template < class Type >
LCL_EXPORT unsigned long hash ( LManagedPointer<Type> & item, unsigned long hash_size )
{
  void const * ptr = item.getConstPtr();
  return hash ( &ptr, sizeof(ptr), hash_size );
}

template < class Type >
LCL_EXPORT unsigned long hash ( LManagedPointer<Type> item, unsigned long hash_size )
{
  void const * ptr = item.getConstPtr();
  return hash ( &ptr, sizeof(ptr), hash_size );
}
*/

template <class Type>
class LCL_EXPORT LElementPointer {
  Type * ptr;
 public:
  LElementPointer ( )
    {
      ptr = NULL;
    }
  LElementPointer ( Type * _ptr )
    {
      ptr = _ptr;
    }
  LElementPointer ( LElementPointer const & p1 )
    {
      ptr = p1.ptr;
    }
  bool operator == ( Type const & p1 )
    {
      return p1 == *ptr;
    }
  Type * operator = ( Type * _ptr )
    {
      ptr = _ptr;
      return ptr;
    }
  Type & operator * ()
    {
      return *p1;
    }
  operator Type ()
    {
      return *p1;
    }
};

template <class Type>
class LCL_EXPORT LElementManagedPointer : public LManagedPointer<Type> { 
 public:

  LElementManagedPointer ()
    {}

  LElementManagedPointer ( Type * _ptr ) :
    LManagedPointer<Type> ( _ptr )
    {
    }

  bool operator == ( Type const & p1 )
    {
      return p1 == *ptr;
    }
  Type & operator * ()
    {
      return LManagedPointer<Type>::operator*();
    }
};

#endif
