#ifndef _LCL_QUEUE_BASE_H_
#define _LCL_QUEUE_BASE_H_

#include <lcl/lqueue.h>
#include <lcl/lheap.h>
#include <lcl/llogsystem.h>

template < class Type >
class LQueueReader : public LQueueReceiver<Type> {
 protected:
 public:
  virtual Type read ( void ) = 0;
};

template < class Type >
class LQueueReaderStatic : public LQueueReader<Type> {
 protected:
  LFIFO<Type> m_storage;
  LSemaphore m_sem;
  virtual void injector ( Type const & );
 public:
  LQueueReaderStatic ( luint32 = 1024 );
  Type read ( void );
};

template < class Type >
class LQueueReaderDynamic : public LQueueReader<Type> {
 protected:
  LGuardMutex storageLock;
  LHeap<Type> m_storage;
  LSemaphore m_sem;
  virtual void injector ( Type const & );
 public:
  LQueueReaderDynamic ( );
  Type read ( void );
};

template < class Type >
class LQueueSimpleMux : public LQueueMux<Type> {
 protected:
  virtual void mux ( Type const & );
 public:
};

// LQueueReaderStatic<Type>
template < class Type >
LQueueReaderStatic<Type>::LQueueReaderStatic ( luint32 size ) : 
  m_storage ( size ),
  m_sem ( 0 )
{
}

template < class Type >
void LQueueReaderStatic<Type>::injector ( Type const & item )
{
  LTRACE ( "LQueueReaderStatic<Type>", "injector called" );
  if ( !m_storage.add ( item ) )
    {
      throw LQueueFullException(__FILE__, __LINE__);
      return;
    }
  m_sem.post ();
}

template < class Type >
Type LQueueReaderStatic<Type>::read ( void )
{
  m_sem.wait ( );
  return m_storage.fetch ();
}

// LQueueReaderDynamic<Type>
template < class Type >
LQueueReaderDynamic<Type>::LQueueReaderDynamic ( ) : 
  m_sem ( 0 )
{
}

template < class Type >
void LQueueReaderDynamic<Type>::injector ( Type const & item )
{
  LTRACE ( "LQueueReaderDynamic<Type>", "injector called" );

  storageLock.lock ();
  m_storage.add ( item );
  storageLock.unlock ();

  m_sem.post ();
}

template < class Type >
Type LQueueReaderDynamic<Type>::read ( void )
{
  m_sem.wait ( );

  storageLock.lock ();
  LManagedPointer<LCursor> cursor = m_storage.createCursor();
  cursor->setToLast();
  Type item = m_storage.elementAt ( cursor );
  m_storage.removeElementAt ( cursor );
  storageLock.unlock ();  

  return item;
}


// LQueueSimpleMux<Type>
template < class Type >
void LQueueSimpleMux<Type>::mux ( Type const & item )
{
  LManagedPointer<LCursor> cursor = m_receiverList.createCursor ();
  forcursor ( cursor )
    {
      if ( m_receiverList.elementAt ( cursor ) != NULL )
	{
	  try {
	    injectTo ( *m_receiverList.elementAt ( cursor ), item );
	  }
	  catch ( LQueueFullException & e )
	    {
	      LLOG ( LLogSystem::ERR, "LQueueSimpleMutex -> Queue is full! Skipped!" );
	    }
	}
    }
}

#endif
