/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

/* LCL - The Linux Class Library
 * Copyright (C) 1998-2002 Jens Reimann
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _LCL_RECT_H_
#define _LCL_RECT_H_

#include <lcl.h>

#include <lcl/lpoint.h>
#include <lcl/lxcpt.h>

class LCL_EXPORT LRect {
public:

  class LCL_EXPORT NoIntersection;

protected:
  lint32 m_x, m_y;
  luint32 m_width, m_height;

  LPoint leftUpIntersection ( LRect const & p1 ) const;
  LPoint rightDownIntersection ( LRect const & p1 ) const;

public:
  LRect ();
  LRect ( LRect const & p1 );
  LRect ( lint32 x, lint32 y, lint32 width, lint32 height );
  LRect ( LPoint const & leftUp, LPoint const & rightDown );
  
  virtual ~LRect ();

  lint32 x () const;
  lint32 y () const;
  void x ( lint32 x );
  void y ( lint32 y );  
  luint32 width () const;
  luint32 height () const;
  void width ( lint32 width );
  void height ( lint32 height );
  lint32 x2 () const;
  lint32 y2 () const;
  
  void setLeftUp ( lint32 x, lint32 y );
  void setRightDown ( lint32 x, lint32 y );
  void setLeftUp ( LPoint const & leftUp );
  void setRightDown ( LPoint const & rightDown );

  void set ( lint32 x, lint32 y, lint32 width, lint32 height );
  void set ( LPoint const & leftUp, LPoint const & rightDown );
  void set ( LRect const & p1 );

  LPoint leftUp ( ) const;
  LPoint rightDown ( ) const;

  LRect & operator = ( LRect const & p1 );
  bool operator == ( LRect const & p1 ) const;

  bool contains ( LPoint const & p1 ) const;
  bool inside ( LPoint const & p1 ) const;
  bool outside ( LPoint const & p1 ) const;

  bool contains ( LRect const & p1 ) const;
  bool inside ( LRect const & p1 ) const;
  bool outside ( LRect const & p1 ) const;

  bool intersects ( LRect const & p1 ) const;
  LRect intersection ( LRect const & p1 ) const;
};


class LCL_EXPORT LRect::NoIntersection : public LBasicException {
protected:
  LRect m_r1, m_r2;
public:
  NoIntersection ( char const * file, luint32 line, LRect const & r1, LRect const & r2 );
  virtual void log ( void );
};


#endif
