#ifndef _LSIMPLELIST_H_
#define _LSIMPLELIST_H_

#include <lcl.h>

/*!
 * \class LSimpleList
 * \author Jens Reimann <ctron@dentrassi.de>
 * \brief A simple doubly linked list
 * \ingroup Collections
 *
 * This is a simple implementation of a linked list
 *
 * The list may be "locked" using the methods LSimpleList::lock and LSimpleList::unlock. When the list is locked entries are not
 * really deleted but marked as deleted. An iterator will then skip these entries autmatically but the containeres holding these values
 * will still remain in the queue.
 *
 * When the method LSimpleList::cleanUp is called (it is implicitly called by LSimpleList::unlock) all marked elements and containers will
 * be \em deleted from the list.
 *
 * \warning The value NULL is not allowed as a item in the list
 *
 * \note This list should only be used in special cases where LSet, LHeap, LMap or LHash cannot be used!
 *
 */
class LCL_EXPORT LSimpleList {
 protected:
  
  /*!
   * A container for a LSimpleList
   */
  class Container {
    friend class LSimpleList;
  protected:
    Container * next, *prev;
    void * ptr;
  };

  Container *head, *tail;
  bool locked;

 public:
  LSimpleList ();  //!< Contructs an empty linked list
  LSimpleList ( LSimpleList const & p1 ); //!< Copy constructor
  virtual ~LSimpleList ();

  void append ( void * );  //!< Append an item to the list
  void remove ( void * );  //!< Remove an item from the list

  void clear ();  //!< Clear the list
  void cleanUp ();

  void lock (); //!< Lock the list
  void unlock (); //!< Unlock the list
  
  void forEach ( void (*func)(void*) ); //!< Iterator for LSimpleList
};

#endif
