#ifndef _LSTACK_H_
#define _LSTACK_H_

#include <lcl.h>

/*!
 * \brief A simple templated stack implementation
 */

template <class Type>
class LCL_EXPORT LStack {
  Type * array;
  long capacity;
  long pos;

 public:
  LStack ( long _capacity );
  LStack ( LStack const & );
  ~LStack ();
  
  void push ( Type const & );
  Type pop ();
  Type peek () const;
  Type & peek ();
  bool isEmpty () const;
  bool isFull () const;
  long entries () const;
  void clear ();
  
};

template <class Type>
LStack<Type>::LStack ( long _capacity ) :
  array ( NULL ),
  capacity ( 0 ),
  pos ( -1 )
{
  capacity = _capacity;
  array = new Type [capacity];  
}

template <class Type>
LStack<Type>::~LStack ( )
{
  delete [] array;
}

template <class Type>
LStack<Type>::LStack ( LStack const & p1 ) :
  array ( NULL ),
  capacity ( 0 ),
  pos ( p1.pos )
{
  capacity = p1.capacity;
  array = new Type [capacity];  
  *array = *p1.array;
}
  
template <class Type>
void LStack<Type>::push ( Type const & element )
{
  if ( isFull () )
    throw "Stack is full!";  
  
  array[++pos] = element;
}

template <class Type>
Type LStack<Type>::pop ()
{
  if ( isEmpty() )
    throw "Stack is empty!";

  return array [ pos-- ];
}

template <class Type>
Type LStack<Type>::peek () const
{
  if ( isEmpty() )
    throw "Stack is empty!";

  return array [ pos ];
}

template <class Type>
Type & LStack<Type>::peek ()
{
  if ( isEmpty() )
    throw "Stack is empty!";

  return array [ pos ];
}

template <class Type>
bool LStack<Type>::isEmpty () const
{
  return pos == -1;
}

template <class Type>
bool LStack<Type>::isFull () const
{
  return pos == capacity - 1;
}

template <class Type>
void LStack<Type>::clear ()
{
  pos = -1;
}

template <class Type>
long LStack<Type>::entries () const
{
  return pos;
}

#endif
