/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

/* LCL - The Linux Class Library
 * Copyright (C) 1998-2002 Jens Reimann
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _LSTRING_TOKENIZER_H_
#define _LSTRING_TOKENIZER_H_

#include <lcl.h>
#include <lcl/lstring.h>
#include <lcl/lheap.h>

/**
 * \class LStringTokenizer
 * \brief Tokenizer class for LString strings
 * \author Jens Reimann <ctron@dentrassi.de>
 * \version 0.1
 * \date 2000-06-19
 * \bug Escaping is missing and not possible with the current internal data handling
 *
 * This class is a string tokenizer which splits up a string into tokens. The object will hold the
 * string and the \e cut-marks but not the split up strings.
 * 
 *
 * Remeber: Words start with #0
 */

class LCL_EXPORT LStringTokenizer {

 protected:

  class LCL_EXPORT Token {
    friend class LStringTokenizer;
  protected:
    long num;
    long start, len;
    LString * str;
  public:
    Token ();
    Token ( LString * str, long num, long start, long len );
    Token ( Token const & );
    bool isValid () const;
  };

  long words;                  //!< Keeps the number of words
  LHeap<Token> tokenList;      //!< Keeps the token info
  LString delims, qmarks;      
  LString str;                 //!< Keeps the tokenized string

  Token getTokenInfo ( long num ) const;

 public:
  //! Creates a new instance of this class
  LStringTokenizer ( LString const & delims = " \t\n", LString const & qmarks = "\"" );

  void clear ();                            //!< Clears the tokenizer
  void tokenize ( LString const & );        //!< Tokenize a string
  long numWords () const;                   //!< Returns to number of tokens
  LString word ( long ) const;              //!< Returns a token
  bool containsWord ( LString const & word, bool caseIns = false ) const;
  LString rightOfWord ( long ) const;       //!< Return everything after a word

  LHeap<LString> asHeap () const; //!< Returns a heap of words
};

#endif
