/* ****************************************************************************
   |      File: ltable.h
   |      Contains: class definition for LTable
   |      Created: 08.09.1998
   |      Last Changed: 20.07.1999
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |
   ************************************************************************* */

#ifndef _L_TABLE_H_
#define _L_TABLE_H_

#include <lcl.h>

template < class Type >
class LCL_EXPORT LTable /*: public LObject*/ {
  protected:
    Type * array;
    unsigned long size;

  private:
    void alloc ( unsigned long _size );
    void copy ( LTable const & p1 );
    void free ();

  public:
    LTable ();
    LTable ( unsigned long table_size );
    LTable ( LTable const & p1 );
    ~LTable ();

    void clear ();
    bool valid () const;

    Type & operator[] ( unsigned long i ) const;

    LTable & operator = ( LTable const & p1 );

    unsigned long getSize () const;
};

#include <lcl/lstring.h>
#include <lcl/lxcpt.h>

template <class Type>
inline void LTable<Type>::alloc ( unsigned long _size )
{
  if ( _size == 0 )
    {
      size = 0;
      array = NULL;
    }
  else
    {
      size = _size;
      array = new Type[size];
    }
}

template <class Type>
inline void LTable<Type>::copy ( LTable const & p1 )
{
  if ( p1.size == 0 )
    {
      size = 0;
      array = NULL;
    }
  else
    {
      unsigned long i, len;
      
      len = (p1.size<size) ? p1.size : size;
      
      for ( i=0; i<len; i++) {
	array[i] = p1.array[i];
      }
    }
}

template <class Type>
inline void LTable<Type>::free ()
{
  if ( array != NULL )
    {
      delete [] array;
      array = NULL;
    }

}

template <class Type>
inline bool LTable<Type>::valid () const
{
  return array != NULL;
}


template <class Type>
inline LTable<Type>::LTable () :
  array ( NULL ),
  size (0)
{
}

template <class Type>
inline LTable<Type>::LTable ( unsigned long table_size ) :
  size ( table_size )
{
   alloc ( table_size );
}

template <class Type>
inline LTable<Type>::LTable ( LTable const & p1 )
{
  // allocate memory
  alloc ( p1.size );
  copy ( p1 );	
}

template <class Type>
inline LTable<Type>::~LTable ()
{
  clear ();
}

template <class Type>
inline void LTable<Type>::clear ()
{
  free ();
}

template <class Type>
Type & LTable<Type>::operator[] ( unsigned long i ) const
{
  if ( (i < size) && valid() ) {
    return array[i];
  } else {
    //throw LException("Out of range").set("Object","LTable").set("Funktion","operator[]");
    //    throw 1;
    throw LBasicException ( __FILE__, __LINE__, "Out of range: idx=" + LString(i) );
    MSTHROW(Type);
  }
}

template <class Type>
LTable<Type> & LTable<Type>::operator = ( LTable const & p1 )
{
  clear ();
  alloc ( p1.size );
  copy ( p1 );
	
  return *this;
}

template <class Type>
unsigned long LTable<Type>::getSize () const
{
  return size;
}


template < class Type, class Serializer >
Serializer & operator << ( Serializer & ser, LTable < Type > const & p1 )
{
  Serializer _ser;

  _ser << p1.getSize();

  for ( unsigned long i = 0; i<p1.getSize(); i++ )
    {
      _ser << p1[i];
    }

  ser << _ser;

  return ser;
}

template < class Type, class Serializer >
Serializer & operator >> ( Serializer & ser, LTable < Type > & p1  )
{

  Serializer _ser;
  ser >> _ser;

  unsigned long len;
  _ser >> len;

  LTable < Type > data ( len );

  for ( unsigned int i = 0; i<len; i++ )
    {
      _ser >> data[i];
    }

  p1 = data;

  return ser;
}


#endif

