/* -*-mode: c++; coding: iso-8859-2-unix; -*- */
/* ****************************************************************************
   |      File: lapp.cc
   |      Contains: class LApplication
   |      Created: 15.02.1998
   |      Last Changed: 13.03.1998
   |      Version: 0.0
   |         
   |      ToDo: + 
   |            + (global) add some real exceptions !!
   |
   ************************************************************************* */

/* ************************************************************************* *
 * Change log:                                                               *
 * -------------                                                             *
 *                                                                           *
 *   -> [v0.00] - 15.02.1998 - Created file and started coding               *
 *   -> [v0.00] - 07.03.1998 - Added destructor list                         *
 *   -> [v0.00] - 13.03.1998 - Added several destructor funcs                *
 *   -> [v0.01] - 13.03.1998 - !Version switch to 0.01                       *
 *   -> [v0.01] - 09.06.1998 - Starting to XCPTionify                        *
 *                                                                           *
 * ************************************************************************* */

#define _L_APP_CC_

#include <iostream>
#include <stdlib.h>

#include <lcl.h>
#include <lcl/lcl_config.h>

#include <lcl/lstring.h>
#include <lcl/lxcpt.h>
#include <lcl/lapp.h>
#include <lcl/ltrace.h>
#include <lcl/lstringtokenizer.h>
#include <lcl/llogsystem.h>
#include <lcl/ltracechannel.h>
#include <lcl/lthread.h>
#include <lcl/lrevision.h>
#include <lcl/lmemory.h>
#include <lcl/lregex.h>
#include <lcl/ltrace_output.h>
#include <lcl/ltrace_file.h>
#include <lcl/lcmdline.h>

#ifdef _LCL_COMPILER_MSVC_
LFEATURE("LCL", lcl_f1, "Built with MSVC" );
#endif
#ifdef _WIN32
LFEATURE("LCL", lcl_f2, "Built for Win32" );
#endif
#ifdef __NUTC__
LFEATURE("LCL", lcl_f3, "Built for NuTCRACKER" );
#endif
#ifdef __GNUC__
LFEATURE("LCL", lcl_f4, "Built with GCC (hooray!)" );
#endif
#ifdef _LCL_COMPILER_DECCXX_
LFEATURE("LCL", lcl_f5, "Built with shitty DEC CXX" );
#endif

/*
 * Command Line Arguments
 */

LCMDARG_LONG(lcl_trace,lcl-trace,true,"Control Tracing");
LCMDARG_LONG(lcl_log,lcl-log,true,"Control Logging");
LCMDARG_LONG(lcl_debug,lcl-debug,false,"Enable Debugging");
LCMDARG_LONG(lcl_version,lcl-version,false,"Show Version");
LCMDARG_LONG(lcl_count,lcl-count,false,"");
LCMDARG_LONG(lcl_help,lcl-help,false,"Show help");


/* ******************************************************************* *
 * *  Thread               <<--------------                          * *
 * ******************************************************************* */

LTRACE_CHANNEL_STATIC("LObjectDumper", __lcl_trace_objdump_1, "Object Instance Dumper" );

class LObjectDumper : public LThreadFn {
public:
  lint32 period;
  void run ( void )
  {
    LTRACE ( "LObjectDumper", "Started object counter dumper" );
    while ( true )
      {
	LTRACE ( "LObjectDumper", LString(LBaseObject::getInstCount()) + " LObjects active" );
	lmem_dump_stats ( lout );
	lcl_sleep ( period );
      }
  }
};

/* ******************************************************************* *
 * *  Static Members       <<--------------                          * *
 * ******************************************************************* */

NAMESPACE_STD(ostream) & LApplication::logStream = NAMESPACE_STD(cerr);
bool LApplication::debugFlag = false;

/* ******************************************************************* *
 * *  START FILE       <<--------------                              * *
 * ******************************************************************* */

LCL_EXPORT LApplication theApp;

#ifdef _LCL_SYS_WIN32_PURE_

bool lcl_win32_default_handler = false;

LONG WINAPI LCLexceptionHandler ( struct _EXCEPTION_POINTERS *ExceptionInfo  )
{
  lerr << " ======== LCL WIN32 EXCEPTION HANDLER ============ " << endl;
  lerr << "  Unhandled exception -> Dumping known stack!" << endl;
  LTraceFunc::dumpStack ( lerr );
  lerr << " ======== LCL WIN32 EXCEPTION HANDLER ============ " << endl;
  return lcl_win32_default_handler ? EXCEPTION_CONTINUE_SEARCH : EXCEPTION_EXECUTE_HANDLER;
}

#endif

LApplication::LApplication ()
{
#if ! ( defined(_LCL_COMPILER_MSVC_) || defined(_LCL_COMPILER_DECCXX_) )
  LTRACE_FUNC ("Application", "LApplication::LApplication ()" );
#endif

  callExit = false;
  isDestructing = false;

#ifdef _LCL_SYS_WIN32_PURE_
  SetUnhandledExceptionFilter ( LCLexceptionHandler );

  //  lcl_win32_default_handler = LString::fromEnv("LCL_DEFAULT_WIN32_FAULT").length();
  lcl_win32_default_handler = ::getenv("LCL_DEFAULT_WIN32_FAULT");
#endif

}


/*!
 * \example app.cc
 * Shows the basic usage of LApplication
 */

void LApplication::init ( LCommandLine const & cmdLine )
{
  m_cmdLine = cmdLine;
}

/*!
 * \param argc argc from your main
 * \param argv argv from your main
 *
 * The instance will be initialized based on the arguments you pass with argc and argv.
 */

void LApplication::init ( int argc, char * argv[] )
{

  /*
  // ******** Parameter parsing ********

  int i = 1;
  unsigned long n;
  LString key, value, str;
 
  argStr = "";

  // ********* Clean Programm Name *****
  LString tmp = argv[0];
  tmp.setDelims ( "\\/" );
  progName = tmp.word(tmp.numWords()-1);
  //  lout << "Programm name: " << progName << endl;

  // ********** Parse... ***************
  
  // Win32 does not always set argv[argc] to NULL!!!
  while ( (argv[i] != NULL) && (i<argc)) {
    str = argv[i];
    
    if (i!=1)
      argStr += " ";
    argStr += argv[i];

    if ( str[0] == '-' ) {

      str = str.subString(1);
      if ( str[0] == '-' ) {
	key = str.subString(1);
	paramDict.set ( key, LString() );
      } else {
	for (n=0; n<str.length(); n++) {
	  key = str[n];
	  paramDict.set ( key, LString() );
	}
      }

    } else {
      if ( key == "" ) {
	paramFiles.add ( str );
      } else {
	value = str;
	paramDict.set ( key, value );
	key = "";
      }
    }

    i++;
  }
  */
  m_cmdLine.init ( argc, argv );
}

void LApplication::initMyVersion ( LString const & version, LString const & date, LString const & host, long build_nr )
{
  myBuildVersion = version;
  myBuildDate = date;
  myBuildHost = host;
  myBuildNr = build_nr;
}

void LApplication::showLCLParametersSyntax ()
{
  lout << endl;
  lout << "  LCL LApplication Parameters:" << endl;
  lout << " ----------------------------------" << endl;
  lout << endl;
  lout << "   --lcl-trace [module1,module2,...,modulen]" << endl;
  lout << "        Enable trace for all modules provided in a" << endl;
  lout << "        comma-seperated list. If no module is specified" << endl;
  lout << "        enable all modules!" << endl;
  lout << endl;
  lout << "   --lcl-log logmodule1[:logmodule2[:logmodulen]]" << endl;
  lout << "        Installs the list of log-modules" << endl;
  lout << "        logmodule -> modulename[?parameter=value+param2=val2]" << endl;
  lout << "        available modules: console, syslog, winnt" << endl;
  lout << "          console's parameters:" << endl;
  lout << "             level : minimum level (BULK,INFO,WARNING,ERROR,FATAL)" << endl;
  lout << "              file : log to file. filename may contains env-vars" << endl;
  lout << "             e.g.: console?level=WARNING+file=$(PUBLIC)/logs/dummy.log" << endl;
  lout << endl;
  lout << "   --lcl-debug" << endl;
  lout << "        Show some debug output of application" << endl;
  lout << endl;
  lout << "   --lcl-version" << endl;
  lout << "        Shows the version number of the LCL used" << endl;
  lout << endl;
#ifdef _LCL_SYS_WIN32_
  lout << "   --lcl-win32-console" << endl;
  lout << "        Activates a Win32 Console Window of none is present for the" << endl;
  lout << "        application." << endl;
  lout << "        This is the case if the application is a WinMain application of" << endl;
  lout << "        the application is started in the background" << endl;
  lout << endl;
#endif
  lout << "   --lcl-count" << endl;
  lout << "        Starts the LObject instance counter dumper" << endl;
  lout << "        This will dump to trace channel LObjectDumper" << endl;
  lout << endl;
  lout << "   --lcl-trace-channels" << endl;
  lout << "        Shows all registers trace channels. There may be more." << endl;
  lout << "        But these are registered." << endl;
  lout << endl;
  lout << "   --lcl-help" << endl;
  lout << "        This information" << endl;
  lout << endl;
  lout << flush;

  LCommandLine::showHelp ( lout );
}

/*!
 * \pre This method requires that the command line arguments where parsed with the LApplication::init method before calling this.
 * \post All LCL specific command line arguments are processed and the needed actions where performed.
 * \note This method should be called directly after the LApplication::init method in your application. Also refer to the example!
 *
 * This method will process all LCL specific command line arguments.
 *
 * All LCL specific parameters start with --lcl so there should be no conflict with your application.
 *
 * The following paremters are currently LCL specific:
 *
 * - --lcl-trace
 * - --lcl-log
 * - --lcl-version
 * - --lcl-debug
 * - --lcl-help
 * - --lcl-count
 *
 */

void LApplication::processLCLParameters ()
{

  bool doExit = false;

  LString traceEnv = LString::fromEnv ( "LCL_TRACE" );

  // LCL Trace
  // --lcl-trace o1.x1{a=1}[1,2,3,4] o2.x2{x=1}[1,2,3,4]
  LString ltraceCmd = traceEnv;
  if ( isParam ( "lcl-trace" ) )
    {
      if ( hasParam ( "lcl-trace" ) )
	{
	  ltraceCmd += " " + getParam("lcl-trace");
	}
      else
	ltraceCmd += " default.stream[.*]";
    }
  ltraceCmd.setDelims ( " " );
  if ( ltraceCmd.numWords() )
    {
      if ( LString::fromEnv("LCL_NNTW").length() == 0 )
	{
	  lerr << "Warning!! Syntax of LCL Trace has changed in LCL-0.7!!" << endl;
	  lerr << "   This warning will be removed in LCL-0.8!" << endl;
	  lerr << "   It can be disabled setting env-var LCL_NNTW to any string" << endl;
	  lerr << flush;
	}
      setTracing ( ltraceCmd );
    }
  /*
  LTraceChannelManager::getInstance () -> addChannel ( "default", "ostream:fd=2" );
  if ( isParam ( "lcl-trace" ) || traceEnv.length() ) {
    LTracer::getInstance() -> state = true;
    if ( !hasParam ( "lcl-trace" ) && !traceEnv.length() ) {
      cout << "Enable all trace modules" << endl;
      LTracer::getInstance() -> enable_all = true;
    } else {
      cout << "Enabling trace modules by name:" << endl;
      LString modules = getParam ( "lcl-trace" ) + "," + traceEnv;
      LStringTokenizer tok ( ", " );
      tok.tokenize ( modules );
      for ( int i = 0; i<tok.numWords(); i++ ) {
	LTracer::getInstance() -> enableModule ( tok.word(i) );
	cout << "  -> " << tok.word(i) << endl;
      }
    }
  
    }*/

  // LCL LOG
  LString logEnv = LString::fromEnv ( "LCL_LOG" );
  LLogSystem::getInstance() -> setModule ( m_cmdLine.getAppName() );
  if ( isParam ( "lcl-log" ) || logEnv.length() ) {
    if ( !hasParam( "lcl-log" ) && !logEnv.length() ) {
      LLogSystem::getInstance () -> addLogHandler ( new LLogHandler_Console );
    } else {
      LString loggers = getParam ( "lcl-log" ) + ":" + logEnv;
      LStringTokenizer tok ( ":" );
      tok.tokenize ( loggers );
      for ( int i = 0; i<tok.numWords(); i++ ) {
	// cout << "Log to " << tok.word(i) << endl;
	LLogSystem::createLogHandler ( tok.word(i) );
      }      
    }
  } else {
    LLogSystem::createLogHandler ( "console?level=ERROR" );
  }

  // LCL Object Dumper
  if ( isParam ( "lcl-count" ) )
    {
      LString period = getParam ( "lcl-count", "1" );
      if ( (int)period == 0 )
	period = "1";
      LObjectDumper * func = new LObjectDumper;
      func->period = period;
      LThread tco ( func );
      tco.start ();
    }

  // LCL Debug
  if ( isParam ( "lcl-debug" ) ) {
    LVersionControlObject::dumpAll ( lout );
    LLogSystem::getInstance () -> dump ();
    LModuleInfo::dump ( lout );
    __lcl_DumpTraceChannels ( lout );
    debugFlag = true;
  }
  if ( isParam ( "lcl-trace-channels" ) )
    {
      __lcl_DumpTraceChannels ( lout );      
      doExit = true;
    }

  // LCL Version
  if ( isParam ( "lcl-version" ) ) {
    lout << "LCL " << LCL_VERSION << endl;
    lout << "  - Date: " << __LCL_TIMESTAMP__ << endl;
    /*
    lout << "  - Host: " << LCL_BUILD_HOST << endl;
    lout << endl;
    */
    if ( myBuildVersion.length() ) {
      lout << m_cmdLine.getAppName() << " " << myBuildVersion << endl;
      lout << "  - Build: " << myBuildNr << endl;
      lout << "  - Date: " << myBuildDate << endl;
      lout << "  - Host: " << myBuildHost << endl;
      lout << endl;
    }
    lout << flush;

    // switched to doExit ... give others a chance
    doExit = true;
  }

  if ( isParam ( "?") || isParam ( "h" ) || isParam ( "help" ) ) {
    lout << "See LCL parameters with --lcl-help" << endl;
  }
  if ( isParam ( "lcl-help" ) ) {
    showLCLParametersSyntax ();
    doExit = true;
  }

  if ( doExit )
    exit ( 0 );
}

void LApplication::setTracing ( LString const & param )
{

  LTracerX::getInstance() -> enabled = false;

  if ( !param.length() )
    return;

  LTracerX::getInstance() -> enabled = true;

  LString temp = param;

  //  lout << param << endl;
  LRegEx exp ( "([[:alnum:]]+)\\.([[:alnum:]]+)(\\{([^\\}]*)\\}|)(\\[([^\\]]*)\\]|)\\:" );

  if ( !exp.isValid() )
    return;

  while ( exp.exec ( temp ) )
    {
      LString name = exp.match(1);
      LString module = exp.match(2);
      LString vars = exp.match(4);
      LString channels = exp.match(6);
      /*
      lout << "Match 0:" << exp.match ( 0 ) << endl;
      lout << "Match 1:" << exp.match ( 1 ) << endl;
      lout << "Match 2:" << exp.match ( 2 ) << endl;
      lout << "Match 3:" << exp.match ( 3 ) << endl;
      lout << "Match 4:" << exp.match ( 4 ) << endl;
      lout << "Match 5:" << exp.match ( 5 ) << endl;
      lout << "Match 6:" << exp.match ( 6 ) << endl;
      */

      lout << "LCL -> Creating new trace output [type=" << module << "] [name=" << name << "]" << endl;

      LManagedPointer<LCursor> cursor;

      lout << "  Active Options:" << endl;
      LHash<LString,LString> opts = vars.parseToHash ( "([^=]*)=([^,]*)(,|)" );
      LSet<LString> keys = opts.getKeys ();
      cursor = keys.createCursor ();
      forcursor ( cursor )
	{
	  lout << "   " << keys.elementAt ( cursor ) << " = " << opts.get(keys.elementAt ( cursor ), "" ) << endl;
	}

      lout << "  Active Filters:" << endl;
      LStringTokenizer tok ( ", " );
      tok.tokenize ( channels );
      LHeap<LString> channelList = tok.asHeap ();
      if ( channelList.entries() == 0 )
	{
	  channelList.add ( ".*" );
	}
      cursor = channelList.createCursor ();
      forcursor ( cursor )
	{
	  lout << "   " << channelList.elementAt ( cursor ) << endl;
	}

      LManagedPointer<LTraceOutput> output;
      if ( module.toUpper() == "STREAM" )
	{
	  output = new LTraceOutput_OStream;
	}
      else if ( module.toUpper() == "FILE" )
	{
	  output = new LTraceOutput_File;
	}
      if ( output.isValid() )
	{
	  output->setOptions ( opts );
	  cursor = channelList.createCursor ();
	  forcursor ( cursor )
	    {
	      output->addFilter ( channelList.elementAt ( cursor ) );
	    }
	  LTracerX::getInstance()->addOutput ( name, output );
	}
      else
	{
	  lout << "  Unkown output [type=" << module << "]. Skipping!!" << endl;
	}

      temp = exp.afterMatch ( 0 );
    }

}


/* *******************************************************************************************
 * *   PARAMETER FUNCS   <<----------------
 * ******************************************************************************************* */

  /*!
   * \param key specifies the parameter name to look for
   * \param def the default value which will be returned if the parameter is not set
   *
   * The method will check if a parameter is set an return the assigned value ( which might also be
   * an empty string ).
   *
   * If the parameter is not specified the default value (def) will be returned.
   */

LString LApplication::getParam ( LString key, LString def )
{
  //  return paramDict.get ( key, def );
  return m_cmdLine.getData ( key, def );
}

/*!
 * \param key specifies the parameter name to look for
 * \return returns \b true if the variable is set, otherwise \b false
 *
 * The method will also return true is the parameter contains an empty string. To check
 * if the parameter contains data use hasParam.
 */

bool LApplication::isParam ( LString const & key )
{
  //  return paramDict.isSet ( key );
  return m_cmdLine.isSet ( key );
}

/*!
 * \param key specifies the parameter name to look for
 * \return returns \b true if the variable has a value assigned, otherwise \b false
 *
 * This method will only return \b true if the parameter has a value assigned. To check only
 * if a parameter was specified use isParam.
 */

bool LApplication::hasParam ( LString const & key )
{
  /*
  if ( !isParam ( key ) )
    return false;

  return paramDict.get ( key ) != "" ;
  */
  return m_cmdLine.hasData ( key );
}

LHeap<LString> LApplication::getFileList()
{
  //  return paramFiles;
  return m_cmdLine.getFileList ();
}

LString LApplication::args()
{
  //  return argStr;
  return "";
}

/*!
 * \return the application name
 *
 * It will return the application file name (normally coming from argv[0]) without the leading path name.
 */

LString LApplication::appName () const
{
  //  return progName;
  return m_cmdLine.getAppName ();
}

LApplication::~LApplication ()
{  
  isDestructing = true;
}

void LApplication::printDebug ( LString const & msg )
{
  if ( !debugFlag )
    return;

  lerr << msg << endl << flush;
}
