/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#include <lcl.h>

#include <time.h>


#include <lcl/lappcontrol.h>

#include <lcl/lutils.h>
#include <lcl/lselectloop.h>
#include <lcl/lapp.h>
#include <lcl/lserialize.h>

#include <lcl/lappcontrol_class.h>

#include <lcl/lappevent.h>

LApplicationControl_DummyClass app_dummyClass;
LApplicationControl_TraceClass app_traceClass;

LTRACE_CHANNEL_STATIC("LApplicationControl_Base", __lcl_trace_appctrl_1, "Application Control Base Messages" );

// LApplicationControl_Base

LApplicationControl_Base::LApplicationControl_Base ( LSocket4Inet const & _socket ) :
  C_timeout ( 10 ),
  socket ( _socket ),
  serial ( 0 ),
  lastData ( time(0) )
{
}

void LApplicationControl_Base::setLastDataNow ()
{
  lastData = time(0);
}

bool LApplicationControl_Base::isConnectionTimedOut () const
{
  return difftime ( time(0), lastData ) > C_timeout;
}

void LApplicationControl_Base::readEvent ( LFileDesc * fd )
{

  LTRACE ( "LApplicationControl_Base", "Got read event" );

  char buffer [ 1024 ];
  int rc = socket.read ( buffer, sizeof(buffer) );

  if ( rc <= 0 )
    {
      socket.close ();
      LTRACE ( "LApplicationControl_Base", "Connection lost (rc=" + LString(rc) + ")" );
      connectionLost ();
    }
  else
    {
      LTRACE ( "LApplicationControl_Base", "Adding " + LString(rc) + " bytes to input buffer" );
      readBuffer.append ( buffer, rc );
      //      readBuffer += LBuffer ( rc, buffer );

      setLastDataNow ();
      processInputData ();
    }
  
}


void LApplicationControl_Base::writeEvent ( LFileDesc * fd )
{
  LTRACE ( "LApplicationControl_Base", "Got write event" );

  if ( writeBuffer.getSize() == 0 )
    {
      setEventFlags ();
      return;
    }

  int rc = socket.write ( writeBuffer.getConstPtr(), writeBuffer.getSize() );

  if ( rc <= 0 )
    {
      connectionLost ();
    }
  else
    {
      (void)writeBuffer.cutFrom ( rc );
    }

  setEventFlags ();
}

void LApplicationControl_Base::setEventFlags ()
{
  if ( writeBuffer.getSize() )
    {
      socket.setEventMask ( LFileDesc::eventRead | LFileDesc::eventWrite );
    }
  else
    {
      socket.setEventMask ( LFileDesc::eventRead );
    }
}


void LApplicationControl_Base::processInputData ( void )
{
  unsigned long i = 0;
  unsigned long len = readBuffer.getSize();
  char * ptr = (char*)readBuffer.getPtr();

  LTRACE ( "LApplicationControl_Base", "Processing " + LString(len) + " bytes of data" );

  for ( i = 0; i<len; i++ )
    {
      if ( ptr[i] == '\n' )
	{

	  // Parse Packet
	  Packet packet;
	  LDefaultSerializer ser;
	  ser.setBuffer ( LBuffer ( i, ptr ) );
	  ser >> packet;

	  readBuffer.cutFrom ( i+1 );
	  ptr = (char*)readBuffer.getPtr();
	  len = readBuffer.getSize();
	  i = 0;
	  LTRACE ( "LApplicationControl_Base", LString(len) + " bytes of data left" );
	  if ( packet.getReplySerial() )
	    {
	      LManagedPointer<Reply> reply = new Reply;
	      reply->getPacket() = packet;
	      processReply ( reply );
	    }
	  else
	    {
	      LManagedPointer<Request> request = new Request;
	      request->getPacket() = packet;
	      processRequest ( request );
	    }

	}
    }
}

void LApplicationControl_Base::processRequest ( LManagedPointer<Request> request )
{
  LTRACE ( "LApplicationControl_Base", "Got " + request->getPacket().getCommand() + " request" );

  if ( request->getPacket().getCommand() == "ping" )
    {
      Reply reply ( request );
      reply.getPacket().getVars().clear();
      reply.getPacket().setCommand ( "pong" );
      sendReply ( reply );
      gotPing ( request );
    }  
  else if ( request->getPacket().getCommand() == "class" )
    {
      marshallClass ( request );
    }
}

void LApplicationControl_Base::marshallClass ( LManagedPointer<Request> request )
{
  LApplicationControl_Class * classObject = classHash.get ( request->getPacket().getTarget().get("class", "" ), NULL );
  if ( classObject )
    {
      classObject->queueRequest ( this, request );
    }
  else
    {
      Reply reply ( request );
      reply.getPacket().setCommand ( "class-not-found" );
      sendReply ( reply );
    }
}

void LApplicationControl_Base::processReply ( LManagedPointer<Reply> reply )
{
  LTRACE ( "LApplicationControl_Base", "Got reply for " + LString(reply->getPacket().getReplySerial() ) );
  LManagedPointer < LCursor > cursor = requestList.createCursor ();
  forcursor ( cursor )
    {
      LManagedPointer<Request> request = requestList.elementAt ( cursor );
      if ( request -> getPacket().getSerial() == reply->getPacket().getReplySerial() )
	{
	  LTRACE ( "LApplicationControl_Base", "Removing request " + LString(reply->getPacket().getReplySerial() ) + " from request list" );
	  // Reply received

	  request->replyData = reply;
	  request->reply = true;
	  request->flagEvent ();
	  request->eventCompleted.emit();
	  request->requestCompleted.emit ( request );
	  eventCompleteRequest ( request );
	  
	  requestList.removeElementAt ( cursor );
	  return;
	  // Our cursor will be invalid from here ... doesn't matter!
	  // We only have one matching entry in here!
	}
    }
}

void LApplicationControl_Base::sendData ( LBuffer const & data )
{
  writeBuffer.append ( data.getConstPtr(), data.getSize() );
  writeBuffer.append ( "\n", 1 );
  setEventFlags ();
}

void LApplicationControl_Base::sendRequest ( LApplicationControl_Base::Request const & _request )
{
  LManagedPointer<Request> request = new Request ( _request );
  sendRequest ( request );
}

void LApplicationControl_Base::sendRequest ( LManagedPointer<Request> request )
{
  request->reply = false;
  request->getPacket().setSerial ( serial++ );
  request->getPacket().setReplySerial ( 0 );
  request->setSendTime ( time(0) );
  sendData ( request->getPacket().getData() );
  requestList.add ( request );
}

void LApplicationControl_Base::sendReply ( Reply const & _reply )
{
  LManagedPointer<Reply> reply = new Reply ( _reply );
  sendReply ( reply );
}

void LApplicationControl_Base::sendReply ( LManagedPointer<Reply> reply )
{
  reply->getPacket().setSerial ( serial++ );
  reply->setSendTime ( time(0) );
  sendData ( reply->getPacket().getData() );
  replyList.add ( reply );
}

void LApplicationControl_Base::sendPing ( void )
{
  LApplicationControl_Base::Request request;
  request.getPacket().setCommand ( "ping" );
  request.getPacket().getVars().set ( "client-name", theApp.appName() );

  sendRequest ( request );
}

void LApplicationControl_Base::processLists ()
{
  LManagedPointer<LCursor> cursor;

  LSet < LManagedPointer < Request > > removeList1;
  LSet < LManagedPointer < Reply > > removeList2;

  cursor = requestList.createCursor ();
  forcursor ( cursor )
    {
      LManagedPointer<Request> request = requestList.elementAt ( cursor );
      if ( request->timedOut () )
	{
	  LTRACE ( "LApplicationControl_Base", "Request " + LString(request->getPacket().getSerial()) + " timed out!" );
	  request->flagEvent();
	  timeoutRequest ( request );
	  removeList1.add ( request );	  
	}
    }

  // Remove elements from remove list
  cursor = removeList1.createCursor ();
  forcursor ( cursor )
    {
      requestList.removeElement ( removeList1.elementAt ( cursor ) );
    }
  removeList1.clear ();

  // Process reply list

  cursor = replyList.createCursor ();
  forcursor ( cursor )
    {
      LManagedPointer<Reply> reply = replyList.elementAt ( cursor );
      if ( reply->timedOut () )
	{
	  LTRACE ( "LApplicationControl_Base", "Reply " + LString(reply->getPacket().getSerial()) + " for request " + LString(reply->getPacket().getReplySerial()) + " timed out!" );
	  timeoutReply ( reply );
	  removeList2.add ( reply );
	}
    }

  // Remove elements from remove list
  cursor = removeList2.createCursor ();
  forcursor ( cursor )
    {
      replyList.removeElement ( removeList2.elementAt ( cursor ) );
    }
  removeList2.clear ();

  LTRACE ( "LApplicationControl_Base", LString(requestList.entries()) + " items remain in request list" );
  LTRACE ( "LApplicationControl_Base", LString(replyList.entries()) + " items remain in reply list" );
  
}

void LApplicationControl_Base::timeoutRequest ( LManagedPointer<Request> request )
{
  requestTimeout.emit ( request );
  request->eventTimedOut.emit();
  request->requestTimedOut.emit ( request );
  eventTimeoutRequest ( request );
}

void LApplicationControl_Base::timeoutReply ( LManagedPointer<Reply> reply )
{
  replyTimeout.emit ( reply );
  reply->eventTimedOut.emit();
}

void LApplicationControl_Base::eventCompleteRequest ( LManagedPointer<Request> request )
{
}

void LApplicationControl_Base::eventTimeoutRequest ( LManagedPointer<Request> request )
{
}

void LApplicationControl_Base::registerClass ( LString const & className, LApplicationControl_Class * classObject )
{
  classHash.set ( className, classObject );
}

void LApplicationControl_Base::unregisterClass ( LString const & className )
{
  classHash.unset ( className );
}

void LApplicationControl_Base::gotPing ( LManagedPointer<Request> request )
{  
}

LManagedPointer<LApplicationControl_Base::Request> LApplicationControl_Base::sendEvent ( LString const & channel, LHash<LString,LString> const & data, LString const & app, LString const & node )
{
  LManagedPointer<Request> request = new Request;
  request->getPacket().setCommand ( "class" );
  request->getPacket().getTarget().set ( "class", "event" );
  if ( app.length() )
    request->getPacket().getTarget().set ( "client", app );
  if ( node.length() )
    request->getPacket().getTarget().set ( "node", node );

  LDefaultSerializer ser;

  ser << channel;
  ser << data;

  request->getPacket().getVars().set ( "command", "push" );
  request->getPacket().getVars().set ( "event", ser.getBuffer() );

  sendRequest ( request );  

  return request;
}

// LApplicationControl

LApplicationControl::LApplicationControl () :
  LApplicationControl_Base ( LSocket4Inet ( AF_INET, SOCK_STREAM ) ),
  connected ( false ),
  addr ( "127.0.0.1", 5886 ),
  runnerFn ( &LApplicationControl::runner, *this )
{
  LThread tco1 ( &runnerFn );
  tco1.start ();

  registerClass ( "dummy", &app_dummyClass );
  registerClass ( "trace", &app_traceClass );

  app_dummyClass.start ();
  app_traceClass.start ();

  registerClass ( "event", LApplicationEvent_Class::getInstance() );
}

void LApplicationControl::connectionLost ()
{
  LTRACE_FUNC ( "LApplicationControl", "void LApplicationControl::connectionLost ()" );
  connected = false;

  socket = LSocket4Inet ( AF_INET, SOCK_STREAM );

  while ( !connected )
    {
      connected = socket.connect ( addr );
      if ( !connected )
	{
	  processLists ();
	  lcl_sleep ( 1 );
	}
    }

  setEventFlags ();
  socket.reg ( LSelectLoop::getInstance() );

  /*
   * The following type cast of the method pointer is needed by the fucking stupid MSVC (5,6 maybe 7) compiler.
   *
   * If you want to fix this bug in the MSVC compiler please donate some C++ specification to the MSVC developer team.
   * If they are able to read maybe there will a be a MSVC compiler in the future which _supports_ templates.
   */
  connect_emitter_1 ( (void (LApplicationControl::*)(LFileDesc*))&LApplicationControl::writeEvent, *this, socket.writeEvent );
  connect_emitter_1 ( (void (LApplicationControl::*)(LFileDesc*))&LApplicationControl::readEvent, *this, socket.readEvent );

  LTRACE ( "LApplicationControl", "Connected!" );
}

void LApplicationControl::timeoutEvent ( void )
{
  tix++;
  if ( tix % 10 == 0 )
    {
      processLists ();
      sendPing ();
    }
}

void LApplicationControl::runner ( void )
{

  LTRACE ( "LApplicationControl", "Runner active..." );

  connectionLost ();

  LSelectLoop::getInstance() -> setTimeout ( 100 );
  connect_emitter_0<LApplicationControl> ( &LApplicationControl::timeoutEvent, *this, LSelectLoop::getInstance()->timeoutEvent );

  while ( true )
    {
      LSelectLoop::getInstance() -> loop ();
    }
}


// LApplicationControl_Base::Packet

LApplicationControl_Base::Packet::Packet () :
  vars ( 10 ),
  target ( 2 )
{
}

LBuffer LApplicationControl_Base::Packet::getData ()
{
  LDefaultSerializer ser;
  ser << (*this);
  return ser.getBuffer();
}

void LApplicationControl_Base::Packet::setReplySerial ( unsigned long _replySerial )
{
  replySerial = _replySerial;
}

unsigned long LApplicationControl_Base::Packet::getReplySerial () const
{
  return replySerial;
}

void LApplicationControl_Base::Packet::setCommand ( LString const & _command )
{
  command = _command;
}

LString LApplicationControl_Base::Packet::getCommand () const
{
  return command;
}

LHash<LString,LString> & LApplicationControl_Base::Packet::getVars ()
{
  return vars;
}

LHash<LString,LString> const & LApplicationControl_Base::Packet::getVars() const
{
  return vars;
}

LHash<LString,LString> & LApplicationControl_Base::Packet::getTarget ()
{
  return target;
}

LHash<LString,LString> const & LApplicationControl_Base::Packet::getTarget () const
{
  return target;
}

void LApplicationControl_Base::Packet::setSerial ( unsigned long _serial )
{
  serial = _serial;
}

unsigned long LApplicationControl_Base::Packet::getSerial () const
{
  return serial;
}

LApplicationControl_Base::Packet LApplicationControl_Base::Packet::fromData ( LBuffer const & data )
{
  LDefaultSerializer ser;
  ser.setBuffer ( data );

  Packet packet;
  ser >> packet;

  return packet;
}

// LApplicationControl::BasicRequest


int LApplicationControl_Base::BasicRequest::C_timeout = 10;

LApplicationControl_Base::BasicRequest::BasicRequest () :
  sendTime ( 0 ),
  timeout ( C_timeout )
{
}

bool LApplicationControl_Base::BasicRequest::timedOut () const
{
  return difftime ( time(0), sendTime ) > timeout;
}

int LApplicationControl_Base::BasicRequest::getTimeout() const
{
  return timeout;
}

void LApplicationControl_Base::BasicRequest::setTimeout ( int _timeout )
{
  timeout = _timeout;
}

void LApplicationControl_Base::BasicRequest::setSendTime ( unsigned long _sendTime )
{
  sendTime = _sendTime;
}

unsigned long LApplicationControl_Base::BasicRequest::getSendTime() const
{
  return sendTime;
}

LApplicationControl_Base::Packet & LApplicationControl_Base::BasicRequest::getPacket ()
{
  return packet;
}

LApplicationControl_Base::Packet const & LApplicationControl_Base::BasicRequest::getPacket () const
{
  return packet;
}

// LApplicationControl_Base::Request

LApplicationControl_Base::Request::Request () :
  reply ( false ),
  semaphore ( new LSemaphore ( 0 ) )
{
}

bool LApplicationControl_Base::Request::gotReply () const
{
  return reply;
}

LManagedPointer<LApplicationControl_Base::Reply> LApplicationControl_Base::Request::getReplyData () const
{
  return replyData;
}

void LApplicationControl_Base::Request::flagEvent ()
{
  semaphore->post ();
}

void LApplicationControl_Base::Request::wait ()
{
  semaphore->wait ();
}

// LApplicationControl_Base::Reply

LApplicationControl_Base::Reply::Reply () :
  BasicRequest ()
{
}

LApplicationControl_Base::Reply::Reply ( Request const & p1 ) :
  BasicRequest ()
{
  packet.setReplySerial ( p1.getPacket().getSerial () );
}

LApplicationControl_Base::Reply::Reply ( LManagedPointer<Request> p1 ) :
  BasicRequest ()
{
  packet.setReplySerial ( p1->getPacket().getSerial() );
}
