/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#include <lcl/lappevent.h>

#include <lcl/ltrace.h>

LTRACE_CHANNEL_STATIC("LApplicationEvent_Class", __lcl_trace_channel_1, "Information about Application Events from the base class" );
LTRACE_CHANNEL_STATIC("LApplicationEvent", __lcl_trace_channel_2, "Information about Application Events data class" );
LTRACE_CHANNEL_STATIC("LApplicationEventQueue", __lcl_trace_channel_3, "Information about Application Events queue class" );

// LApplicationEvent

void LApplicationEvent::setChannel ( LString const & _channel )
{
  channel = _channel;
}

LString LApplicationEvent::getChannel () const
{
  return channel;
}

void LApplicationEvent::setData ( LHash<LString,LString> const & _data )
{
  data = _data;
}

LHash<LString,LString> const & LApplicationEvent::getData () const
{
  return data;
}

// LApplicationEventQueue

void LApplicationEventQueue::inject ( LApplicationEvent const & e )
{
  if ( match ( e.channel ) )
    {
      m_sender.send ( e );
    }
}

bool LApplicationEventQueue::match ( LString const & channel )
{
  LAutoLock _lock ( controlLock );
  LManagedPointer<LCursor> cursor = m_channelList.createCursor ();
  forcursor ( cursor )
    {
      LTRACE ( "LApplicationEventQueue", "Trying queue match: " + m_channelList.elementAt ( cursor ) );
      LRegEx exp ( m_channelList.elementAt ( cursor ) );
      if ( exp.exec ( channel ) )
	{
	  return true;
	}
    }
  return false;
}


LApplicationEventQueue::LApplicationEventQueue ( LApplicationEventManager & mgr ) :
  m_mgr ( mgr )
{
  m_mgr.registerQueue ( this );
}

LApplicationEventQueue::LApplicationEventQueue ( LApplicationEventQueue const & p1 ) :
  m_mgr ( p1.m_mgr ),
  m_channelList ( p1.m_channelList )
{
  m_mgr.registerQueue ( this );
}

LApplicationEventQueue::~LApplicationEventQueue ()
{
  m_mgr.unregisterQueue ( this );
}
  
void LApplicationEventQueue::setQueueReceiver ( LQueueReceiver<LApplicationEvent> & receiver )
{
  m_sender.setReceiver ( receiver );
}

void LApplicationEventQueue::addChannel ( LString const & channel )
{
  LAutoLock _lock ( controlLock );
  m_channelList.add ( channel );
}

void LApplicationEventQueue::removeChannel ( LString const & channel )
{
  LAutoLock _lock ( controlLock );
  m_channelList.removeElement ( channel );
}

// LApplicationEventManager

void LApplicationEventManager::registerQueue ( LApplicationEventQueue * queue )
{
  LAutoLock _lock ( controlLock );
  m_queues.add ( queue );
}

void LApplicationEventManager::unregisterQueue ( LApplicationEventQueue * queue )
{
  LAutoLock _lock ( controlLock );
  m_queues.removeElement ( queue );
}

void LApplicationEventManager::push ( LApplicationEvent const & e )
{
  LAutoLock _lock ( controlLock );
  LManagedPointer<LCursor> cursor = m_queues.createCursor ();
  LTRACE ( "LApplicationEventManager", "Pushing event for channel " + e.getChannel() + " to " + LString(m_queues.entries()) + " queues" );
  forcursor ( cursor )
    {
      m_queues.elementAt ( cursor ) -> inject ( e );
    }
}

// LApplicationEvent_Class
LApplicationEvent_Class::LApplicationEvent_Class ()
{
}

LApplicationEvent_Class * LApplicationEvent_Class::theInstance = NULL;

LApplicationEvent_Class * LApplicationEvent_Class::getInstance()
{
  if ( !theInstance ) 
    {
      theInstance = new LApplicationEvent_Class;
      theInstance->start ();
    }
  return theInstance;
}

void LApplicationEvent_Class::runner ( void )
{
  while ( true )
    {
      LManagedPointer<Request> request = fetchRequest ();

      LString cmd = request->getData().getVars().get ( "command", "" );      

      LApplicationControl_Base::Packet packet;

      LString reply;
      
      packet.setCommand( "OK" );
      if ( cmd == "push" )
	{
	  LTRACE ( "LApplicationEvent_Class", "Got push" );
	  LHash<LString,LString> data;
	  LString channel;
	  try
	    {
	      LDefaultSerializer ser;
	      ser.setBuffer ( request->getData().getVars().get ( "event", "" ) );
	      ser >> channel;
	      ser >> data;
	      LTRACE ( "LApplicationEvent_Class", "Push is for channel: " + channel );
	      LApplicationEvent e;
	      e.setChannel ( channel );
	      e.setData ( data );
	      m_mgr.push ( e );
	      reply = "ok";
	    }
	  catch ( ... )
	    {
	      packet.setCommand ( "error" );
	      packet.getVars().set ( "error", "invalid data" );
	    }
	}
      else if ( cmd == "help" )
	{
	  reply = \
	    "\n\n" \
	    "LCL Application Event Class:\n" \
	    "============================\n" \
	    "\n";
	    
	}
      else
	{
	  packet.setCommand ( "error" );
	  packet.getVars().set ( "error", "Unkown command: '" + cmd + "'! Use 'help' to get some info!" );
	}

      if ( reply.length() )
	packet.getVars().set ( "reply", reply );

      request->sendReply ( packet );
    }
}

LApplicationEventManager & LApplicationEvent_Class::getManager ()
{
  return m_mgr;
}

LApplicationEvent_Class::~LApplicationEvent_Class ()
{
}

