/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#include <lcl/lcmdline.h>

typedef struct _LCommandLineInfo {
  char short_param;
  char const * long_param;
  char const * desc;
  bool has_data;
  _LCommandLineInfo * next;
} LCommandLineInfo;

LCommandLineInfo * _commandLineInfoList = NULL;
LCommandLineInfo * _commandLineInfoList_last = NULL;

void lcl_cmdLineInfoAdd ( LCommandLineInfo * info )
{
  info->next = NULL;
  if ( _commandLineInfoList )
    {
      _commandLineInfoList_last->next = info;
      _commandLineInfoList_last = info;
    }
  else
    {
      _commandLineInfoList_last = _commandLineInfoList = info;
    }
}

bool lcl_cmdLineInfoHasData ( char c )
{
  LCommandLineInfo * current = _commandLineInfoList;
  while ( current )
    {
      if ( current->short_param == c )
	{
	  return current->has_data;
	}
      current = current->next;
    }
  return false;
}

bool lcl_cmdLineInfoHasData ( LString const & str )
{
  LCommandLineInfo * current = _commandLineInfoList;
  while ( current )
    {
      if ( current->long_param )
	if ( current->long_param == str )
	  {
	    return current->has_data;
	  }
      current = current->next;
    }
  return false;
}

LCommandLineInfoClass::LCommandLineInfoClass ( char const * short_param, char const * long_param, bool has_data, char const * desc )
{
  LCommandLineInfo * info = new LCommandLineInfo;
  info->short_param = short_param[0];
  info->long_param = strdup ( long_param );
  info->has_data = has_data;
  info->desc = strdup ( desc );
  lcl_cmdLineInfoAdd ( info );
}

LCommandLine::LCommandLine ( )
{
}

LCommandLine::~LCommandLine ()
{
}

bool LCommandLine::isSet ( LString const & param ) const
{
  return parameters.isSet ( param );
}

bool LCommandLine::hasData ( LString const & param ) const
{
  return parameters.get ( param, "" ).length() > 0;
}

LString LCommandLine::getData ( LString const & param, LString const & _default ) const
{
  return parameters.get ( param, _default );
}

LHeap<LString> LCommandLine::getFileList () const
{
  return fileList;
}

LString LCommandLine::getAppName () const
{
  return appName;
}

LHash<LString,LString> LCommandLine::getParameters () const
{
  return parameters;
}

void LCommandLine::init ( int argc, char * argv [] )
{
  int i = 0;

  parameters.clear ();
  fileList.clear ();
  appName = "";

  bool param0 = true;
  bool inParam = false;
  LString key, value;

  // Now step through all parameters
  while ( (i < argc) && (argv[i]) )
    {
      if ( param0 )
	{
	  appName = argv[i];
	  param0 = false;
	  i++;
	  continue;
	}
      
      LString temp = argv[i];
      if ( temp.left(2) == "--" )
	{
	  if ( inParam )
	    {
	      parameters.set ( key, "" );
	    }
	  key = temp.subString(2);
	  if ( lcl_cmdLineInfoHasData ( key ) )
	    {
	      inParam = true;
	    }
	  else
	    {
	      inParam = false;
	      parameters.set ( key, "" );
	    }
	}
      else if ( temp.left(1) == "-" )
	{
	  int j = 1;
	  char c;
	  while ( c = temp.get(j) )
	    {
	      if ( inParam )
		parameters.set ( key, "" );
	      key = c;
	      if ( lcl_cmdLineInfoHasData ( c ) )
		{
		  inParam = true;
		}
	      else
		{
		  inParam = false;
		  parameters.set ( key, "" );
		}
	      j++;
	    }
	}
      else
	{
	  if ( inParam )
	    {
	      parameters.set ( key, temp );
	      inParam = false;
	    }
	  else
	    {
	      fileList.add ( temp );
	    }
	}

      i++;      
    }
  if ( inParam )
    parameters.set ( key, "" );
}

void LCommandLine::showHelp ( LOStream & outs )
{
  outs << "Application help" << endl;
  outs << "--------------------" << endl;
  outs << endl;

  LCommandLineInfo * current = _commandLineInfoList;
  while ( current ) 
    {
      outs << "\t";
      if ( current->short_param )
	{
	  outs << "-" << current->short_param;
	}
      else
	{
	  outs << "  ";
	}
      if ( current->long_param )
	{
	  if ( current->short_param )
	    outs << ", ";
	  else
	    outs << "  ";
	  outs << "--" << current->long_param;
	}
      else
	{
	}
      outs << endl;
      outs << "\t\t";
      outs << current->desc;
      outs << endl << endl;
      current = current->next;
    }

  outs << endl;
  outs << flush;
}
