#include <lcl/lcond.h>

/*
 * LCondVar:
 *  Implementation of a condition variable. Uses posix condition vars.
 *  Problem: currently no mutexes are allowed inside a cond var
 *
 */

#include <time.h>

LCondVar::LCondVar ()
{
  pthread_cond_init ( &cond, 0 );
}

LCondVar::~LCondVar ()
{
  pthread_cond_destroy ( &cond );
}

bool LCondVar::wait ()
{
  return pthread_cond_wait ( &cond, &mutex.mutxp ) == 0;
}

bool LCondVar::timedWait ( int sec )
{
  timespec tm;
  tm.tv_sec = time(NULL) + sec;
  tm.tv_nsec = 0;

  return pthread_cond_timedwait ( &cond, &mutex.mutxp, &tm ) == 0;
}

void LCondVar::signal ()
{
  pthread_cond_signal ( &cond );
}

void LCondVar::broadcast ()
{
  pthread_cond_broadcast ( &cond );
}

LMutex & LCondVar::getMutex()
{
  return mutex;
}



