#include <lcl/lconfmodule.h>

#include <lcl/lfstream.h>

class ConfigWriterIter_default_2 : public LHashIterator < LString, LString > {
public:
  LOStream * stream;
  void applyTo ( LString & _key, LString & _value ) {
    LString key = _key;
    LString value = _value;
    key.change ( "\\", "\\\\" );
    key.change ( ":", "\\:" );
    key.change ( ";", "\\;" );
    key.change ( "{", "\\{" );
    key.change ( "}", "\\}" );
    value.change ( "\\", "\\\\" );
    value.change ( ":", "\\:" );
    value.change ( ";", "\\;" );
    value.change ( "{", "\\{" );
    value.change ( "}", "\\}" );
    (*stream) << "  " << key << ":" << value << ";" << endl;
  }
};

class ConfigWriterIter_default_1 : public LIterator < LConfigObject > {
public:
  LOStream * stream;
  ConfigWriterIter_default_2 cwi;

  void applyTo ( LConfigObject & obj )
  {
    (*stream) << obj.getType() << " {" << endl;
    cwi.stream = stream;
    obj.getHash().allElementsDo ( cwi );
    (*stream) << "}" << endl << endl;
  }
};

// ----------->>>>>>>> LConfigReader

LConfigReader::~LConfigReader ()
{
}

// ----------->>>>>>>> LConfigWriter

LConfigWriter::~LConfigWriter ()
{
}

// ----------->>>>>>>> LConfigFileReader

LConfigFileReader::LConfigFileReader ( LIStream & instream ) :
  m_stream ( &instream )
{
}

LConfigFileReader::LConfigFileReader ( LString const & fname ) :
  m_stream ( NULL )
{
  m_file = new LFile ( fname, LFDManager::read );
  if ( m_file.isValid() )
    {
      m_stream = new LFIStream ( *m_file );
    }
}

LConfigFileReader::~LConfigFileReader ()
{
  // If the file object is valid, then the stream was created by ourself!
  if ( m_file.isValid() )
    delete m_stream;
}

void LConfigFileReader::read ( LConfigSystem & config ) const
{
  try {
    read_stream ( *m_stream, config );
  }
  catch ( LEndOfStreamException & e )
    {
    }
  catch ( LErrorStreamException & e )
    {
    }
}

// ----------->>>>>>>> LConfigFileWriter


LConfigFileWriter::LConfigFileWriter ( LOStream & outstream ) :
  m_stream ( &outstream )
{
}

LConfigFileWriter::LConfigFileWriter ( LString const & fname, int oa, int om ) :
  m_stream ( NULL )
{
  m_file = new LFile ( fname, oa | LFDManager::write, om );
  if ( m_file.isValid() )
    {
      m_stream = new LFOStream ( *m_file );
    }
}

LConfigFileWriter::~LConfigFileWriter ()
{
  // If the file object is valid, then the stream was created by ourself!
  if ( m_file.isValid() )
    delete m_stream;
}

void LConfigFileWriter::write ( LConfigSystem & config ) const
{
  write_stream ( *m_stream, config );
}


// ----------->>>>>>>> LConfigWriter_default

LConfigWriter_default::LConfigWriter_default ( LOStream & _stream ) :
  LConfigFileWriter ( _stream )
{
}


LConfigWriter_default::LConfigWriter_default ( LString const & fname, int oa, int om ) :
  LConfigFileWriter ( fname, oa, om )
{
}

LConfigWriter_default::~LConfigWriter_default ()
{
}

void LConfigWriter_default::write_stream ( LOStream & stream, LConfigSystem & config ) const
{
  ConfigWriterIter_default_1 cwi;
  cwi.stream = &stream;
  config.forEach ( cwi );
}

// ----------->>>>>>>> LConfigReader_default

LConfigReader_default::LConfigReader_default ( LIStream & _stream ) :
  LConfigFileReader ( _stream )
{
}

LConfigReader_default::LConfigReader_default ( LString const & fileName ) :
  LConfigFileReader ( fileName )
{
}

LConfigReader_default::~LConfigReader_default ()
{
}

void LConfigReader_default::read_stream ( LIStream & stream, LConfigSystem & config ) const
{
  config.read ( new LDefaultConfigParser ( stream, "#" ) );
}

