

#include <lcl/lconfmodule_tcl.h>

#include <lcl/lstringtokenizer.h>
#include <lcl/ltrace.h>

#ifndef DOXYGEN_RUNNING
class ConfigWriterIter_TCL_1 : public LIterator < LConfigObject > {
public:
  LOStream * stream;
  LHash<LString,LTable<LString> > m_ref;

  void applyTo ( LConfigObject & obj )
  {
    (*stream) << obj.getType() << " ";

    LTable<LString> const & ref = m_ref.get(obj.getType(),LTable<LString>(0) );

    // Skip if the entry is not configured
    if ( !ref.getSize() )
      return;

    luint32 i;
    for ( i = 0; i<ref.getSize(); i++ )
      {
	LString item = obj.get(ref[i]);
	
	item = item.enSave();
	item.setDelims ( " " );
	switch ( item.numWords() )
	  {
	  case 0:
	    (*stream) << "\"" << item << "\"";
	    break;
	  case 1:
	    (*stream) << item;
	    break;
	  default:
	    (*stream) << "\"" << item << "\"";
	    break;
	  }
	(*stream) << " ";
      }
    (*stream) << endl;
  }
};
#endif

// ----------->>>>>>>> LConfigWriter_TCL

LConfigWriter_TCL::LConfigWriter_TCL ( LHash < LString, LTable < LString > > const & ref, LOStream & _stream ) :
  LConfigFileWriter ( _stream ),
  m_reference ( ref )
{
}


LConfigWriter_TCL::LConfigWriter_TCL ( LHash < LString, LTable < LString > > const & ref, LString const & fname, int oa, int om ) :
  LConfigFileWriter ( fname, oa, om ),
  m_reference ( ref )
{
}

LConfigWriter_TCL::~LConfigWriter_TCL ()
{
}

void LConfigWriter_TCL::write_stream ( LOStream & stream, LConfigSystem & config ) const
{
  ConfigWriterIter_TCL_1 cwi;
  cwi.stream = &stream;
  cwi.m_ref = m_reference;
  config.forEach ( cwi );
}

// ----------->>>>>>>> LConfigReader_TCL

LConfigReader_TCL::LConfigReader_TCL ( LHash < LString, LTable < LString > > const & ref, LIStream & _stream ) :
  LConfigFileReader ( _stream ),
  m_reference ( ref )
{
}

LConfigReader_TCL::LConfigReader_TCL ( LHash < LString, LTable < LString > > const & ref, LString const & fileName ) :
  LConfigFileReader ( fileName ),
  m_reference ( ref )
{
}

LConfigReader_TCL::~LConfigReader_TCL ()
{
}

void LConfigReader_TCL::read_stream ( LIStream & stream, LConfigSystem & config ) const
{
  try {
    
    LStringTokenizer tok ( " \t" );
    while ( true )
      {
	LConfigObject obj;
	luint32 i;

	LString _line = stream.gets ();

	LTRACE("LConfigReader_TCL", "Pure line: '" + _line + "'" );

	LString line;
	for ( i=0; (i<_line.length()) && (_line[i]!='#'); i++ )
	  {
	    line += _line[i];
	  }

	LTRACE("LConfigReader_TCL", "Uncommented line: '" + line + "'" );	

	tok.tokenize ( line );
	try {
	  LTable<LString> const & ref = m_reference.get(tok.word(0) );
	  for ( i = 0; i<MIN(tok.numWords(),ref.getSize()+1); i++ )
	    {
	      LString item = tok.word(i).deSave();	    
	      if ( i )
		{
		  LTRACE("LConfigReader_TCL", "Set Data: '" + ref[i-1] + "' = '" + item + "'" );
		  obj.set ( ref[i-1], item );
		}
	      else
		{
		  LTRACE("LConfigReader_TCL", "Set Type: '" + item + "'" );	
		  obj.setType ( item );
		}
	    } // for
	  if ( i )
	    config.addObject ( obj );
	}
	catch ( ... )
	  {
	    //FIXME: don't catch ...	    
	  }
      } // while
  }
  catch ( LEndOfStreamException & e )
    {
    }

}

