/* ****************************************************************************
   |      File: lconfparser.cc
   |      Contains: base class LConfigParser, class LDefaultConfigParser
   |      Created: 18.03.1998
   |      Last Changed: 18.03.1998
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |
   ************************************************************************* */

/* ************************************************************************* *
 * Change log:                                                               *
 * -------------                                                             *
 *                                                                           *
 *   -> [v0.00] - 18.03.1998 - Created file and started coding               * 
 *   -> [v0.00] - 03.04.1999 - Fixed bug in LDefaultParser::parseObject      *
 *                             to be able to read comment chars with \       *
 * ************************************************************************* */


// **** LCL Includes

#include <lcl.h>
#include <lcl/lconfparser.h>
#include <lcl/lfstream.h>
#include <lcl/lutils.h>
#include <lcl/ltrace.h>
#include <lcl/llogsystem.h>

#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

LVCS_INIT(LConfigParser);
LVCS_INIT(LDefaultConfigParser);
LVCS_INIT(LCGIConfigParser);

void LConfigParser::parserError ( LString err, LString desc )
{
  NAMESPACE_STD(cout) << "\n";
  NAMESPACE_STD(cout) << "parser Error: " << err << "\n";
  NAMESPACE_STD(cout) << desc << "\n";
  NAMESPACE_STD(cout) << "\n";
  //  throw;
  exit(0);
}

void LConfigParser::parserError ( LString err, LString desc, int line )
{
  NAMESPACE_STD(cout) << "\n";
  NAMESPACE_STD(cout) << "parser Error @ line " << line << ": " << err << "\n";
  NAMESPACE_STD(cout) << desc << "\n";
  NAMESPACE_STD(cout) << "\n";
  //  throw;
  exit(0);
}


/* 
 * *********************************************************************
 * *  LDefaultConfigParser
 * *
 * *    A config parser for the default LCL Conf files
 * *
 * *  Config level: Complex
 * *
 * *********************************************************************
 */


LDefaultConfigParser::LDefaultConfigParser ( LString const & fileName, LString const & comments, LString const & _quoteChars ) :
  fileline ( 0 ),
  myInFile ( fileName, LFDManager::read ),
  myInStream ( myInFile ),
  instream ( myInStream ),
  commentChars ( comments ),
  quoteChars ( _quoteChars ),
  open_br ( '{' ),
  close_br ( '}' ),
  delim_1 ( ':' ),
  delim_2 ( ';' ),
  escape ( '\\' ),
  system_ch ( '!' ) 
{
  LTRACE_FUNC ( "LDefaultConfigParser", "LDefaultConfigParser ( LString const & )" );

  if ( !myInFile.isValid() )
    LLOG ( LLogSystem::ERR, "Could not open config: " + fileName );
}

LDefaultConfigParser::LDefaultConfigParser ( LIStream & _instream, LString const & comments, LString const & _quoteChars ) :
  fileline ( 0 ),
  instream ( _instream ),
  commentChars ( comments ),
  quoteChars ( _quoteChars ),
  open_br ( '{' ),
  close_br ( '}' ),
  delim_1 ( ':' ),
  delim_2 ( ';' ),
  escape ( '\\' ),
  system_ch ( '!' )
{
  LTRACE_FUNC ( "LDefaultConfigParser", "LDefaultConfigParser ( LIStream &  )" );
}

LDefaultConfigParser::~LDefaultConfigParser ()
{
}

void LDefaultConfigParser::internalCommand ( LConfigSystem * sys, LString cmd )
{
  LTRACE_FUNC ( "LDefaultConfigParser", "internalCommand" );

  LString fname;
  cmd.setDelims ( " \t" );
  cmd.setQMarks ( "\"");

  if ( cmd.word(0).toLower() == "setchar" ) {
    LString what = cmd.word(1).toLower ();
    char c = cmd.word(2).firstChar();

    if ( c == 0 )
      return;

    if ( ( c == open_br ) || 
	 ( c == close_br ) ||
	 ( c == escape ) ||
	 ( c == delim_1 ) ||
	 ( c == delim_2 ) ||
	 ( c == system_ch )
	 )
      {
	LTRACE ( "LDefaultConfigParser", "'" + LString(c) + "' is already used and will be ignored!" );
	LLOG ( LLogSystem::ERR, "'" + LString(c) + "' is already used and will be ignored!" );
	return;
      }

    LTRACE ( "LDefaultConfigParser", "Trying to set " + what + " to " + LString(c ) );

    if ( what == "open" )
      {
	open_br = c;
      }
    else if ( what == "close" )
      {
	close_br = c;
      }
    else if ( what == "escape" )
      {      
	escape = c;
      }
    else if ( what == "delim" )
      {
	delim_1 = c;
      }
    else if ( what == "end" )
      {
	delim_2 = c;
      }
    else if ( what == "system" )
      {
	system_ch = c;
      }
    else
      {
	LTRACE ( "LDefaulfConfigParser", "Unkown replacement: " + what );
	return;
      }
  }

  if ( cmd.word(0).toLower() == "include" ) {
    LTRACE ( "LDefaultConfigParser", "Include command: " + cmd.word(1) );
    fname = cmd.word(1);
    LTRACE ( "LDefaultConfigParser", "Include command: " + fname );
    fname.removeLeading ("\"");
    LTRACE ( "LDefaultConfigParser", "Include command: " + fname );
    fname.removeTrailing ( "\"\n\r" );
    LTRACE ( "LDefaultConfigParser", "Include command: " + fname );
    fname = fname.replaceWithEnvs ();

    LTRACE ( "LDefaultConfigParser", "Including file: " + fname );

    #ifdef __NUTC__

    if ( access ( Lnutc2win32(fname), R_OK ) == -1 ) {
      parserError ( "Could not include file \"" + Lnutc2win32(fname) + "\"", strerror(errno), fileline );
    }
    LDefaultConfigParser * subParser = new LDefaultConfigParser ( Lnutc2win32(fname), commentChars, quoteChars );

    #else

    if ( access ( fname, R_OK ) == -1 ) {
      parserError ( "Could not include file \"" + fname + "\"", strerror(errno), fileline );
    }
    LDefaultConfigParser * subParser = new LDefaultConfigParser ( fname, commentChars, quoteChars );

    #endif
   
    subParser->parseMain ( sys );
    delete subParser;
  }

}

void LDefaultConfigParser::parseHeader ( LConfigSystem * sys )
{
}

void LDefaultConfigParser::parseMain ( LConfigSystem * sys )
{
  LTRACE_FUNC ( "LDefaultConfigParser", "parseMain" );

  LString objType;
  LString cmdStr;
  bool parseCmd = false;
  LConfigObject confObj;
  char c;
  
  if ( !instream.eos() ) {

    try {
    while ( 1 ) {
      instream >> c;
      
      if ( commentChars.contains(c) ) {
	readToEOL();
      } else {
	
	if ( c == system_ch )
	  {
	    if ( parseCmd )	      
	      cmdStr += c;
	    else
	      parseCmd = true;
	  }
	
	else if ( c == '\r' )
	  ;
	else if ( c == '\t' )
	  ;
	else if ( c == '\n' )
	  {
	    fileline++;
	    if ( parseCmd ) {
	      cmdStr.removeLeading ( '!' );
	      internalCommand ( sys, cmdStr );
	      parseCmd = false;
	      cmdStr = "";
	    }
	  }
	else if ( c == ' ' )
	  {
	    if ( parseCmd )
	      cmdStr += " ";
	  }
	else if ( c == open_br )
	  {
	    if ( parseCmd )
	      {
		cmdStr += c;
	      }
	    else 
	      {
		confObj = LConfigObject ( objType );
		confObj.clear ();
		parseObject ( confObj );
		sys->addObject ( confObj );
		objType = "";
	      }
	  }
	else
	  {
	    if ( parseCmd ) {
	      cmdStr += c;
	    } else {
	      objType += c;
	    }
	  }

	/*
	switch (c) {
	case '!':
	  parseCmd = true;
	  break;

	case '\r':
	case '\t':  // Fixed a bug where tab was added to object type
	  break;
	  
	case '\n':
	  fileline++;
	  if ( parseCmd ) {
	    cmdStr.removeLeading ( '!' );
	    internalCommand ( sys, cmdStr );
	    parseCmd = false;
	    cmdStr = "";
	  }
	  break;
	  
	case ' ':
	  if ( parseCmd )
	    cmdStr += " ";
	  break;
	  
	case '{':
	  //cout << "Create obj type: " << objType << "\n";
	  confObj = LConfigObject ( objType );
	  confObj.clear ();
	  parseObject ( confObj );
	  sys->addObject ( confObj );
	  objType = "";
	  break;
	  
	default:
	  if ( parseCmd ) {
	    cmdStr += c;
	  } else {
	    objType += c;
	  }
	  break;
	  } */ // replaced by if-else 
	// end-switch
	
      } // end-if comment
      //} // end-if feof
      
    } // end-while
    } // end try
    catch ( ... ) {
    }
    
    if ( parseCmd ) {
      cmdStr.removeLeading ( '!' );
      internalCommand ( sys, cmdStr );
      parseCmd = false;
      cmdStr = "";
    }
    
  } // if file open
}

void LDefaultConfigParser::parseObject ( LConfigObject & obj )
{
  LTRACE_FUNC ( "LDefaultConfigParser", "parseObject" );

  char c;
  int isExit=FALSE;
  LString key, value;
  LString *str = &key;
  bool noCtl = false;
  bool noAnyCtl = false;

  try {
  while ( !isExit ) {
    instream >> c;

    if ( commentChars.contains(c) ) {
      if ( noCtl || noAnyCtl ) {
	(*str) += c;
	noCtl = false;
      } else 
	readToEOL();

    } else if ( quoteChars.contains(c) ) {
      if ( noCtl ) {
	(*str) += c;	  
	noCtl = false;
      } else {
	noAnyCtl = !noAnyCtl;
      }

    } else {

      if ( c == '\r' )
	;
      else if ( c == '\n' )
	{
	  fileline++;
	  (*str) += c;
	  noCtl = false;
	}
      else if ( c == '\t' )
	{
	  if ( noAnyCtl )
	    (*str) += c;
	}
      else if ( c == escape )
	{
	  if ( noCtl ) {
	    (*str) += c;
	    noCtl = false;
	  } else {
	    noCtl = true;
	  } 
	}
      else if ( c == delim_1 )
	{
	  if (noCtl || noAnyCtl ) {
	    (*str) += c;	
	    noCtl = false;
	  } else {
	    if ( str == &value )
	      parserError ( "Syntax error", "Possibly forgotten ->;<-", fileline );
	    str = &value;
	    
	    noCtl = false;
	  }
	}
      else if ( c == delim_2 )
	{
	  if ( noCtl || noAnyCtl ) {
	    (*str) += c;
	    noCtl = false;
	  } else {
	    
	    key.removeLeading(" \n\t");
	    key.removeTrailing(" \n\t");
	    
	    obj.set(key, value);
	    str = &key;
	    key = value = "";
	    
	    noCtl = false;
	  }
	}
      else if ( c == close_br )
	{
	  if ( noCtl || noAnyCtl ) {
	    (*str) += c;
	  } else {

	    isExit = 1;

	    noCtl = false;
	  }
	}
      else 
	{
	  (*str) += c;
	  noCtl = false;
	}
	  
      /*
      switch ( c ) {
      case '\r':
	break;
      case '\n':
	fileline++;
	  (*str) += c;
	  noCtl = false;
	break;
      case '\t':
	if ( noAnyCtl )
	  (*str) += c;
	break;

      case '\\':
	if ( noCtl ) {
	  (*str) += c;
	  noCtl = false;
	} else {
	  noCtl = true;
	}
	break;

	//      case '\'':
	//	break;

      case ':':
	if (noCtl || noAnyCtl ) {
	  (*str) += c;	
	  noCtl = false;
	} else {
	  if ( str == &value )
	    parserError ( "Syntax error", "Possibly forgotten ->;<-", fileline );
	  str = &value;
	  
	  noCtl = false;
	}
	break;

      case ';':	
	if ( noCtl || noAnyCtl ) {
	  (*str) += c;
	  noCtl = false;
	} else {

	  key.removeLeading(" \n\t");
	  key.removeTrailing(" \n\t");

	  obj.set(key, value);
	  str = &key;
	  key = value = "";

	  noCtl = false;
	}
	break;

      case '}':
	if ( noCtl || noAnyCtl ) {
	  (*str) += c;
	} else {

	  isExit = 1;

	  noCtl = false;
	}
	break;
      
      default:	
	(*str) += c;
	noCtl = false;
	break;

      } // end-switch
      */ // replaced by if-else


    } // end if comment 
  } // end while 

  } // end try
  catch ( ... ) {
  }

}

void LDefaultConfigParser::readToEOL ()
{
  LTRACE_FUNC ( "LDefaultConfigParser", "readToEOL" );

  //  while ((!feof(fp)) && (fgetc(fp)!='\n') ) {
  //  }
  try {
    char c = 0;
    while ( c != '\n' ) {
      instream >> c;
    }
  }
  catch ( ... ) {
  }
  fileline++;
}

/* 
 * *********************************************************************
 * *  LCGIConfigParser
 * *
 * *    A config parser for CGI-Queries
 * *
 * *  Config level: Simple
 * *
 * *********************************************************************
 */

LCGIConfigParser::LCGIConfigParser ()
{
  LString method = LString::fromEnv("REQUEST_METHOD").toUpper();

  if (  method == "POST" ) {
    char c;
    while (!NAMESPACE_STD(cin).eof()) {     
      NAMESPACE_STD(cin) >> c;
      if ( !NAMESPACE_STD(cin).eof() ) {
	query += c;
      }
    }
  } else if ( method == "GET" ) {
    query = LString::fromEnv ("QUERY_STRING");
  }

    query.setDelims ( "&" );

}

LCGIConfigParser::LCGIConfigParser ( LString request )
{
  query = request;
  query.setDelims ( "&" );
}

void LCGIConfigParser::parseHeader ( LConfigSystem * sys )
{
}

void LCGIConfigParser::parseMain ( LConfigSystem * sys )
{
  if ( query != "" ) {

    int i, cnt = query.numWords();
    LConfigObject obj ( "query" ) ;
    
    for (i=0;i<cnt;i++) {
      parseObject ( obj, query.word(i) );
    }
    sys->addObject ( obj );
  }
}

void LCGIConfigParser::parseObject ( LConfigObject & obj, LString str )
{
  str.setDelims ( "=" );

  //  cout << "Word #1: " << str.word(0) << "<BR>\n";

  if ( str.numWords() != 2 )
    return;


  obj.set ( str.word(0), str.word(1) );

}





