
#include <lcl.h>
/*
#include <unistd.h>
#include <errno.h>
*/

#include <iostream.h>
#include <fcntl.h>

#include <lcl/lfstream.h>
#include <lcl/lutils.h>
#include <lcl/ltrace.h>

// **********************************************
// Implementation of a saveread function ...
// It savely reads toread amount of bytes although it
// might be interrupted by signals. It returns
// 0 on success, -1 on error, -2 on EOF
// The amount of data read is stored in *cnt
// **********************************************

int saveread ( LFileDesc & fd, void * buffer, unsigned long toread, unsigned long * cnt )
{
  int erc;
  unsigned long readcnt, num; 
  char * tmpbuffer = (char*)buffer;

  for ( readcnt = 0; readcnt < toread; readcnt+=num ) {
    num = fd.read ( tmpbuffer+readcnt, toread-readcnt );
    erc = getLastError();

    *cnt = readcnt;

    if ( num < 0 ) {
      return -1;
    }
    if ( num == 0 ) {
      return -2; /* EOF */
    }

  }

  *cnt = readcnt;
  return 0;
}

/*
LFOStream::LFOStream ( LFileDescriptor _handle ) :
  handle ( new LFileDesc(_handle) )
{
}

LFIStream::LFIStream ( LFileDescriptor _handle ) :
  handle ( new LFileDesc(_handle) )
{
}
*/

LFOStream::LFOStream ( LFileDesc const & _handle ) :
  handle ( _handle )
{
}

LFIStream::LFIStream ( LFileDesc const & _handle ) :
  handle ( _handle )
{
}

LFIStream::LFIStream ( )
{
}

LFOStream::LFOStream ( )
{
}

int LFIStream::read ( char & c )
{
  int rc = handle.read ( &c, sizeof ( char ) );

  if ( rc == 0 )
    {
      LTRACE ( "LFIStream", "Read end of stream" );
      throw LEndOfStreamException ( *this );
    }
  if ( rc == -1 )
    {
      LTRACE ( "LFIStream", "Error while reading" );
      throw LErrorStreamException ( getLastError() );
    }

  return rc;
}

int LFIStream::read ( void * _ptr, unsigned long size )
{
  unsigned long cnt;
  int rc = ::saveread ( handle, _ptr, size, &cnt);

  switch ( rc ) {
  case 0:
    return cnt;

  case -1:
    LTRACE ( "LFIStream", "Error while reading" );
    throw LErrorStreamException ( getLastError() );

  case -2:
    LTRACE ( "LFIStream", "Read end of stream" );
    throw LEndOfStreamException ( *this, cnt );
  }

  return -1;
}


LFIStream::~LFIStream ()
{
}


/*
bool LFIStream::mayRead ()
{
  bool rc = true;
  bool flag = getBlock ();

  setBlock ( false );

  if ( handle.read ( NULL, 0 ) == -1 )
    rc = false;  
  
  setBlock ( flag );

  return rc;
}

void LFIStream::setBlock ( bool flag )
{
  long fl = handle.fcntl ( F_GETFL );

  if ( flag )
    fl = fl & (~O_NONBLOCK);
  else
    fl = fl | O_NONBLOCK;

  handle.fcntl ( F_SETFL, fl );
}

bool LFIStream::getBlock ( )
{
  return ! ( handle.fcntl ( F_GETFL ) & O_NONBLOCK );
}


bool LFOStream::mayWrite ( )
{
  bool rc = true;
  bool flag = getBlock ();
  setBlock ( false );

  if ( handle.write ( NULL, 0 ) == -1 )
    rc = false;  
  
  setBlock ( flag );

  return rc;
}

void LFOStream::setBlock ( bool flag )
{
  int fl = handle.fcntl ( F_GETFL );

  if ( flag )
    fl = fl & (~O_NONBLOCK);
  else
    fl = fl | O_NONBLOCK;

  handle.fcntl ( F_SETFL, fl );
}

bool LFOStream::getBlock ( )
{
  return ! ( handle.fcntl ( F_GETFL ) & O_NONBLOCK );
}
*/

int LFOStream::write ( char c )
{
  return handle.write ( &c, sizeof(char) );
}

int LFOStream::write ( void const * _ptr, unsigned long len  )
{
  return handle.write ( _ptr, len );
}


LFOStream::~LFOStream ()
{
}


/*
LFIOStream::LFIOStream ( LFileDescriptor _handle ) :
  LFIStream ( _handle ),
  LFOStream ( _handle ),
  LIOStream (),
  LIStream (),
  LOStream ()
{
}
*/

LFIOStream::LFIOStream ( LFileDesc const & _handle ) :
  LFIStream ( _handle ),
  LFOStream ( _handle ),
  LIOStream (),
  LIStream (),
  LOStream ()
{
}

LFIOStream::LFIOStream ( )
{
}

LFIOStream::~LFIOStream ()
{
}
