#include <lcl/limage.ppm.h>

#include <lcl/lstringtokenizer.h>
#include <lcl/ltrace.h>

#include <lcl/lstreamproxy.h>

LImageWriter_PPM::LImageWriter_PPM ( ModeFlag _mode ) :
  mode ( _mode )
{
}

bool LImageWriter_PPM::write_as_rgb_ascii ( LImage const & image, LOStream & stream ) const
{
  stream << "P3" << endl;
  stream << "# Created by LCL as RGB ascii!" << endl;

  stream << image.getWidth() << " " << image.getHeight() << endl;
  stream << "255" << endl;

  LColor color;

  unsigned long i, j;
  for ( j = 0 ; j<image.getHeight(); j++ ) {
    for ( i = 0; i<image.getWidth(); i++ ) {
      color = image.getPixel ( i, j );
      stream << (long)color.getRedAsChar() << endl;
      stream << (long)color.getGreenAsChar() << endl;
      stream << (long)color.getBlueAsChar() << endl;
    }
  }
  return true;
}

bool LImageWriter_PPM::write_as_rgb_binary ( LImage const & image, LOStream & stream ) const
{
  stream << "P6" << endl;
  stream << "# Created by LCL as RGB binary!" << endl;

  stream << image.getWidth() << " " << image.getHeight() << endl;
  stream << "255" << endl;

  LColor color;

  int cnt = 0;
  unsigned long i, j;
  for ( j = 0 ; j<image.getHeight(); j++ ) {
    for ( i = 0; i<image.getWidth(); i++ ) {
      cnt++;
      color = image.getPixel ( i, j );
      stream << ( char)color.getRedAsChar();
      stream << ( char)color.getGreenAsChar();
      stream << ( char)color.getBlueAsChar();
      if ( cnt > 60 ) {
	//	stream << endl << endl;
	cnt = 0;
      }
    }
  }
  return true;
}

bool LImageWriter_PPM::write_as_gray_ascii ( LImage const & image, LOStream & stream ) const
{
  stream << "P2" << endl;
  stream << "# Created by LCL as gray ascii!" << endl;

  stream << image.getWidth() << " " << image.getHeight() << endl;
  stream << "255" << endl;

  LColor color;

  unsigned long i, j;
  for ( j = 0 ; j<image.getHeight(); j++ ) {
    for ( i = 0; i<image.getWidth(); i++ ) {
      color = image.getPixel ( i, j );
      stream << (long)color.asGray().getRedAsChar() << endl;
    }
  }
  return true;
}

bool LImageWriter_PPM::write_as_gray_binary ( LImage const & image, LOStream & stream ) const
{
  stream << "P5" << endl;
  stream << "# Created by LCL as gray binary!" << endl;

  stream << image.getWidth() << " " << image.getHeight() << endl;
  stream << "255" << endl;

  LColor color;

  int cnt = 0;
  unsigned long i, j;
  for ( j = 0 ; j<image.getHeight(); j++ ) {
    for ( i = 0; i<image.getWidth(); i++ ) {
      color = image.getPixel ( i, j );
      stream << (char)color.asGray().getRedAsChar();
      if ( cnt > 60 ) {
	//	stream << endl << endl;
	cnt = 0;
      }
    }
  }
  return true;
}

bool LImageWriter_PPM::impl_write ( LImage const & image, LOStream & stream ) const
{

  long bpp = image.getBitsPerPixel();

  switch ( mode ) {
  case ASCII:

    switch ( bpp ) {
    case 1:
      return write_as_gray_ascii ( image, stream );

    case 4:
    case 8:
      if ( image.getColorMap().getSize() ) 
	return write_as_rgb_ascii ( image, stream );
      else
	return write_as_gray_ascii ( image, stream );

    case 16:
    case 24:
    case 32:
      return write_as_rgb_ascii ( image, stream );

    default:
      return false;
    }

  case BINARY:

    switch ( bpp ) {
    case 1:
      return write_as_gray_binary ( image, stream );

    case 4:
    case 8:
      if ( image.getColorMap().getSize() ) 
	return write_as_rgb_binary ( image, stream );
      else
	return write_as_gray_binary ( image, stream );

    case 16:
    case 24:
    case 32:
      return write_as_rgb_binary ( image, stream );

    default:
      return false;
    }
  default:
    return false;

  };
}

bool LImageReader_PPM::read_as_rgb_ascii ( LImage & image, LIStream & stream ) const
{
  LString line;
  LString size_x, size_y, max_val;
  unsigned long cx, cy;

  image.setBitsPerPixel ( 24 );

  while ( size_x.length() == 0 ) {
    line = stream.gets ();
    if ( (line.firstChar() != '#') && (line.numWords()) ) {
      size_x = line.word(0);
      size_y = line.word(1);
    }
  }

  cx = size_x;
  cy = size_y;
  image.setSize ( cx, cy );

  while ( max_val.length() == 0 ) {
    line = stream.gets ();
    if ( (line.firstChar() != '#') && (line.numWords()) ) {
      max_val = line.word(0);
    }
  }  

  long num_vals = 3 * cx * cy;
  long cnt_vals = 0;

  char * bits = (char*)malloc ( num_vals );

  LStringTokenizer tok;

  unsigned long i;

  while ( cnt_vals < num_vals ) {
    line = stream.gets ();

    tok.tokenize ( line );

    if ( tok.word(0).firstChar() == '#' )
      continue;

    for ( i = 0; i<tok.numWords(); i++ ) {

      bits[cnt_vals] = (unsigned char)(int)tok.word(i);
      cnt_vals++;

    }
  }
  image.setBits ( LBuffer(num_vals,bits) );
  free ( bits );
  return true;
}

bool LImageReader_PPM::read_as_gray_ascii ( LImage & image, LIStream & stream ) const
{
  LString line;
  LString size_x, size_y, max_val;
  unsigned long cx, cy;

  image.setBitsPerPixel ( 8 );

  while ( size_x.length() == 0 ) {
    line = stream.gets ();
    if ( (line.firstChar() != '#') && (line.numWords()) ) {
      size_x = line.word(0);
      size_y = line.word(1);
    }
  }

  cx = size_x;
  cy = size_y;
  image.setSize ( cx, cy );

  while ( max_val.length() == 0 ) {
    line = stream.gets ();
    if ( (line.firstChar() != '#') && (line.numWords()) ) {
      max_val = line.word(0);
    }
  }  

  long num_vals = cx * cy;
  long cnt_vals = 0;

  char * bits = (char*)malloc ( num_vals );

  LStringTokenizer tok;

  unsigned long i;

  while ( cnt_vals < num_vals ) {
    line = stream.gets ();

    tok.tokenize ( line );

    if ( tok.word(0).firstChar() == '#' )
      continue;

    for ( i = 0; i<tok.numWords(); i++ ) {

      bits[cnt_vals] = (unsigned char)(int)tok.word(i);
      cnt_vals++;

    }
  }
  image.setBits ( LBuffer(num_vals,bits) );
  free ( bits );
  return true;
}


bool LImageReader_PPM::read_as_rgb_binary ( LImage & image, LIStream & stream ) const
{
  LString line;
  LString size_x, size_y, max_val;
  unsigned long cx, cy;

  image.setBitsPerPixel ( 24 );

  while ( size_x.length() == 0 ) {
    line = stream.gets ();
    if ( (line.firstChar() != '#') && (line.numWords()) ) {
      size_x = line.word(0);
      size_y = line.word(1);
    }
  }

  cx = size_x;
  cy = size_y;
  image.setSize ( cx, cy );

  while ( max_val.length() == 0 ) {
    line = stream.gets ();
    if ( (line.firstChar() != '#') && (line.numWords()) ) {
      max_val = line.word(0);
    }
  }  

  long num_vals = 3 * cx * cy;
  long cnt_vals = 0;

  char * bits = (char*)malloc ( num_vals );

  char c;

  while ( cnt_vals < num_vals ) {

    stream >> c;
    
    bits[cnt_vals] = c;
    cnt_vals++;

  }
  bits[cnt_vals] = c;
  image.setBits ( LBuffer(num_vals,bits) );
  free ( bits );
  return true;
}


bool LImageReader_PPM::read_as_gray_binary ( LImage & image, LIStream & stream ) const
{
  LString line;
  LString size_x, size_y, max_val;
  unsigned long cx, cy;

  image.setBitsPerPixel ( 8 );

  while ( size_x.length() == 0 ) {
    line = stream.gets ();
    if ( (line.firstChar() != '#') && (line.numWords()) ) {
      size_x = line.word(0);
      size_y = line.word(1);
    }
  }

  cx = size_x;
  cy = size_y;
  image.setSize ( cx, cy );

  while ( max_val.length() == 0 ) {
    line = stream.gets ();
    if ( (line.firstChar() != '#') && (line.numWords()) ) {
      max_val = line.word(0);
    }
  }  

  long num_vals = cx * cy;
  long cnt_vals = 0;

  char * bits = (char*)malloc ( num_vals );

  char c;

  while ( cnt_vals < num_vals ) {

    stream >> c;
    
    bits[cnt_vals] = c;
    cnt_vals++;

  }
  bits[cnt_vals] = c;
  image.setBits ( LBuffer(num_vals,bits) );
  free ( bits );
  return true;
}


bool LImageReader_PPM::impl_read ( LImage  & image, LIStream & stream ) const
{
  LString line;

  LString mode;  

  while ( true ) {
    line = stream.gets ();
    if ( line.firstChar() != '#' ) {
      if ( mode.length() == 0 ) {
	mode = line.word(0);

	if ( mode == "P3" ) {
	  return read_as_rgb_ascii ( image, stream );

	} else if ( mode == "P6" ) {
	  return read_as_rgb_binary ( image, stream );

	} else if ( mode == "P2" ) {
	  return read_as_gray_ascii ( image, stream );

	} else if ( mode == "P5" ) {
	  return read_as_gray_binary ( image, stream );

	} else
	  return false;

      }
    }
  }

}
