#include <lcl/linterpreter_commands.h>

#include <lcl/ltrace.h>
#include <lcl/lparser.h>

/*
 * used by:
 *   ctime
 */
#include <lcl/ldatetime.h>

/*
 * used by:
 *   round
 */
#include <lcl/lmath.h>

#include <lcl/llogsystem.h>
#include <lcl/ltrace.h>


/*!
 * \namespace LCommands
 * \brief Basic LInterpreter commands
 */
namespace LCommands {

  /*!
   * \namespace Base
   * \brief Basic LInterpreter commands of group Base
   */
  namespace Base {

    /*!
     * \return the value rounded
     * \param value the value to be rounded
     */

    _BASE_COMMAND_SIMPLE_1 ( add, ((double)args[0] + (double)args[1]) );
    _BASE_COMMAND_SIMPLE_1 ( sub, ((double)args[0] - (double)args[1]) );
    _BASE_COMMAND_SIMPLE_1 ( mul, ((double)args[0] * (double)args[1]) );
    _BASE_COMMAND_SIMPLE_1 ( div, ((double)args[0] / (double)args[1]) );
    _BASE_COMMAND_SIMPLE_1_RES ( and, ((luint32)args[0] & (luint32)args[1]) );
    _BASE_COMMAND_SIMPLE_1_RES ( or, ((luint32)args[0] | (luint32)args[1]) );
    _BASE_COMMAND_SIMPLE_1_RES ( xor, ((luint32)args[0] ^ (luint32)args[1]) );
    _BASE_COMMAND_SIMPLE_1 ( inv, ~ (luint32)args[0] );

    _BASE_COMMAND_SIMPLE_1_RES ( int, (lint32)args[0] );
    _BASE_COMMAND_SIMPLE_1_RES ( long, (lint32)args[0] );
    _BASE_COMMAND_SIMPLE_1_RES ( short, (lint16)args[0] );
    _BASE_COMMAND_SIMPLE_1_RES ( unsigned_short, (luint16)args[0] );
    _BASE_COMMAND_SIMPLE_1_RES ( unsigned_int, (luint32)args[0] );
    _BASE_COMMAND_SIMPLE_1_RES ( unsigned_long, (luint32)args[0] );
    _BASE_COMMAND_SIMPLE_1_RES ( double, (double)args[0] );
    _BASE_COMMAND_SIMPLE_1_RES ( float, (float)(double)args[0] );
    _BASE_COMMAND_SIMPLE_1_RES ( char, (lint8)args[0] );
    _BASE_COMMAND_SIMPLE_1_RES ( unsigned_char, (luint8)args[0] );

    _BASE_COMMAND_SIMPLE_1 ( round, lcl_round(args[0]) );

    _BASE_COMMAND_SIMPLE_2 ( flush, NAMESPACE_STD(cout).flush() );
    _BASE_COMMAND_SIMPLE_2_RES ( sleep, lcl_sleep((luint32)args[0]) );

    _BASE_COMMAND_CONSTRUCTOR (inc)
      _BASE_COMMAND_BODY_START (inc)
    {
      double i;
      LVariant v;
      if ( args.getSize() == 1 )
	{
	  i = 1;
	}
      else if ( args.getSize() > 1 )
	{
	  i = args[1];
	}
      LHash<LString,LVariant> & hash = context.getLocal ( "local_vars" );
      v = hash.get ( args[0], 0 );

      if ( v.isDouble() )
	{
	  v = ((double)v) + i;
	}
      else
	{
	  v = v.asInt32() + i;
	}
      hash.set ( args[0], v );
      return v;
    }
    _BASE_COMMAND_BODY_END(inc)

    _BASE_COMMAND_CONSTRUCTOR (dec)
      _BASE_COMMAND_BODY_START (dec)
    {
      double i;
      LVariant v;
      if ( args.getSize() == 1 )
	{
	  i = 1;
	}
      else if ( args.getSize() > 1 )
	{
	  i = args[1];
	}
      LHash<LString,LVariant> & hash = context.getLocal ( "local_vars" );
      v = hash.get ( args[0], 0 );

      if ( v.isDouble() )
	{
	  v = ((double)v) - i;
	}
      else
	{
	  v = v.asInt32() - i;
	}
      hash.set ( args[0], v );
      return v;
    }
    _BASE_COMMAND_BODY_END(dec)
    /*
    _BASE_COMMAND_CONSTRUCTOR (flush)
    _BASE_COMMAND_BODY_START (flush)
    {
      cout.flush();
      return "";
    }
    _BASE_COMMAND_BODY_END (flush)
    */

    _BASE_COMMAND_CONSTRUCTOR (equal)
    _BASE_COMMAND_BODY_START (equal)
    {
      if ( args[0] == args[1] )
	return true;
      else
	return false;
    }
    _BASE_COMMAND_BODY_END (equal)

    _BASE_COMMAND_CONSTRUCTOR (not)
    _BASE_COMMAND_BODY_START (not)
    {
      if ( args[0].asBool() )
	return false;
      else
	return true;
    }
    _BASE_COMMAND_BODY_END (not)

    _BASE_COMMAND_CONSTRUCTOR (echo)
    _BASE_COMMAND_BODY_START (echo)
    {
      int i;
      for ( i = 0; i<args.getSize(); i++ )
	{
	  lout << args[i];
	}
      return "";
    }
    _BASE_COMMAND_BODY_END (echo)

    _BASE_COMMAND_CONSTRUCTOR (set)
    _BASE_COMMAND_BODY_START (set)
    {
      LHash<LString,LVariant> & hash = context.getLocal ( "local_vars" );
      hash.set ( args[0], args[1] );
      return args[1];
    }
    _BASE_COMMAND_BODY_END (set)

    _BASE_COMMAND_CONSTRUCTOR (get)
    _BASE_COMMAND_BODY_START (get)
    {
      LHash<LString,LVariant> & hash = context.getLocal ( "local_vars" );
      return hash.get ( args[0], "" );
    }
    _BASE_COMMAND_BODY_END (get)

    _BASE_COMMAND_CONSTRUCTOR (is_set)
    _BASE_COMMAND_BODY_START (is_set)
    {
      LHash<LString,LVariant> & hash = context.getLocal ( "local_vars" );
      if ( hash.isSet ( args[0] ) )
	return true;
      else
	return false;	  
    }
    _BASE_COMMAND_BODY_END (is_set)

      // Local arguments

    _BASE_COMMAND_CONSTRUCTOR (set_arg)
    _BASE_COMMAND_BODY_START (set_arg)
    {
      LHash<LString,LVariant> & hash = context.getLocal ( "local_args" );
      hash.set ( (unsigned int)args[0], args[1] );
      return args[1];
    }
    _BASE_COMMAND_BODY_END (set_arg)

    _BASE_COMMAND_CONSTRUCTOR (get_arg)
    _BASE_COMMAND_BODY_START (get_arg)
    {
      LHash<LString,LVariant> & hash = context.getLocal ( "local_args" );
      return hash.get ( (unsigned int)args[0], "" );
    }
    _BASE_COMMAND_BODY_END (get_arg)

    _BASE_COMMAND_CONSTRUCTOR (arg_num)
    _BASE_COMMAND_BODY_START (arg_num)
    {
      LHash<LString,LVariant> & hash = context.getLocal ( "local_args" );
      return (luint32)hash.entries();
    }
    _BASE_COMMAND_BODY_END (arg_num)

    _BASE_COMMAND_CONSTRUCTOR (func)
    _BASE_COMMAND_BODY_START (func)
    {
      LString name = args[0].asString();
      LString code = args[1].asString();
      LParser_LSLL parser;
      context.registerCommand ( new LInterpreter::ScriptCommand ( name, parser.parse ( code ) ) );
      return name;
    }
    _BASE_COMMAND_BODY_END (func)

    _BASE_COMMAND_CONSTRUCTOR (unfunc)
    _BASE_COMMAND_BODY_START (unfunc)
    {
      LString name = args[0].asString();
      context.unregisterCommand ( name );
      return name;
    }
    _BASE_COMMAND_BODY_END (unfunc)

    LCL_EXPORT void registerAll ( LInterpreter & inter )
    {
      
      _BASE_COMMAND_REGISTER ( func );
      _BASE_COMMAND_REGISTER ( unfunc );

      _BASE_COMMAND_REGISTER ( equal );
      _BASE_COMMAND_REGISTER ( not );
      _BASE_COMMAND_REGISTER ( echo );
      _BASE_COMMAND_REGISTER ( flush );
      _BASE_COMMAND_REGISTER ( set );
      _BASE_COMMAND_REGISTER ( get );
      _BASE_COMMAND_REGISTER ( is_set );

      _BASE_COMMAND_REGISTER (get_arg);
      _BASE_COMMAND_REGISTER (set_arg);
      _BASE_COMMAND_REGISTER (arg_num);

      _BASE_COMMAND_REGISTER ( add );
      _BASE_COMMAND_REGISTER ( sub );
      _BASE_COMMAND_REGISTER ( mul );
      _BASE_COMMAND_REGISTER ( div );
      _BASE_COMMAND_REGISTER_RES ( and );
      _BASE_COMMAND_REGISTER_RES ( or );
      _BASE_COMMAND_REGISTER_RES ( xor );
      _BASE_COMMAND_REGISTER ( inv );
      _BASE_COMMAND_REGISTER ( inc );
      _BASE_COMMAND_REGISTER ( dec );

      _BASE_COMMAND_REGISTER ( round );
      _BASE_COMMAND_REGISTER_RES ( sleep );

      _BASE_COMMAND_REGISTER_RES ( int );
      _BASE_COMMAND_REGISTER_RES ( long );
      _BASE_COMMAND_REGISTER_RES ( short );
      _BASE_COMMAND_REGISTER_RES ( double );
      _BASE_COMMAND_REGISTER_RES ( float );
      _BASE_COMMAND_REGISTER_RES ( char );
      _BASE_COMMAND_REGISTER_RES ( unsigned_int );
      _BASE_COMMAND_REGISTER_RES ( unsigned_long );
      _BASE_COMMAND_REGISTER_RES ( unsigned_short );
      _BASE_COMMAND_REGISTER_RES ( unsigned_char );
    }

  }

  /*!
   * \namespace String
   * \brief Basic LInterpreter commands of group String
   */
  namespace String {

    cmd_strcat::cmd_strcat () :
      LInterpreter::Command ( "strcat" )
    {
    }
    
    LVariant cmd_strcat::exec ( LInterpreter::Context & context, LTable<LVariant> const & args )
    {
      LString ret;
      
      int i;
      for ( i = 0; i<args.getSize(); i++ )
	{
	  ret += args[i];
	}
      return ret;
    }

    _BASE_COMMAND_CONSTRUCTOR (strlist)
    _BASE_COMMAND_BODY_START (strlist)
    {
      LString ret;
      
      int i;
      for ( i = 0; i<args.getSize(); i++ )
	{
	  if ( i )
	    ret += " ";
	  ret += args[i];
	}
      return ret;
    }
    _BASE_COMMAND_BODY_END (strlist)


    _BASE_COMMAND_CONSTRUCTOR (trim)
    _BASE_COMMAND_BODY_START (trim)
    {
      LString ret, chars;

      LString str = args[0].asString();
      if ( args.getSize() > 1 )
	chars = args[1].asString();
      else
	chars = "\n\r \t";

      ret = str.removeLeading(chars).removeTrailing(chars);
      return ret;
    }
    _BASE_COMMAND_BODY_END (trim)

    _BASE_COMMAND_CONSTRUCTOR (lcl_log)
    _BASE_COMMAND_BODY_START (lcl_log)
    {
      if ( args[0].asString().toUpper() == "FATAL" ) {	
	LLOG ( LLogSystem::FATAL, args[1].asString() );
      } else if ( args[0].asString().toUpper() == "ERROR" ) {
	LLOG ( LLogSystem::ERR, args[1].asString() );
      } else if ( args[0].asString().toUpper() == "WARNING" ) {
	LLOG ( LLogSystem::WARNING, args[1].asString() );
      } else if ( args[0].asString().toUpper() == "INFO" ) {
	LLOG ( LLogSystem::INFO, args[1].asString() );
      } else if ( args[0].asString().toUpper() == "BULK" ) {
	LLOG ( LLogSystem::BULK, args[1].asString() );       
      } else {
	LLOG ( LLogSystem::BULK, args[1].asString() );
      }
      return true;
    }
    _BASE_COMMAND_BODY_END (lcl_log)

    _BASE_COMMAND_CONSTRUCTOR (lcl_trace)
    _BASE_COMMAND_BODY_START (lcl_trace)
      LTRACE ( args[0].asString(), args[1].asString() );
      return true;
    _BASE_COMMAND_BODY_END (lcl_trace)

    LCL_EXPORT void registerAll ( LInterpreter & inter )
    {
      _BASE_COMMAND_REGISTER(strcat);
      _BASE_COMMAND_REGISTER(strlist);
      _BASE_COMMAND_REGISTER(trim);
      _BASE_COMMAND_REGISTER(lcl_log);
      _BASE_COMMAND_REGISTER(lcl_trace);
    }

  } // namespace string

  /*!
   * \namespace Time
   * \brief Basic LInterpreter commands of group Time
   */
  namespace Time
  {

    _BASE_COMMAND_CONSTRUCTOR (ctime)
    _BASE_COMMAND_BODY_START ( ctime )
    {
      if ( args.getSize() == 0 )
	{
	  // Ctime with time(0)
	  return LDateTime::now().asString();
	}
      else if ( args.getSize() == 1 )
	{
	  // ctime with arg
	  return LDateTime((luint32)(args[0])).asString();
	}
      else
	{
	  // too many parameters
	  return "";
	}
    }    
    _BASE_COMMAND_BODY_END ( ctime );


    // ============>>>>>>>>>> time

    _BASE_COMMAND_CONSTRUCTOR (time)
    _BASE_COMMAND_BODY_START (time)
    {
      return (luint32)::time(0);
    }    
    _BASE_COMMAND_BODY_END ( time );


    LCL_EXPORT void registerAll ( LInterpreter & inter )
    {
      _BASE_COMMAND_REGISTER ( ctime );
      _BASE_COMMAND_REGISTER ( time );
    }

  } // namespace time

  #ifdef _WIN32
  /*!
   * \namespace Win32
   * \brief Basic LInterpreter commands of group Win32
   */


  //  namespace Win32 {

    /*
    LCL_EXPORT void registerAll ( LInterpreter & inter )
    {
      _BASE_COMMAND_REGISTER ( MessageBox );
    }
    */
    
  //  } //namespace Win32

  #endif // WIN32

} // namespace LCommands
