#include <lcl/lparser.h>

#include <lcl/lstringtokenizer.h>
#include <lcl/lstreamproxy.h>
#include <lcl/lregex.h>
#include <lcl/ltrace.h>

LVCS_INIT(LParser);
LVCS_INIT(LParser_LSLL);

LParser::Exception::Exception ( LString const & _reason ) :
  LBasicException ( __FILE__, __LINE__, "Parser Exception", LCL_ERROR_CODE("PAR", 0x00000001) ),
  reason ( _reason )
{
}

void LParser::Exception::log ( ) const
{
  LBasicException::log ();

  lerr << " >> LCL Parser Exception" << endl;
  lerr << "  >> Reason: " << reason << endl;
  lerr << flush;
}

LString LParser_LSLL::prepareWS ( LString const & str )
{
  LString newstr;
  unsigned long i;
  char c;
  bool inword = false;

  LString crlf ( "\n\r" );

  for ( i = 0; i<str.length(); i++ )
    {
      c = str[i];

      switch ( c )
	{
	case '#':
	  if ( inword )
	    {
	      newstr +=c;
	    }
	  else
	    {
	      while ( (i<str.length()) && (!crlf.contains(str[i])) )
		{
		  i++;
		}
	    }
	  break;
	case '(':
	case ')':
	  if ( inword )
	    newstr +=c;
	  else
	    {
	      newstr += " ";
	      newstr += c;
	      newstr += " ";
	    }
	  break;

	case '\"':
	case '\'':
	  inword = !inword;
	default:
	  newstr += c;
	  break;
	}

    }
  return newstr;
}

LVariant LParser_LSLL::createConst ( LString const & str )
{
  LRegEx exp ( "^[0-9]+$" );
  if ( exp.exec ( str ) )
    {
      LTRACE ( "LParser_LSLL", "Const: detected '" + str + "' as luint32");
      return (luint32)str;
    }

  exp.set ( "^-[0-9]+$" );
  if ( exp.exec ( str ) )
    {
      LTRACE ( "LParser_LSLL", "Const: detected '" + str + "' as lint32");
      return (lint32)str;
    }

  exp.set ( "^-?[0-9]+\\.[0-9]*$" );
  if ( exp.exec ( str ) )
    {
      LTRACE ( "LParser_LSLL", "Const: detected '" + str + "' as double");
      return (double)str;
    }

  if ( str.toUpper() == "TRUE" )
    {
      LTRACE ( "LParser_LSLL", "Const: detected '" + str + "' as bool(true)");
      return true;
    }

  if ( str.toUpper() == "FALSE" )
    {
      LTRACE ( "LParser_LSLL", "Const: detected '" + str + "' as bool(false)");
      return false;
    }
  
  LTRACE ( "LParser_LSLL", "Const: detected '" + str + "' as string");
  LString tmp = str;
  tmp.change ( "\\n", "\n" );
  tmp.change ( "\\r", "\r" );
  tmp.change ( "\\t", "\t" );
  return tmp;
}

LManagedPointer<LInterpreter::Sequence> LParser_LSLL::step_parse ( LStringTokenizer & tok, unsigned long & pos )
{
  LManagedPointer<LInterpreter::Sequence> seq;
  int i = 0;

  // check if we are in-range
  if ( pos >= tok.numWords() )
    throw Exception ( "Syntax Error" );

  while ( pos < tok.numWords() )
    {
      LTRACE ( "LParser_LSLL", "Word: " + tok.word(pos) );

      if ( tok.word(pos) == "(" )
	{
	  LTRACE ( "LParser_LSLL", "Starting new parse step" );
	  seq->addParameter ( step_parse ( tok, ++pos ) );
	}
      else if ( tok.word(pos) == ")" )
	{
	  LTRACE ( "LParser_LSLL", "finish command" );
	  return seq;
	}
      else
	{
	  if ( i > 0 )
	    {
	      LTRACE ( "LParser_LSLL", "adding const " + tok.word(pos) );
	      seq->addParameter ( new LInterpreter::Sequence_Const ( createConst ( tok.word(pos) ) ) );
	    }
	  else
	    {
	      LString name = tok.word(pos);
	      if ( name == "if" )
		{
		  LTRACE ( "LParser_LSLL", "setting if switch" );
		  seq = new LInterpreter::Sequence_If;
		}
	      else if ( name == "group" )
		{
		  LTRACE ( "LParser_LSLL", "setting new group" );
		  seq = new LInterpreter::Sequence_Group;
		}
	      else if ( name == "for" )
		{
		  LTRACE ( "LParser_LSLL", "setting for loop" );
		  seq = new LInterpreter::Sequence_For;
		}
	      else if ( name == "while" )
		{
		  LTRACE ( "LParser_LSLL", "setting while loop" );
		  seq = new LInterpreter::Sequence_While;
		}
	      else
		{
		  LTRACE ( "LParser_LSLL", "setting command " + tok.word(pos) );
		  seq = new LInterpreter::Sequence_Command ( tok.word(pos) );
		}
	    }
	}	  
      i++;
      pos++;
    }
  throw Exception ( "Syntax Error" );
  return NULL;
}

LManagedPointer<LInterpreter::Sequence> LParser_LSLL::parse ( LString const & command )
{
  LString cmd = command;
  LTRACE ( "LParser_LSLL", "CMD: " + cmd );

  cmd = prepareWS ( cmd );

  LTRACE ( "LParser_LSLL", "CMD(prepared): " + cmd );

  LStringTokenizer tok ( " \t\n\r", "\'" );
  tok.tokenize ( cmd );

  LManagedPointer<LInterpreter::Sequence> seq;

  unsigned long pos = 0;
  while ( pos < tok.numWords() )
    {
      if ( tok.word(pos) == "(" ) 
	{
	  return step_parse ( tok, ++pos );
	}
      else 
	throw Exception ( "Syntax Error" );

      pos++;
    }

  return seq;
}
