/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#include <lcl/lrect.h>

LRect::LRect () :
  m_x ( 0 ),
  m_y ( 0 ),
  m_width ( 0 ),
  m_height ( 0 )
{
}

LRect::LRect ( LRect const & p1 ) :
  m_x ( p1.m_x ),
  m_y ( p1.m_y ),
  m_width ( p1.m_width ),
  m_height ( p1.m_height )
{
}

LRect::LRect ( lint32 x, lint32 y, lint32 width, lint32 height )
{
  set ( x, y, width, height );
}

LRect::LRect ( LPoint const & leftUp, LPoint const & rightDown )
{
  set ( leftUp.x(), leftUp.y(), rightDown.x()-leftUp.x(), rightDown.y()-leftUp.y() );
}

LRect::~LRect ()
{
}

lint32 LRect::x () const
{
  return m_x;
}

lint32 LRect::y () const
{
  return m_y;
}

void LRect::x ( lint32 x )
{
  m_x = x;
}

void LRect::y ( lint32 y )
{
  m_y = y;
}

lint32 LRect::x2 () const
{
  return m_x+m_width;
}

lint32 LRect::y2 () const
{
  return m_y+m_height;
}

luint32 LRect::width () const
{
  return m_width;
}

luint32 LRect::height () const
{
  return m_height;
}

void LRect::width ( lint32 width )
{
  if ( width>=0 )
    {
      m_width = width;
    }
  else
    {
      m_x = m_x+width;
      m_width = -width;
    }
}

void LRect::height ( lint32 height )
{
  if ( height>=0 )
    {
      m_height = height;
    }
  else
    {
      m_y = m_y+height;
      m_height = -height;
    }
}

void LRect::setLeftUp ( lint32 _x, lint32 _y )
{
  x(_x);
  y(_y);
}

void LRect::setRightDown ( lint32 x, lint32 y )
{
  width(x-m_x);
  height(y-m_y);
}

void LRect::setLeftUp ( LPoint const & leftUp )
{
  setLeftUp ( leftUp.x(), leftUp.y() );
}

void LRect::setRightDown ( LPoint const & rightDown )
{
  setRightDown ( rightDown.x(), rightDown.y() );
}

LPoint LRect::leftUp ( ) const
{
  return LPoint ( m_x, m_y );
}

LPoint LRect::rightDown ( ) const
{
  return LPoint ( m_x+m_width, m_y+m_width );
}


void LRect::set ( lint32 x, lint32 y, lint32 width, lint32 height )
{
  if ( width>=0 )
    {
      m_x = x;
      m_width = width;      
    }
  else
    {
      m_x = x+width;
      m_width = -width;
    }
  if ( height>=0 )
    {
      m_y = y;
      m_height = height;
    }
  else
    {
      m_y = y+height;
    }
}

void LRect::set ( LPoint const & leftUp, LPoint const & rightDown )
{
  set ( leftUp.x(), leftUp.y(), rightDown.x()-leftUp.x(), rightDown.y()-leftUp.y() );
}

void LRect::set ( LRect const & p1 )
{
  m_x = p1.m_x;
  m_y = p1.m_y;
  m_width = p1.m_width;
  m_height = p1.m_height;
}

LRect & LRect::operator = ( LRect const & p1 )
{
  set ( p1 );
  return *this;
}

bool LRect::operator == ( LRect const & p1 ) const
{
  return
    ( m_x == p1.m_x ) &&
    ( m_y == p1.m_y ) &&
    ( m_width == p1.m_width ) &&
    ( m_height == p1.m_height );
}

bool LRect::contains ( LPoint const & p1 ) const
{
  return
    ( (p1.x() >= m_x) && (p1.x() <= (m_x+m_width)) ) &&
    ( (p1.y() >= m_y) && (p1.y() <= (m_y+m_height)) );
}

bool LRect::inside ( LPoint const & p1 ) const
{
  return
    ( (p1.x() > m_x) && (p1.x() < (m_x+m_width)) ) &&
    ( (p1.y() > m_y) && (p1.y() < (m_y+m_height)) );
}

bool LRect::outside ( LPoint const & p1 ) const
{
  return !contains ( p1 );
}

bool LRect::contains ( LRect const & p1 ) const
{
  return
    contains ( p1.leftUp() ) &&
    contains ( p1.rightDown() );
}

bool LRect::inside ( LRect const & p1 ) const
{
  return
    inside ( p1.leftUp() ) &&
    inside ( p1.rightDown() );
}

bool LRect::outside ( LRect const & p1 ) const
{
  return
    outside ( p1.leftUp() ) &&
    outside ( p1.rightDown() );
}

LPoint LRect::leftUpIntersection ( LRect const & p1 ) const
{
  return LPoint ( MAX(p1.x(),m_x), MAX(p1.y(),m_y) );
}

LPoint LRect::rightDownIntersection ( LRect const & p1 ) const
{
  return LPoint ( MIN(p1.x2(),x2()), MIN(p1.y2(),y2()) );
}

bool LRect::intersects ( LRect const & p1 ) const
{
  LPoint lu ( leftUpIntersection(p1) );
  LPoint rd ( rightDownIntersection (p1) );

  return
    ((rd.x()-lu.x())>=0) &&
    ((rd.y()-lu.y())>=0);
}

LRect LRect::intersection ( LRect const & p1 ) const
{
  if ( !intersects ( p1 ) )
    {
      throw NoIntersection ( __FILE__, __LINE__, *this, p1 );
    }

  return LRect ( leftUpIntersection(p1), rightDownIntersection(p1) );
}

// LRect::NoIntersection
LRect::NoIntersection::NoIntersection ( char const * file, luint32 line, LRect const & r1, LRect const & r2 ) :
  LBasicException ( file, line, "No Intersection" ),
  m_r1 ( r1 ),
  m_r2 ( r2 )
{
}

void LRect::NoIntersection::log ( void )
{
  LBasicException::log ();
}
