#include <lcl/lregex.h>
#include <lcl/ltrace.h>

#define STATIC
#include <pcre.h>

LRegEx::LRegEx () :
  pcre_compiled ( NULL ),
  ovector ( NULL ),
  foundMatches ( 0 )
{
}

LRegEx::LRegEx ( LString const & str ) :
  pcre_compiled ( NULL ),
  ovector ( NULL ),
  foundMatches ( 0 )
{  
  set ( str );
}

LRegEx::~LRegEx ()
{
  clear ();
}

int LRegEx::sys_getMaxCount ()
{
  if ( !isValid() )
    return -1;

  int i;

  int rc = pcre_fullinfo ( (pcre*)pcre_compiled, NULL, PCRE_INFO_CAPTURECOUNT, &i );

  if ( rc )
    return -1;

  return i;
}

long LRegEx::exec ( LString const &str, int flags )
{
  if ( !isValid() )
    return -1;

  if ( str.length() == 0 )
    return 0;

  long len = ((maxMatches+1)*3);

  foundMatches = 0;

  if ( ovector )
    delete ovector;

  ovector = new int [ len ];

  dupe = str;

  int rc = pcre_exec ( (const pcre*)pcre_compiled, NULL, str, str.length(), 0, 0, ovector, len );
  
  if ( ( rc <= 0 ) && ( rc != PCRE_ERROR_NOMATCH ) )
    {
      foundMatches = 0;
      return -1;
    }

  if ( rc == PCRE_ERROR_NOMATCH )
    {
      foundMatches = 0;
      return 0;
    }

  foundMatches = rc;
  return rc;
}

long LRegEx::numMatches () const
{
  return foundMatches;
}

LString LRegEx::match ( long i )
{
  if ( !isValid () || !ovector )
    return LString();

  if ( i > foundMatches )
    return LString ();

  int start, end;
  start = ovector[2*i];
  end = ovector[2*i+1];

  return dupe.subString ( start, end-start );
}

LString LRegEx::beforeMatch ( long i )
{
  if ( !isValid () || !ovector )
    return LString();

  if ( i > foundMatches )
    return LString ();

  int start, end;
  start = ovector[2*i];
  end = ovector[2*i+1];

  return dupe.left ( start );
}

LString LRegEx::afterMatch ( long i )
{
  if ( !isValid () || !ovector )
    return LString();

  if ( i > foundMatches )
    return LString ();

  int start, end;
  start = ovector[2*i];
  end = ovector[2*i+1];

  return dupe.subString ( end );
}

void LRegEx::clear ()
{
  if ( pcre_compiled )
    {
      pcre_free ( pcre_compiled );
      pcre_compiled = NULL;
    }  
  if ( ovector )
    {
      delete ovector;
      ovector = NULL;
    }
}

bool LRegEx::set ( LString const & str )
{
  clear ();

  int error_offset;
  char const * error_str;

  pcre_compiled = pcre_compile ( str, 0, &error_str, &error_offset, NULL );
  if ( !pcre_compiled )
    {
      LTRACE ( "LRegEx", "PCRE_ERROR: " + LString(error_str) + " at " + LString(error_offset) );
      //      pcre_free ( error_str );
    }
  else
    {
      maxMatches = sys_getMaxCount ();
    }
  
  return isValid();
}

bool LRegEx::isValid () const
{
  return pcre_compiled != NULL;
}


