#include <lcl/lserialize.h>

#include <lcl/lregex.h>
#include <lcl/ltrace.h>

LISerializer::LISerializer ( LIStream & _instream ) :
  instream ( _instream )
{
}

LOSerializer::LOSerializer ( LOStream & _outstream ) :
  outstream ( _outstream )
{
}

LIOSerializer::LIOSerializer ( LIOStream & _iostream ) :
  LISerializer ( _iostream ),
  LOSerializer ( _iostream )
{
}

LSerializer::LSerializer ( LString const & _buffer ) :
  buffer ( _buffer )
{
}

LSerializer::~LSerializer ()
{
}

void LSerializer::setBuffer ( LString const & _buffer )
{
  buffer = _buffer;
}

LString LSerializer::getBuffer ( void ) const
{
  return buffer;
}


LCL_EXPORT LSerializer & operator << ( LSerializer & ser, int p1 )
{
  ser.freeze ( "int", p1 );
  return ser;
}

LCL_EXPORT LSerializer & operator >> ( LSerializer & ser, int & p1 )
{
  p1 = ser.thaw ( "int" );
  return ser;
}

LCL_EXPORT LSerializer & operator << ( LSerializer & ser, long p1 )
{
  ser.freeze ( "long", p1 );
  return ser;
}

LCL_EXPORT LSerializer & operator >> ( LSerializer & ser, long & p1 )
{
  p1 = ser.thaw ( "long" );
  return ser;
}

LCL_EXPORT LSerializer & operator << ( LSerializer & ser, char p1 )
{
  ser.freeze ( "char", p1 );
  return ser;
}

LCL_EXPORT LSerializer & operator >> ( LSerializer & ser, char & p1 )
{
  p1 = ser.thaw ( "char" );
  return ser;
}

LCL_EXPORT LSerializer & operator << ( LSerializer & ser, unsigned long p1 )
{
  ser.freeze ( "unsigned long", p1 );
  return ser;
}

LCL_EXPORT LSerializer & operator >> ( LSerializer & ser, unsigned long & p1 )
{
  p1 = ser.thaw ( "unsigned long" );
  return ser;
}

LCL_EXPORT LSerializer & operator << ( LSerializer & ser, double p1 )
{
  ser.freeze ( "double", p1 );
  return ser;
}

LCL_EXPORT LSerializer & operator >> ( LSerializer & ser, double & p1 )
{
  p1 = ser.thaw ( "double" );
  return ser;
}

LCL_EXPORT LSerializer & operator << ( LSerializer & ser, LSerializer const & p1 )
{
  ser.freeze ( "serializer", p1.getBuffer() );
  return ser;
}

LCL_EXPORT LSerializer & operator >> ( LSerializer & ser, LSerializer & p1 )
{
  p1.setBuffer ( ser.thaw ( "serializer" ) );
  return ser;
}


// ============>>>>>>>>>>>> DEFAULT

#define DEFAULT_FREEZE(type) \
LSerializer & LDefaultSerializer::operator << ( type p1 ) \
{ \
  freeze ( #type, LString(p1) ); \
  return *this; \
}

#define DEFAULT_THAW(type) \
LSerializer & LDefaultSerializer::operator >> ( type & p1 ) \
{ \
  p1 = (type)thaw(#type); \
  return *this; \
}



LDefaultSerializer::LDefaultSerializer ( LString const & _buffer ) :
  LSerializer ( _buffer )
{
}

LDefaultSerializer::~LDefaultSerializer ()
{
}

LString LDefaultSerializer::encode ( LString const & input )
{
  return input.enSave ( "[:]" );
}

LString LDefaultSerializer::decode ( LString const & input )
{
  return input.deSave();
}

void LDefaultSerializer::freeze ( LString const & type, LString const & value )
{
  buffer += "[" + type + ":" + encode(value) + "]"; 
  LTRACE ( "LDefaultSerializer", "Buffer is now: " + buffer );
}

LString LDefaultSerializer::thaw ( LString const & type )
{
  LString myType, value;
  LRegEx exp ( "\\[([a-zA-Z0-9 ]+)\\:([^]]*)\\][ \\t\\n\\r]*" );
  //               "\\[([a-zA-Z0-9 ]+)\\:([^]]*)\\][ \n\t\r]*"

  if ( exp.exec ( buffer ) )
    {
      myType = exp.match(1);
      value = decode(exp.match(2));
      LTRACE ( "LDefaultSerializer", "Found type: " + exp.match(1) + " value: " + exp.match(2) );
      if ( type == myType )
	{
	  buffer = exp.afterMatch ( 0 );
	  LTRACE ( "LDefaultSerializer", "Buffer is now: " + buffer );
	  return value;
	}
      else
	{
	  LTRACE ( "LDefaultSerializer", "Type mismatch: " + myType + "!=" + type );
	}

    }
  else
    LTRACE ( "LDefaultSerializer", "Invalid token!" );
  throw 1;

  return "";
}
