#include <lcl/lsockaddr.4inet.h>

LSockAddr4Inet::LSockAddr4Inet ()
{
  in4_addr.sin_family = AF_INET;
  in4_addr.sin_port = 0;
  in4_addr.sin_addr.s_addr = INADDR_NONE;
}

LSockAddr4Inet::LSockAddr4Inet ( LSockAddr4Inet const & p1 )
{
  memcpy ( &in4_addr, &p1.in4_addr, sizeof(in4_addr) );
}

LSockAddr4Inet::LSockAddr4Inet ( unsigned short _port )  
{
  in4_addr.sin_family = AF_INET;
  in4_addr.sin_port = 0;
  in4_addr.sin_addr.s_addr = INADDR_NONE;

  setAddrDirect ( INADDR_ANY );
  setPort ( _port );
}

LSockAddr4Inet::LSockAddr4Inet ( LString const & _addr )
{
  LString addr = _addr;
  addr.setDelims ( ":" );

  in4_addr.sin_family = AF_INET;
  in4_addr.sin_port = htons ( (unsigned short)addr.word(1) );
  in4_addr.sin_addr.s_addr = inet_addr ( addr.word(0) );
  
  if ( in4_addr.sin_addr.s_addr == INADDR_NONE ) {
    hostent * hi = gethostbyname ( addr.word(0) );
    if ( hi ) {
      memcpy ( &in4_addr.sin_addr.s_addr, hi->h_addr, hi->h_length );
    }
  }   

  //  invalidate ();
}

LSockAddr4Inet::LSockAddr4Inet ( LString const & _addr, unsigned short _port )
{

  in4_addr.sin_family = AF_INET;
  in4_addr.sin_port = htons ( _port );
  in4_addr.sin_addr.s_addr = inet_addr ( _addr );
  
  if ( in4_addr.sin_addr.s_addr == INADDR_NONE ) {
    hostent * hi = gethostbyname ( _addr );
    if ( hi ) {
      memcpy ( &in4_addr.sin_addr.s_addr, hi->h_addr, hi->h_length );
    }
  } 

  //  invalidate ();
}

LSockAddr4Inet::LSockAddr4Inet ( sockaddr_in const * _addr )
{

  in4_addr.sin_family = AF_INET;
  in4_addr.sin_port = 0;
  in4_addr.sin_addr.s_addr = INADDR_NONE;

  if ( _addr->sin_family != AF_INET )
    return;

  memcpy ( &in4_addr, _addr, length () );

  //  invalidate ();
}

LString LSockAddr4Inet::nameLookup () const
{
  hostent * hi = gethostbyaddr ( (const char*)&in4_addr.sin_addr, sizeof(in4_addr.sin_addr), AF_INET );
  if ( hi ) {
    return hi->h_name;
  } else {
    return numLookup ();
  }
}

LString LSockAddr4Inet::numLookup () const
{
  if ( in4_addr.sin_addr.s_addr == INADDR_ANY )
    return "<INADDR_ANY>";

  if ( in4_addr.sin_addr.s_addr == INADDR_NONE )
    return "<INADDR_NONE>";

  return inet_ntoa ( in4_addr.sin_addr );
}

LSockAddr4Inet::operator sockaddr () const
{
  return *(sockaddr*)(&in4_addr);
}

LSockAddr4Inet::operator sockaddr* ()
{
  return (sockaddr*)&in4_addr;
}

LSockAddr4Inet::operator sockaddr const * () const
{
  return ( sockaddr const * ) &in4_addr;
}

LSockAddr4Inet::operator in_addr () const
{
  return in4_addr.sin_addr;
}

L_ADDRLEN_T LSockAddr4Inet::length () const
{
  return sizeof ( sockaddr_in );
}

void LSockAddr4Inet::setAddrDirect ( unsigned long _addr )
{
  in4_addr.sin_addr.s_addr = _addr;
  //  invalidate ();
}

void LSockAddr4Inet::setPortDirect ( unsigned short _port )
{
  in4_addr.sin_port = _port;
}

LSockAddr4Inet LSockAddr4Inet::any ()
{
  LSockAddr4Inet addr;
  addr.setAddrDirect ( INADDR_ANY );
  return addr;
}

LSockAddr4Inet LSockAddr4Inet::none ()
{
  LSockAddr4Inet addr;
  addr.setAddrDirect ( INADDR_NONE );
  return addr;
}

void LSockAddr4Inet::setPort ( unsigned short _port )
{
  setPortDirect ( htons ( _port ) );
}

unsigned short LSockAddr4Inet::getPort () const
{
  return ntohs ( in4_addr.sin_port );
}

unsigned long LSockAddr4Inet::getAddr () const
{
  return ntohl ( in4_addr.sin_addr.s_addr );
}

LString LSockAddr4Inet::dump () const
{
  return asString() + ":" + LString((long)getPort());
}

bool LSockAddr4Inet::operator == ( LSockAddr4Inet const & p1 ) const
{
  return ( p1.in4_addr.sin_family == in4_addr.sin_family ) &&
  ( p1.in4_addr.sin_port == in4_addr.sin_port ) &&
  ( p1.in4_addr.sin_addr.s_addr == in4_addr.sin_addr.s_addr );
}
