/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

/*!
 * \example string.cc
 * This example shows you how to use LString
 */

/*!
 * \example lstring_tok.cc
 * This example shows you how to use LString and LStringTokenizer string tokenizing
 */

/*!
 * \example replace.cc
 * This example shows you how to use replacement functions
 */

#include <lcl.h>

class LCL_EXPORT LString;

#include <lcl/lstring.h>
#include <lcl/lstreamproxy.h>
#include <lcl/lutils.h>
#include <lcl/lstringtokenizer.h>
#include <lcl/lregex.h>
#include <lcl/lapp.h>
#include <lcl/lstringreplace.h>
//#include <lcl/lmemory.h>

#include <string.h>

#include <malloc.h>
#include <stdlib.h>
//#include <assert.h>
#include <stdio.h>
#include <ctype.h>

#ifdef _WIN32
#include <tchar.h>
#endif

char * LString::defaultDelims = " \n\r\t";
char * LString::defaultQMarks = "\"";

#include <lcl/ltracefunc.h>

LVCS_INIT(LString);

int _isbinarydigit ( int c )
{
  return (c == '1') || (c == '0');
}

int _ishexdigit ( int c )
{
  return isdigit ( c ) || ( ((c>='a')&&(c<='f')) || ((c>='A')&&(c<='F')) ) ;
}


/*!
 * A specialized hash function.
 *
 * If we would use the default hash function it would calculate the hash value from the four pointer bytes
 * which will change as soon as you copy the string. The "real" value of the string is the string itself, not the
 * value of the pointer! 
 */
unsigned long hash ( LString const & item, unsigned long hash_size )
{
  return hash ( item, item.length(), hash_size );
}

//const int LString::C_block_size = 64;

LString::LString () :
  delims ( NULL ),
  qmarks ( NULL )
{
  ptr_used = ptr_space = 0;
  string_ptr = NULL;
}

LString::LString ( const char p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = (char*)malloc ( 2 );
  string_ptr [0] = p1;
  string_ptr [1] = 0;
  ptr_used = ptr_space = 2;
}

LString::LString ( const char * p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  if ( p1 == NULL ) {
    string_ptr = NULL;
    ptr_used = ptr_space = 0;
  } else {
    string_ptr = lcl_strdup ( p1 );
    ptr_used = ptr_space = strlen ( p1 )+1;
  }
}

LString::LString ( LBuffer const & p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  if ( p1.getSize() != 0 ) {
    string_ptr = (char*)malloc ( p1.getSize()+1 );
    memcpy ( string_ptr, p1.getPtr(), p1.getSize() );
    string_ptr[p1.getSize()] = 0;
    ptr_used = ptr_space = p1.getSize()+1;
  } else {
    string_ptr = NULL;
    ptr_used = ptr_space = 0;
  }
}

LString::LString ( LString const & p1 ) :
  delims ( p1.delims ? strdup ( p1.delims ) : NULL ),
  qmarks ( p1.qmarks ? strdup ( p1.qmarks ) : NULL )
{
  if ( p1.string_ptr ) {
    ptr_used = ptr_space = strlen ( p1.string_ptr )+1;
    string_ptr = lcl_strdup ( p1.string_ptr );
  } else {
    string_ptr = NULL;
    ptr_used = ptr_space = 0;
  }
}

LString::LString ( double p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = NULL;
  ptr_used = ptr_space = 0;

  char buffer [64];
  sprintf ( buffer, "%f", p1 );  
  
  clear();
  append ( buffer );
}

LString::LString ( unsigned long p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = NULL;
  ptr_used = ptr_space = 0;

  char buffer [64];
  sprintf ( buffer, "%lu", p1 );
  
  clear();
  append ( buffer );
}

LString::LString ( signed long p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = NULL;
  ptr_used = ptr_space = 0;

  char buffer [64];
  sprintf ( buffer, "%ld", p1 );
  
  clear();
  append ( buffer );
}

LString::LString ( signed int p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = NULL;
  ptr_used = ptr_space = 0;

  char buffer [64];
  sprintf ( buffer, "%d", p1 );
  
  clear();
  append ( buffer );
}

LString::LString ( unsigned int p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = NULL;
  ptr_used = ptr_space = 0;

  char buffer [64];
  sprintf ( buffer, "%u", p1 );
  
  clear();
  append ( buffer );
}

LString::LString ( signed short int p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = NULL;
  ptr_used = ptr_space = 0;

  char buffer [64];
  sprintf ( buffer, "%hd", p1 );
  
  clear();
  append ( buffer );
}

LString::LString ( unsigned short int p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = NULL;
  ptr_used = ptr_space = 0;

  char buffer [64];
  sprintf ( buffer, "%hu", p1 );
  
  clear();
  append ( buffer );
}

LString::LString ( lint64 p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = NULL;
  ptr_used = ptr_space = 0;

  char buffer [64];
  sprintf ( buffer, "%lld", p1 );
  
  clear();
  append ( buffer );
}

LString::LString ( luint64 p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = NULL;
  ptr_used = ptr_space = 0;

  char buffer [64];
  sprintf ( buffer, "%llu", p1 );
  
  clear();
  append ( buffer );
}

LString::~LString ()
{
  /*
  if ( string_ptr )
    free ( string_ptr );

  if ( delims )
    free ( delims );
  if ( qmarks )
    free ( qmarks );
  */
  clear ();
}

#ifdef HAVE_MFC
LString::LString ( CString const & p1 ) :
  delims ( NULL ),
  qmarks ( NULL )
{
  string_ptr = NULL;
  ptr_used = ptr_space = 0;

  append ( (const char*)p1 );
}

LString::operator CString () const
{
  return CString ( (const char*)this );
}

#endif

void LString::clear ()
{
  LStringBase::clear ();

  if ( delims )
    {
      free ( delims );
      delims = NULL;
    }
  if ( qmarks )
    {
      free ( qmarks );
      qmarks = NULL;
    }

}

/*!
 * \return always 1
 * \param p1 the string to append
 *
 * Append a character string to this
 */
void LString::append ( const char * p1 )
{
  if ( string_ptr ) {

    if ( p1 ) {
            
      fitMem ( length()+strlen(p1)+1 );
      strcat ( string_ptr, p1 );
      return;

    } else {

      // p1 is is a NULL string
      return;

    }

  } else {

    if ( p1 ) { 

      // we append to an empty string ( -> assign )
      ptr_space = ptr_used = strlen ( p1 )+1;
      string_ptr = lcl_strdup ( p1 );
      return;

    } else {

      // our string is empty and so is p1
      return;

    }

  }

}

/*!
 * Append a string to this
 *
 * \param p1 the string to append
 *
 */
void LString::append ( LString const & p1 )
{
  if ( p1.string_ptr == NULL )
    {
      return;
    }

  append ( p1.string_ptr );
}

void LString::prepend ( const char * p1 )
{
  // The string to prepend is empty -> NOOP
  if ( !p1 )
    return;

  if ( string_ptr )
    {
      // Do the read prepend

      // Fit to new size
      unsigned long p1_len = strlen(p1);
      unsigned long t_len = length();
      fitMem ( t_len+p1_len+1 );
      lcl_memmove ( string_ptr+p1_len, string_ptr, t_len );
      memcpy ( string_ptr, p1, p1_len );
      string_ptr[t_len+p1_len] = 0;
    }
  else
    {
      // We are currently empty, therefore we have a set operation
      ptr_space = ptr_used = strlen ( p1 )+1;
      string_ptr = lcl_strdup ( p1 );
    }
}

void LString::prepend ( LString const & p1 )
{
  if ( !p1.string_ptr )
    return;

  prepend ( p1.string_ptr );
}

LString & LString::operator = ( const char * p1 )
{
  clear ();

  if ( p1 ) {
    append ( p1 );
    //    string_ptr = lcl_strdup ( p1 );
  }

  return *this;
}

LString & LString::operator = ( LString const & p1 )
{
  if ( this == &p1 )
     return *this;

  clear ();

  if ( p1.string_ptr ) {
    append ( p1 );
    //    string_ptr = lcl_strdup ( p1.string_ptr );
  }

  qmarks = p1.qmarks ? strdup ( p1.qmarks ) : NULL;
  delims = p1.delims ? strdup ( p1.delims ) : NULL;

  return *this;
}

LString & LString::operator = ( char value )
{
  clear ();

  char T[2];
  T[0] = value;
  T[1] = 0;

  append ( T );

  return *this;
}

LString & LString::operator = ( double value )
{
  clear ();

  LString tmp ( value );
  append ( tmp );

  return *this;
}

LString & LString::operator = ( long value )
{
  clear ();

  LString tmp ( value );
  append ( tmp );

  return *this;
}

bool LString::operator ! ( ) const
{
  if (!string_ptr)
    return true;
  if ( !length() )
    return true;

  return false;
}

bool LString::operator == ( const char * p1 ) const
{

  if ( string_ptr ) {

    if ( p1 ) {
      // return TRUE if strings are equal
      return strcmp ( string_ptr, p1 ) == 0;
    } else {
      // string is OK but parameter is NULL
      return string_ptr[0] == 0;
    }

  } else {

    if ( p1 ) {
      // we have a "NULL" string, check if parameter is also "NULL"
      return p1[0] == 0;
    } else {
      // great: string is NULL and p1 is also NULL
      return true;
    }

  }
}

bool LString::operator == ( LString const & p1 ) const
{
  return (*this) == p1.string_ptr;
}

bool LString::operator != ( const char * p1 ) const
{
  return ! ( (*this) == p1 );
}

bool LString::operator != ( LString const & p1 ) const
{
  return ! ( (*this) == p1.string_ptr );
}

LString::operator bool () const
{
  if ( !string_ptr )
    return false;
  if ( string_ptr[0] == 0 )
    return false;

  return true;
}

LString::operator const char * () const
{
  if ( length() ) {
    return (const char*)string_ptr;
  } else {
    return "";
  }
}

LString::operator const char * ()
{
  if ( length() ) {
    return (const char*)string_ptr;
  } else {
    return "";
  }
}

LString::operator char * ()
{
  return string_ptr;
}

LString::operator short unsigned int () const
{
  //  short unsigned int i;

  if ( length() ) {
    //    sscanf ( string_ptr, "%hu", &i );
    //    return i;
    return strtoul ( string_ptr, NULL, 0 );
  } else {
    return 0;
  }
}

LString::operator short signed int () const
{
  //  short signed int i;

  if ( length() ) {
    //    sscanf ( string_ptr, "%hd", &i );
    //    return i;
    return strtol ( string_ptr, NULL, 0 );
  } else {
    return 0;
  }
}

LString::operator long () const
{
  //  long l;

  if ( length() ) {
    //    sscanf ( string_ptr, "%ld", &l );    
    //    return l;
    return strtol ( string_ptr, NULL, 0 );
  } else {
    return 0;
  }
}

LString::operator unsigned long () const
{
  unsigned long ul;

  if ( length() ) {
    //    sscanf ( string, "%lu", &ul );
    ul = strtoul ( string_ptr, NULL, 0 );
    return ul;
  } else {
    return 0;
  }
}

LString::operator int () const
{
  //  int i;

  if ( length() ) {
    //    sscanf ( string_ptr, "%d", &i );
    //    return i;
    return strtol ( string_ptr, NULL, 0 );
  } else {
    return 0;
  }
}


LString::operator unsigned int () const
{
  unsigned int i;

  if ( length() ) {
    //    sscanf ( string_ptr, "%u", &i );
    return strtoul ( string_ptr, NULL, 0 );
      //    return i;
  } else {
    return 0;
  }
}

LString::operator double () const
{
  //  double d;

  if ( length() ) {
    //    sscanf ( string_ptr, "%lf", &d );
    return strtod ( string_ptr, NULL );
    //    return d;
  } else {
    return 0;
  }
}

LString::operator lint8 () const
{
  if ( length() )
    return strtol ( string_ptr, NULL, 0 );
  else
    return 0;
}

LString::operator luint8 () const
{
  if ( length() )
    return strtoul ( string_ptr, NULL, 0 );
  else
    return 0;
}

LString::operator char () const
{
  return get(0);
}

LString LString::fromDouble ( double value, long acc, long len, char filler )
{
  char buffer[256];

  LString form = "%";
  if ( (filler==' ') || (filler=='0') ) {
    form += filler;
  }
  if ( len != -1 ) {
    form += LString ( len );
  }
  if ( acc != -1 ) {
    form += "." + LString (acc);
  }
  form += "f";

  sprintf ( buffer, form, value );
  return buffer;
}

#ifndef _WIN32
LString::operator NAMESPACE_STD(string) () const
{
  return NAMESPACE_STD(string) ((const char*)*this);
}

#endif

/*!
 * \return the length of the string in characters
 */
unsigned long LString::length () const
{
  if ( string_ptr ) {
    return strlen ( string_ptr );
  } else {
    return 0;
  }
}

/*!
 * \param c the character to check
 * \return the number of time the string contains the character
 *
 * Check if the string contains the character
 */
unsigned long LString::contains ( char c ) const
{
  unsigned long i, len, counter=0;

  len = length();

  for ( i = 0; i<len; i++) {
    if ( string_ptr[i] == c )
      counter++;
  }
  return counter;
}

unsigned long LString::numOf ( char c ) const
{
  return contains ( c );
}

/*!
 * \retval true if it matches
 * \retval false if it does not match
 * \param string the string which should match
 * \param caseSens if true the match is case sensitive, otherwise it is case-in-sensitve
 *
 * Checks if the parameter matches the start of the string
 *
 * \code
// A small example...

LString str1 = "Hello World!";
bool result1 = str1.beginsWith ( "hello", false ); // Would be true
bool result2 = str1.beginsWith ( "world" ); // Would be false
 * \endcode
 */
bool LString::beginsWith ( LString const & string, bool caseSens ) const
{
  if ( (!length()) || (!string.length()) )
    return false;

  if ( caseSens )
    return left(string.length()).toUpper() == string.toUpper();
  else
    return left(string.length()) == string;
}

/*!
 * Equivalent of LString::beginsWith
 *
 */
bool LString::endsWith ( LString const & string, bool caseSens ) const
{
  if ( (!length()) || (!string.length()) )
    return false;

  if ( caseSens )
    return right(string.length()).toUpper() == string.toUpper();
  else
    return right(string.length()) == string;
}

bool LString::isOfFunc ( int (*func)( int ) ) const
{
  if ( !string_ptr )
    return false;
  long i = 0;
  while ( string_ptr[i] ) {
    if ( !func( string_ptr[i] ) ) {
      return false;
    }
    i++;
  }
  return true;
}

/*!
 * \retval true the string is fully lowercase
 * \retval false one or more characters are uppercase
 */
bool LString::isLowerCase () const
{
  return isOfFunc ( islower );
}

/*!
 * \retval true the string is fully uppercase
 * \retval false one or more characters are lowercase
 */
bool LString::isUpperCase () const
{
  return isOfFunc ( isupper );
}

/*!
 * \retval true the string contains only digits
 * \retval false the string also contains some non-digits
 */
bool LString::isDigits () const
{
  return isOfFunc ( isdigit );
}

/*!
 * \retval true the string contains only binary digits
 * \retval false the string also contains some non-binary-digits
 *
 * Binary digits are 0 or 1
 */
bool LString::isBinaryDigits () const
{
  return isOfFunc ( _isbinarydigit );
}

/*!
 * \retval true the string contains only hex digits
 * \retval false the string also contains some non-hex-digits
 *
 * Hex digits are "0-9A-Za-z"
 */
bool LString::isHexDigits () const
{
  return isOfFunc ( _ishexdigit );
}

bool LString::isAlphabetic () const
{
  return isOfFunc ( isalpha );
}

bool LString::isAlphanumeric () const
{
  return isOfFunc ( isalnum );
}

/*!
 * \retval true the string contains only printable characters
 * \retval false the string also contains some non-printable characters
 */
bool LString::isPrintable () const
{
  return isOfFunc ( isprint );
}

/*!
 * \retval true the string contains only control characters
 * \retval false the string also contains some non-control characters
 */
bool LString::isControl () const
{
  return isOfFunc ( iscntrl );
}

/*!
 * converts a string to uppercase characters
 *
 * \return the uppercase string
 * \post this string will not change
 */
LString LString::toUpper () const
{
  LString str = *this;

  unsigned long i, len = str.length();
  for ( i = 0; i<len; i++ ) {
    str.string_ptr[i] = toupper ( str.string_ptr[i] );
  }

  return str;
}

/*!
 * converts a string to lowercase characters
 *
 * \return the lowercase string
 * \post this string will not change
 */
LString LString::toLower () const
{
  LString str ( *this );

  unsigned long i, len = str.length();
  for ( i = 0; i<len; i++ ) {
    str.string_ptr[i] = tolower ( str.string_ptr[i] );
  }

  return str;
}


LString LString::operator + ( const char * p1 ) const
{
  LString str ( *this );
  str.append ( p1 );
  return str;
}

LString LString::operator + ( char * p1 ) const
{
  LString str ( *this );
  str.append ( p1 );
  return str;
}

LString LString::operator + ( LString const & p1 ) const
{
  LString str ( *this );
  str.append ( p1.string_ptr );
  return str;
}

LString LString::operator + ( LString & p1 ) const
{
  LString str ( *this );
  str.append ( p1.string_ptr );
  return str;
}

char & LString::operator [] ( unsigned int l )
{
  if ( (l >= length()) || (string_ptr==NULL) ) {
    throw "LString::operator[] ( int )";
    MSTHROW(char);
  } else {
    return string_ptr[l];
  }
}

char LString::operator [] ( unsigned int l ) const
{
  if ( (l >= length()) || (string_ptr==NULL) ) {
    return 0;
  } else {
    return string_ptr[l];
  }
}


char & LString::operator [] ( int l )
{
  if ( (l >= length()) || (string_ptr==NULL) ) {
    throw "LString::operator[] ( int )";
    MSTHROW(char);
  } else {
    return string_ptr[l];
  }
}

char LString::operator [] ( int l ) const
{
  if ( (l >= length()) || (string_ptr==NULL) ) {
    return 0;
  } else {
    return string_ptr[l];
  }
}


char & LString::operator [] ( long int l )
{
  if ( (l >= length()) || (string_ptr==NULL) ) {
    throw "LString::operator[] ( int )";
    MSTHROW(char);
  } else {
    return string_ptr[l];
  }
}

char LString::operator [] ( long int l ) const
{
  if ( (l >= length()) || (string_ptr==NULL) ) {
    return 0;
  } else {
    return string_ptr[l];
  }
}


char & LString::operator [] ( unsigned long l )
{
  if ( (l >= length()) || (string_ptr==NULL) ) {
    throw "LString::operator[] (long l)";

    MSTHROW(char);
  } else {
    return string_ptr[l];
  }
}

char LString::operator [] ( unsigned long l ) const
{
  if ( (l >= length()) || (string_ptr==NULL) ) {
    return 0;
  } else {

    return string_ptr[l];
  }
}

char LString::get ( unsigned long l) const
{
  if ( (l >= length()) || (string_ptr==NULL) ) {
    return 0;
  } else {
    return string_ptr[l];
  }  
}


LString & LString::operator += ( LString const & p1 )
{
  append ( p1.string_ptr );
  return *this;
}

LString & LString::operator += ( const char * p1 )
{
  append ( p1 );
  return *this;
}

LString & LString::operator += ( char c )
{
  char T[2];
  T[0] = c;
  T[1] = 0;
  append ( T );
  return *this;
}


void LString::setDelims ( const char * str )
{
  if ( delims )
    free ( delims );
  delims = str ? strdup ( str ) : NULL;
}

void LString::setQMarks ( const char * str )
{
  if ( qmarks )
    free ( qmarks );

  qmarks = str ? strdup ( str ) : NULL;
}

/*!
 * \return the number of words found in the string
 * \warning This method will count the number of words each time is is called
 * \warning When you call this method often for the same string use LStringTokenizer instead
 *
 * This method will count the number of words based on the delimters (LString::setDelims) and quotation marks (LString::setQMarks)
 *
 */
unsigned long LString::numWords () const
{
  unsigned long i, len=length(), wordCounter=0;
  bool quote = false;
  bool lastWS = true;

  for ( i=0; i<len; i++ ) {
    if ( strchr ( qmarks ? qmarks : defaultQMarks, get(i) ) ) {
      quote = !quote;
      if ( quote && lastWS )
	wordCounter++;
    }

    if ( strchr ( delims ? delims : defaultDelims, get(i) ) ) {
      lastWS = true;
    } else {
      if ( lastWS && !quote )
	wordCounter++;

      lastWS = false;
    }
  }

  //  if ( !lastWS )
  //  wordCounter++;

  return wordCounter;
}

/*!
 * \return The word at the given position.
 * \return If there is no word with this position an empty string is returned
 * \warning This method will parse the words each time is is called
 * \warning When you call this method often for the same string use LStringTokenizer instead
 *
 * This method will count the number of words based on the delimters (LString::setDelims) and quotation marks (LString::setQMarks)
 *
 * Remember: Words start at 0
 */
LString LString::word ( unsigned long nWord ) const
{
  unsigned long i, len=length(), wordCounter=0;
  bool quote = false;
  bool lastWS = true;
  LString theWord;

  for ( i=0; i<len; i++ ) {

    if ( strchr ( qmarks ? qmarks : defaultQMarks, get(i) ) ) {
      quote = !quote;
      if ( quote && lastWS )
	wordCounter++;
    }

    if ( strchr ( delims ? delims : defaultDelims, get(i) ) ) {
      if ( quote ) {
	if ( wordCounter == nWord+1 )
	  theWord += get(i);	
      } else {
	lastWS = true;
      }

    } else {

      if ( lastWS && !quote )
	wordCounter++;

      lastWS = false;

      if ( wordCounter == nWord+1 )
	theWord += get(i);

    }
  }

  return theWord;
}


//! Checks if a specific word is contained in the string
/*!
 * \param word The word to look for
 * \param caseIns Should the comparsion be case sensitive ( false=sensitive, true=insensitive)
 * \return true if the string contains the word
 * \code
 LString str = "This is a very stupid test!";

 if ( str.containsWord ( "very" )
    cout << "String '" << str << "' contains the word 'very'" << endl;
 * \endcode
 */
bool LString::containsWord ( LString const & _word, bool caseIns ) const
{
  long num = numWords();
  long i;
  for ( i = 0; i<num; i++ ) {
    if ( caseIns ) {
      if ( word(i).toUpper() == _word.toUpper() )
	return true;
    } else {
      if ( word(i) == _word )
	return true;
    }
  }
  return false;
}

/*!
 * \return the substring of this string
 * \param from the starting position
 *
 * returns the substring starting at the provided starting position and ending at the end of the string
 */
LString LString::subString ( unsigned long from ) const
{
  if ( from >= length() )
    return LString();

  lcl_assert ( string_ptr );

  char * tmp = string_ptr;
  tmp += from;

  return LString ( tmp );
}

/*!
 * \return the substring of this string
 * \param from the starting position
 * \param len the number of characters
 *
 * returns the substring starting at the provided starting position with the maximum length of characters provided.
 *
 * when less characters are available than requested, the returned string will be shorter
 */
LString LString::subString ( unsigned long from, unsigned long len ) const
{
  // Requesting from is too big
  if ( from >= length() )
    return LString();
  
  lcl_assert ( string_ptr );

  char * tmp = string_ptr;
  tmp += from;

  if ( strlen ( tmp ) <= len ) {

    // String is smaller than requested
    return LString (tmp);

  } else {

    // String has to be cut down
    tmp = lcl_strdup ( tmp );
    lcl_assert ( tmp );
    tmp[len] = 0;
    LString str (tmp);
    free ( tmp );
    return str;

  }
}

/*!
 * \return the substring of this string
 * \param from the starting position
 * \param len the number of characters
 *
 * returns the substring starting at the provided starting position with the maximum length of characters provided.
 *
 * when less characters are available than requested, the returned string will be shorter
 */
LString LString::subString ( unsigned long from, unsigned long len, char filler ) const
{
  // Requesting from is too big
  if ( from >= length() )
    return LString::filled ( len, filler );
  
  lcl_assert ( string_ptr );

  char * tmp = string_ptr;
  tmp += from;

  if ( strlen ( tmp ) <= len ) {

    // String is smaller than requested
    return LString (tmp) + LString::filled ( len-strlen(tmp), filler );

  } else {

    // String has to be cut down
    tmp = lcl_strdup ( tmp );
    lcl_assert ( tmp );
    tmp[len] = 0;
    LString str (tmp);
    free ( tmp );
    return str;

  }
}

/*!
 * \return the left side of the string
 * \param num the numbers of characters to fetch from the left side
 *
 * When the number exceeds the actual string length, it will only return the maximum available number of characters
 */
LString LString::left ( unsigned long num ) const
{
  return subString ( 0, num );
}

/*!
 * \return the right side of the string
 * \param num the numbers of characters to fetch from the right side
 *
 * When the number exceeds the actual string length, it will only return the maximum available number of characters
 */

LString LString::right ( unsigned long num ) const
{
  long start = length() - num;
  if ( start < 0 )
    start = 0;

  return subString ( start );
}

LString LString::perror ( int errnum )
{
  LString str ( strerror ( errnum ) );
  return str;
}

LString LString::filled ( unsigned long num, char c)
{
  char * buffer = (char*)malloc ( num+1 );
  memset ( buffer, c, num );
  buffer[num] = 0;
  LString str = buffer;
  free ( buffer );
  return str;
}

//! Fills the string with the content of a environment variable
/*!
 * The variable is read in via \c getenv and assigned to the current string. The content of
 * the previous string is therefore lost.
 *
 * \param var Name of the environment variable
 *
 * \post the current string will be replaced with the value from the environmet
 */
void LString::getEnv ( const char * env )
{
  clear ();
  append ( getenv ( env ) );
}


//! Returns a new string with the contents of an environment variable
/*!
 * \param env Name of the environment variable
 * \code

LString myString = LString::fromEnv ( "PATH" );

 * \endcode
 */
LString LString::fromEnv ( const char * env )
{
  LString str;
  str.getEnv ( env );
  return str;
}

LString LString::fromLastError( unsigned long errcode )
{
  #ifdef _LCL_SYS_WIN32_
  return fromErrorCode ( errcode );
  #else
  return perror ( errcode );
  #endif
}

#ifdef _LCL_SYS_WIN32_

/*!
 * Get the error description from the operating system
 */
LString LString::fromErrorCode ( DWORD rc )
{
  LPVOID lpMsgBuf;

  FormatMessage ( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, rc, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf, 0, NULL );

  LString str = (char*)lpMsgBuf;

  LocalFree( lpMsgBuf );

  return str;
}

#endif

//! Return the first character of the string. '\0' if none would exists. This is an equivalent to string[0] with the diffrence that it does not throw an exception of the string length is zero!
char LString::firstChar () const
{
  if ( string_ptr ) {
    return string_ptr[0];
  } else {
    return 0;
  }
}

//! Return the last character of the string.
/*!
 * \return the last character of the string. '\0' if none exists (e.g. length of the string is zero).
 *
 * This is an equivalent to string[string.length()-1] with the difference that it does not throw an
 * exception of the string length is zero!
 */
char LString::lastChar () const
{
  if ( string_ptr ) {
    return string_ptr[length()-1];
  } else {
    return 0;
  }
}

/*!
 * Removes any characters from the beginning of the string which are contained in this string
 *
 * \return a reference to this string
 * \param c a set of characters which are chosen for removement
 * \post the string will have all characters removed which match
 *
 * \code

LString str = "  \tLine from file\n\r";
str = str.removeLeading ( " \n\r\t" ).removeTrailing ( " \n\r\t" );  // clean it up

 * \endcode
 */
LString & LString::removeLeading ( LString const & c )
{
  if ( c == "" )
    return *this;

  while ( c.contains ( firstChar() ) ) {
    removeFirstChar ();
  }

  return *this;
}

/*!
 * equivalent to LString::removeLeading. It only operates at the end of the string.
 *
 * \return a reference to this string
 * \param c a set of characters which are chosen for removement
 * \post the string will have all characters removed which match
 */

LString & LString::removeTrailing ( LString const & c )
{
  if ( c == "" )
    return *this;

  while ( c.contains ( lastChar() ) ) {
    removeLastChar ();
  }

  return *this;
}

/*!
 * Removes the first character of the string if one exists. If the string is empty nothing will happen.
 *
 * \return a reference to this string
 * \post the string will have its first character removed
 */
LString & LString::removeFirstChar ()
{
  (*this) = subString(1);
  return *this;
}

/*!
 * Removes the last character of the string if one exists. If the string is empty nothing will happen.
 *
 * \return a reference to this string
 * \post the string will have its last character removed
 */
LString & LString::removeLastChar ()
{
  if ( length() == 0 )
    return *this;

  if ( length() == 1 ) {
    clear();
    return *this;
  }

  // this will remove the length by one byte (since length is number of char not
  // inluding the trailing \0
  unsigned long len = length();
  //  string_ptr = (char*) lcl_realloc ( string_ptr, len );
  fitMem(len);
  string_ptr[len-1] = 0;

  return *this;
}

/*!
 * \return the character pointer of this string
 * \warning You might get a NULL pointer when the string pointer is empty
 */
char * LString::toChar ()
{
  return string_ptr;
}

/*!
 * \return the character pointer of this string
 */
const char * LString::toCharConst () const
{
  if ( !string_ptr )
    return "";
  else
    return string_ptr;
}

long LString::toLong () const
{
  if ( length() ) {
    long l;
    sscanf ( string_ptr, "%ld", &l );
    return l;
  } else {
    return 0;
  }
}

double LString::toDouble () const
{
  if ( length() ) {
    double d;
    d = strtod ( string_ptr , NULL );
    return d;
  } else {
    return 0;
  }
}

LCL_EXPORT NAMESPACE_STD(ostream) & operator << ( NAMESPACE_STD(ostream) & os, LString const & p1 )
{
  os << (const char*)p1;
  return os;
}

LCL_EXPORT LOStream & operator << ( LOStream & los, LString const & p1 )
{
  los << (const char*)p1;
  return los;
}

LCL_EXPORT LString operator +( const char * p1, LString const & p2 )
{
  LString str1 ( p1 );
  return str1 + p2;
}

LCL_EXPORT LString operator +( const char * p1, LString & p2 )
{
  LString str1 ( p1 );
  return str1 + p2;
}

LCL_EXPORT LString operator + ( char * p1, LString const & p2 )
{
  LString ( str1 ( p1 ) );
  return str1 + p2;
}

LCL_EXPORT LString operator + ( char * p1, LString & p2 )
{
  LString ( str1 ( p1 ) );
  return str1 + p2;
}

void LString::change ( LString const & scan, LString const &replace, unsigned long nums )
{
  if ( scan == "" )
    return;

  if ( !string_ptr )
    return;

  unsigned long count = 0;
  unsigned long pos = 0;
  bool possible = false;
  char c;
  LString newstr;
  LString tag;

  while ( (pos < length()) && ((count<nums)||(nums==0)) ) {
    c = string_ptr[pos];

    if ( c == scan.get(tag.length()) ) {
      possible = true;
    } else {
      possible = false;
      if ( tag.length() > 0 ) {
	newstr += tag;
	tag = "";
      }
    }

    if ( possible ) {
      tag += c;
    } else {
      newstr += c;
    }

    if ( tag == scan ) {
      tag = "";
      newstr += replace;
      count++;
    }

    pos++;
  }
  newstr+=tag;
  *this = newstr;
}


const char * LString::C_tag_exp_default = "\\$\\(([^\\$]*?)\\)";
const char * LString::C_tag_exp_bash =    "\\$\\{([^\\$]*?)\\}";


//! Replace variables in a string
/*!
 * This function is using LString::replace with a default argument of
 * LString::C_tag_exp_default as expression pattern.
 */

LString LString::replace ( Replacer const & replacer ) const
{
  return replace ( replacer, C_tag_exp_default, 100 );
}

LString LString::replace_XXX ( Replacer const & replacer ) const
{
  LString newStr;
  int i = 0;
  bool dollar = false, isVar = false;
  LString varName;

  if ( !string_ptr )
    return *this;

  while ( string_ptr[i] != 0 ) {
    switch ( string_ptr[i] ) {
    case '$':
      /*
      if ( dollar ) {
	newStr += "$$";
	dollar = false;
      } else {
	dollar = true;
      }
      */
      if (dollar)
	newStr += "$";
      dollar = true;
      break;
    case '(':
      if ( dollar ) {
	varName = "";
	isVar = true;
	dollar = false;
      } else {
	newStr += '(';
      }
      break;  
    case ')':
      if ( isVar ) {
	newStr += replacer.sys_replace ( varName );
	isVar = false;

      } else {
	if ( dollar ) {
	  newStr += "$";
	  dollar = false;
	}
	newStr += ')';
      }
      break;

    default:
      if ( dollar ) {
	newStr += "$";
	dollar = false;
      }
      if ( isVar ) {
	varName += string_ptr[i];
      } else {
	newStr += string_ptr[i];
      }
      break;
    } // ... case
    i++;
  } // ... while   

  return newStr;
}

/*!
 * \internal
 */

LString LString::doReplace ( LString str, Replacer const & replacer, LRegEx & exp, luint32 currec )
{
  if ( currec == 0 )
    return str;

  LString temp;

  //  cout << endl << "->" << str << endl;

  while ( exp.exec(str) )
    {
      str = exp.beforeMatch(0);
      temp = exp.afterMatch(0);
      str += replacer.sys_replace ( doReplace ( exp.match(1), replacer, exp, currec-1 ) ) + temp;
    }
  //  cout << endl << "-->" << str << endl;
  return str;
}
//! Find tags using regular expressions and replacing them using a generic replacer
/*!
 * This is the most generic replacement method in LString. It replaces using a replacement
 * class and searches for items to replace using a regular expression.
 *
 * The regular expression pattern is used to detect tags in the string. The whole tag must
 * be match #0 and the extracted tag name must be match #1. Only the tag name is used
 * to look up the replacement. The suroundings are removed. See refer to LRegEx for more 
 * information about regular expressions.
 *
 * The extracted tag name will be looked up using the "Replacer". But before the lookup
 * is actually replaced in the string all tag functions are evaluated.
 *
 * Tag functions are like bash's "parameter expansion". If the tag name is "TAG:-DEFAULT"
 * for example and the tag with the name "TAG" is either empty or not defined the resulting
 * replacement is "DEFAULT" and not "". If the tag would simply be "TAG" and it would not
 * be definend it would result in "".
 *
 * The following tag functions are available:
 *
 *   TAG:-ARG or TAG!-ARG
 *       if TAG is not defined or empty ARG will be used instead
 *   
 *   TAG?-ARG
 *       if TAG is not defined ARG will be used instead
 *
 *   TAG:+ARG or TAG!+ARG
 *       if TAG is defined and not emptry ARG will be used
 *
 *   TAG?+ARG
 *       if TAG is defined ARG will be used
 *
 *   TAG:ARG or TAG|ARG
 *       This will cut off the first ARG characters of the replacement of TAG
 *
 *   TAG:ARG1:ARG2 or TAG|ARG1|ARG2
 *       This will cut out a substring from the replacement of TAG starting at offset
 *       ARG1 with a length of ARG2 characters
 *
 *   TAG!ARG1:ARG2 or TAG!ARG1!ARG2
 *       If TAG is defined and not empty ARG1 will be used, otherwise ARG2.
 *       Note that : instead of the ! is not allowed here!!
 *
 *   TAG?ARG1:ARG2 or TAG?ARG1!ARG2
 *       If TAG is defined ARG1 will be used. Otherwise ARG2
 *
 * Putting a # before the tag name (e.g. #TAG or #TAG?+XXX) will return the number of
 * characters this string would have instead of the actual string.
 *
 * Most of the functions are equal to the "bash" expansions. Important is that "?" instead
 * of ":" sometimes can be used as "not defined" whereas ":" means "not defined _or_ empty".
 *
 * Instead of ":" "!" can be used in many places, since ":" is a reserved character in
 * LCLs default configuration file syntax.
 *
 * The work of evaluating these functions is actually done in Replacers::sys_replace. But
 * descriptions fits better here!
 *
 * The function is designed to evaluate inside to outside which means that inside tags are
 * replace before outside tags (see example).
 *
 * \code
LString exp = ""; // defined to detect tags as $(TAGNAME)
LString str;      // hold out new string

LHash<LString,LString> hash;
hash.set ( "R1", "-R1-" ); // defined and not empty
hash.set ( "R2", "" );     // defined but empty

LString template = "R1: $(R1) R2: $(R2) R3: $(R3:-R3) R4:";

str = template.replace ( LStringHashReplacer(), exp );

// str would be "R1: -R1- R2: R3: R3 R4:"


\endcode
 *
 * This example only shows how it is basically working. Refer to the example attached for
 * more information. 
 *
 * \note Your regular expressions needs also to recognize characters like : or ? in the
 * tag name since they can be used for special tag functions.
 * \param replacer An instance of any class used for replacing tags in the string
 * \param _regEx Is the regular expression used do detect tags in the string
 * \param maxrec The number of maximum recursions allowed in replacing. You should leave the
 * default value unless you really need it.
 * \return The replaced string
 * \post The current string will not changed. The result is returned in a new string.
 */

LString LString::replace ( Replacer const & replacer, LString const & _regEx, luint32 maxrec ) const
{
  LRegEx exp ( _regEx );
  if ( !exp.isValid() )
    return *this;

  return doReplace ( *this, replacer, exp, maxrec );
}

//! Replaces a set of variables in a string which come from a hash table
/*!

 * This function parses a string for the occurence of variables and replaces theses variables whith the values found in the hash table. The variable syntax
 * is equal the the Makefile variable syntax. \b $(VARIABLE)
 * The variables should be stored in the hash table by using \c LHash::set ( "VARNAME", "VARVALUE" );
 * If a variable is not defined it is replaced with an empty string.
 * The current string is not modified but the new one is returned.

 * \param table A Hash table which contains the variables/value pairs
 * \param caseSens true if the variable names are case sensitive. If you want case insensitive variables they have to be insereted to the hash table as upper-case strings
 * \return Returns the filled up string
 * \code 
 LString template = "Hello World! My name is $(NAME) and I like $(LIKE)!";

 LHash<LString,LString> hash;
 hash.set ( "NAME", "Arthut Dent" );
 hash.set ( "LIKE", "Sandwiches" );

 LString filled1 = template.replaceVars ( hash );
 // is now "Hello World! My name is Arthur Dent and I like sandwiches!"

 hash.set ( "NAME", "Tux" );
 hash.set ( "LIKE", "snow" );

 LString filled2 = template.replaceVars ( hash );
 // is now "Hello World! My name is Tux and I like snow!"


 * \endcode
 */

LString LString::replaceVars ( LHash<LString, LString> const & table, bool caseSens ) const
{
  return replace ( LStringHashReplacer ( table, caseSens ) );
}

//! Replaces like LString::replaceVars but from the environment
/*!
 * Acutally this method calls LString::replace by using the class LStringEnvReplacer.
 */
LString LString::replaceWithEnvs ( ) const
{
  return replace ( LStringEnvReplacer () );
}

/*!
 * \return the searched and replaced string
 * \param regexp the regular expression used for search and replace
 * \param replaceTag the replace tag describing how the replace a match
 *
 * This method will go throught the string searching and replacing using a regular expression. This is
 * compareable to the Perl "s/find/replace/" method.
 *
 * The replacement tag uses $0 .. $n for inserting the sub-matches.
 *
 * \code

LString str1 = "This is foo test!";

LString rc = str1.searchAndReplace ( "foo", "bar" );
// rc is now "This is bar test!"

LString str2 = "From: <ctron@dentrassi.de>";

rc = str2.searchAndReplace ( "From: (.*)", "To: $1" );
// rc is now "To: <ctron@dentrassi.de>"

 * \endcode
 */

LString LString::searchAndReplace ( LString const & regexp, LString const & replaceTag ) const
{
  if ( !length() )
    return LString ();

  LRegEx regExp ( regexp );

  LString afterMatch = *this;
  LString newString;
  long i;

  while ( regExp.exec ( afterMatch ) && afterMatch.length() ) {
    newString += regExp.beforeMatch ( 0 );
    afterMatch = regExp.afterMatch ( 0 );
    //    cout << regExp.beforeMatch ( 0 ) << " <-> " << regExp.afterMatch ( 0 ) << endl << flush;

    for ( i = 0; i<replaceTag.length(); i++ ) {
      if ( replaceTag.get(i) == '$' ) {
	if ( replaceTag.get(i+1) == '$' ) {
	  i++;
	  newString += '$';
	} else {
	  newString += regExp.match ( (long)LString(replaceTag.get(i+1)) );
	  i++;
	}
      } else {
	newString += replaceTag.get(i);
      }
    }

  }

  newString += afterMatch;

  return newString;
}

// *****************************************************************************************************
// *****      CoDec Functions
// *****************************************************************************************************


LString LString::deHex ()
{
  int i, len = length();
  LString str;
  LString hex;
  char c;

  for (i=0; i<len; i++) {
    c = string_ptr[i];
    if ( c=='%' ) {
      hex = "0x00";
      hex[(long)2] = string_ptr[i+1];
      hex[(long)3] = string_ptr[i+2];
      c = (char)((unsigned long)hex);
      str += c;
      i+=2;
      
    } else {
      if ( c == '+' ) {
	str += ' ';
      } else {
	str += c;
      }
    }
  }

  (*this) = str;
  return str;
}

LString LString::enHex ()
{
  LString str;
  int i, len=length();
  char c;
  char buffer[3];

  for (i=0; i<len; i++) {
    c = (*this)[(long)i];
    if ( !isalnum(c) ) {
      sprintf ( buffer, "%02X", (int)c );
      str += "%";
      str += buffer;
    } else {
      str += c;
    }
  }

  return str;
}

#define EN_SAVE(s,r) case s: str += r; break;

LString LString::enSave ( LString const & unprint ) const
{
  char buffer[10];
  int len = length();
  LString str;
  for ( int i = 0; i<len; i++ )
    {
      char c = get(i);
      switch ( c )
	{
	  EN_SAVE('\n',"\\n");
	  EN_SAVE('\r',"\\r");
	  EN_SAVE('\t',"\\t");
	  EN_SAVE('\b',"\\b");
	  EN_SAVE('\"',"\\\"");
          EN_SAVE('\'',"\\\'");
	  EN_SAVE('\\',"\\\\");
	  EN_SAVE('\a',"\\a");
	  EN_SAVE('\v',"\\v");
	  EN_SAVE('\?',"\\?");
	  EN_SAVE('\0',"\\0");

	default:
	  if ( (!isprint(c)) || unprint.contains(c) )
	    {
	      sprintf ( buffer, "\\x%02x", c );
	      str += buffer;
	    }
	  else
	    {
	      str += c;
	    }
	  break;
	}
    }
  return str;
}

#define DE_SAVE(s,r) case s: if ( escaped ) str += r; else str+=s; escaped = false; break;

LString LString::deSave () const
{
  LString str;
  int len = length();
  bool escaped = false;

  for ( int i = 0; i<len; i++ )
    {
      char c = get(i);
      switch ( c )
	{
	case '\\':
	  if ( escaped )
	    {
	      str += '\\';
	      escaped = false;
	    }
	  else
	    escaped = true;
	  break;

	  DE_SAVE('a','\a');
	  DE_SAVE('b','\b');
	  DE_SAVE('n','\n');
	  DE_SAVE('r','\r');
	  DE_SAVE('v','\v');
	  DE_SAVE('t','\t');
	  DE_SAVE('?','\?');
	  DE_SAVE('0','\0');
	  DE_SAVE('\"','\"');
	  DE_SAVE('\'','\'');
	case 'x':
	  if ( escaped )
	    {
	      char t[3];
	      int j;
	      t[0] = get(i+1);
	      t[1] = get(i+2);
	      t[2] = 0;
	      sscanf ( t, "%x", &j );
	      str += (char)j;
	      i+=2;
	    }
	  else
	    str += c;
	  escaped = false;
	  break;

	default:
	  str += c;
	  escaped = false;
	  break;
	}
    }
  return str;
}

/*!
 * \return the filled hash
 * \param regexp The regular expression used for parsing
 *
 * The method parses the string using the provided regular expression into key/value pairs and fills 
 * them into the hash table.
 *
 * The key must be the first match the value the second.
 */

LHash<LString,LString> LString::parseToHash ( LString const & regexp ) const
{
  LRegEx exp ( regexp );
  LString temp = *this;
  LHash<LString,LString> hash;

  while ( exp.exec ( temp ) && temp.length() ) {
    //    lout << exp.match(0) << " - " << temp << endl << flush;
    if ( LApplication::debugFlag ) lout << "LString -> parse to hash: " + exp.match(1) + " = " + exp.match(2) << endl << flush;
    hash.set ( exp.match ( 1 ), exp.match(2) );
    temp = exp.afterMatch ( 0 );
  }
  return hash;
}

//! Create a string using printf
/*!
 *
 * This functions provides sprintf functionality for LString.
 * A new string is generated which is created by taking the format string and
 * the variable argument list and passing these over to printf.
 *
 * On systems where it is possible, this function will perform a out of bounds check
 * for the generated string. This must not be larger than the provided maximum. But
 * since this might not available on all systems it is no functionality on
 * which you can rely. 
 *
 * Therefore it is recommended to specify maxlen as big as needed.
 *
 * If the system can check for an out of bounds condition the newly generated
 * string will be cut at this point.
 *
 * During runtime this functions needs at least maxlen bytes on the heap to generate the
 * new string. Afterwards only the memory which is needed for the string to fit in
 * LString memory chuncks is allocated.
 *
 * \return the formatted string
 * \param maxlen The maximum lenght of the generated string.
 * \param format the format of the string to be generated (in printf style)
 * \param ... all parameters passed on to printf
 * \warning On systems where a limited printf is not available the function might crash if
 * the generated string exceeds maxlen
 */

LString LString::fromPrintf ( luint32 maxlen, char const * format, ... )
{
  va_list vap;
  va_start(vap, format);

  char * buffer = (char*)::malloc ( maxlen );
  if ( buffer == NULL )
    throw LOutOfMemoryException(__FILE__,__LINE__);

  buffer[0] = 0;

  #ifdef HAVE__VSNPRINTF
  _vsnprintf ( buffer, maxlen, format, vap );
  #else
  vsprintf ( buffer, format, vap );
  #endif

  LString str = buffer;

  ::free(buffer);

  va_end(vap);

  return str;
}


LCL_EXPORT LSerializer & operator << ( LSerializer & ser, LString const & str )
{
  ser.freeze ( "string", str );
  return ser;
}

LCL_EXPORT LSerializer & operator >> ( LSerializer & ser, LString & str )
{
  str = ser.thaw ( "string" );
  return ser;
}

LCL_EXPORT bool operator == ( char const * p1, LString const & p2 )
{
  return p2 == p1;
}

// LString::Replacer

/*!
 * \internal
 */

LString LString::Replacer::sys_replace ( LString const & tag ) const
{
  LRegEx exp ( "(#?)([^:!\\?\\|]*)([:\\|\\?!]?[-+]?)([^:!\\|]*)([\\:!\\|]?)(.*)" );
  if ( exp.exec(tag) )
    {
      LString rep;
      bool isNull, isEmpty = true;
      LString head = exp.match(1);
      LString tag = exp.match(2);
      LString cmd = exp.match(3);
      LString arg1 = exp.match(4);
      LString cmd2 = exp.match(5);
      LString arg2 = exp.match(6);
      LString temp;

      try
	{
	  //	  cout << endl << "Replace: " << tag << endl << endl;
	  rep = replace ( tag );
	  isNull = false;
	  isEmpty = rep.length()==0;
	}
      catch ( TagNotFound & tnf )
	{
	  isNull = true;
	  isEmpty = true;
	}

      if ( (cmd == ":-") || (cmd =="!-"))
	return (head=="#") ?
	  LString(((isNull || isEmpty) ? arg1 : rep).length()) :
	  ((isNull || isEmpty) ? arg1 : rep);

      else if ( cmd == "?-" )
	return (head=="#") ? 
	  LString((isNull ? arg1 : rep).length()) :
  	  (isNull ? arg1 : rep);

      else if ( (cmd == ":+") || (cmd == "!+") )
	return (head=="#") ?
	  LString((!(isNull || isEmpty) ? arg1 : rep).length()) :
	  (!(isNull || isEmpty) ? arg1 : rep);

      else if ( cmd == "?+" )
	return (head=="#") ?
	  LString((!isNull ? arg1 : rep).length()) :
	  (!isNull ? arg1 : rep);

      else if ( (cmd == ":") || (cmd == "|") )
	if ( ((long)arg2) > 0 )
	  return (head=="#") ? 
	    LString((rep.subString((luint32)arg1, (luint32)arg2)).length()) :
            (rep.subString((luint32)arg1, (luint32)arg2));
	else
	  return (head=="#") ?
	    LString((rep.subString((luint32)arg1)).length()) :
	    (rep.subString((luint32)arg1));

      else if ( (cmd == "?") && ( (cmd2==":") || (cmd2=="!") ) )
	{
	  temp = isNull ? arg2 : arg1;
	  return (head=="#") ? LString(temp.length()) : temp;
	}
      else if ( (cmd == "!") && ( (cmd2==":") || (cmd2=="!") ) )
	{
	  temp = (isNull || isEmpty) ? arg2 : arg1;
	  return (head=="#") ? LString(temp.length()) : temp;
	}

      else if ( head == "#" )
	return rep.length();

      return rep;
    }
  else
    {
      try
	{
	  return replace ( tag );
	}
      catch ( TagNotFound & tnf )
	{
	  return "";
	}
    }
}

