/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#include <lcl/lstringbase.h>

#include <lcl/lxcpt.h>

#include <string.h>
#include <malloc.h>

const luint16 LStringBase::C_block_size = 64;


LCL_EXPORT char * lcl_strdup ( char const * str )
{
  //  LTRACE_FUNC ( "lcl", "char * lcl_strdup ( char const * str )" );
  #ifdef _LCL_COMPILER_MSVC_
  unsigned long size = strlen ( str );
  char * tmp = (char*)malloc ( size+1 );
  strcpy ( tmp, str );
  return tmp;
  #else
  return strdup ( str );
  #endif
}

LCL_EXPORT void * lcl_realloc ( void * ptr, unsigned long size )
{
  //  LTRACE_FUNC ( "lcl", "void * lcl_realloc ( void * ptr, unsigned long size )" );

  #ifndef _LCL_COMPILER_MSVC_
  return realloc ( ptr, size );
  #else
  if ( ptr == NULL )
    return malloc ( size );

  void * tmp = malloc ( size );
  memcpy ( tmp, ptr, MIN(_msize(ptr),size) );
  free ( ptr );
  return tmp;
  #endif
}


#ifdef HAVE_MEMMOVE
#else
LFEATURE("LCL String", __lcl_str_base_memmove_1, "Emulated memmove()" );
LCL_EXPORT void * lcl_memmove ( void * dest, const void * src, size_t n )
{
  throw 1; // FIXME: need implementation
  return NULL;
}
#endif

/*!
 * adds more memory to the string pointer if needed
 */
void LStringBase::fitMem ( luint32 top )
{  
  unsigned long new_size = top;
  if ( new_size % C_block_size )
    {
      new_size = ( C_block_size - ( new_size % C_block_size ) ) + new_size;
    }

  if ( ptr_space == new_size )
    return;

  // re-alloc more memory
  string_ptr = (char*)lcl_realloc ( string_ptr, new_size );
  if ( !string_ptr )
    {
      if ( new_size )
	throw LOutOfMemoryException(__FILE__,__LINE__);
      ptr_space = new_size;
    }
  else
    {
      string_ptr[new_size-1]=0;
      ptr_space = new_size;
    }
}


/*!
 * Clears the string. Frees all memory and sets the internal string pointer to NULL again.
 *
 */
void LStringBase::clear ()
{
  if ( string_ptr )
    free ( string_ptr );

  string_ptr = NULL;
  ptr_space = 0;
}

LStringBase::LStringBase () :
  string_ptr ( NULL ),
  ptr_space ( 0 )
{
}

LStringBase::~LStringBase ()
{
  clear ();
}
