
#define _BSD_SOURCE

#include <lcl.h>
#include <lcl/lcl_config.h>

#ifdef HAVE_UTSNAME_H
#include <sys/utsname.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <lsysinfo.h>

#include <ltrace.h>
#include <lutils.h>
#include <lstreamproxy.h>

#define LSYSINFO_FUNCTION_UNSUPP(f) \
  f { throw LSystemInfoException ( LSystemInfoException::NOT_SUPPORTED , __FILE__, __LINE__ ); }

/*
#define LSYSINFO_FUNCTION(f,c) \
  #ifdef f\
  #else \
  f { throw LSystemInfoException ( LSystemInfoException::NOT_SUPPORTED , __FILE__, __LINE__ ); }
  #endif
*/

LSystemInfoException::LSystemInfoException ( LSystemInfoException::ErrorCode _ec, const char * _filename, unsigned long _line ) :
  LBasicException ( _filename, _line ),
  ec ( _ec )
{
}

void LSystemInfoException::log ( ) const
{
  LBasicException::log ();
  lerr << " >> System Info Exception!" << endl;;
  switch ( ec )
    {
    case NOT_SUPPORTED:
      lerr << "  >> Error Code: NOT_SUPPORTED" << endl;
      break;
    case NOT_AVAILABLE:
      lerr << "  >> Error Code: NOT_AVAILABLE" << endl;
      break;
    default:
      lerr << "  >> Error Code: <unkown>" << endl;
      break;

    }
  lerr << flush;
}

LSystemInfoException::ErrorCode LSystemInfoException::getErrorCode () const
{
  return ec;
}



LSystemInfo * LSystemInfo::theInstance = NULL;

LSystemInfo::LSystemInfo ()
{
  
}

LSystemInfo * LSystemInfo::getInstance ()
{
  if ( !theInstance )
    theInstance = new LSystemInfo;

  return theInstance;
}

LString LSystemInfo::getHostName () const
{
  char buffer[1024];
  if ( gethostname ( buffer, sizeof(buffer) ) == -1 )
    {
      LTRACE_PERROR ( );
      throw LSystemInfoException ( LSystemInfoException::UNKOWN );
    }
  else
    {
      return buffer;
    }
}

// uname

#ifdef HAVE_UNAME

LString LSystemInfo::getMachine () const
{
  struct utsname uinfo;
  uname ( &uinfo );
  return uinfo.machine;
}

LString LSystemInfo::getOSRelease () const
{
  struct utsname uinfo;
  uname ( &uinfo );
  return uinfo.release;
}

LString LSystemInfo::getOS() const
{
  struct utsname uinfo;
  uname ( &uinfo );
  return uinfo.sysname;
}

LString LSystemInfo::getOSVersion() const
{
  struct utsname uinfo;
  uname ( &uinfo );
  return uinfo.version;
}

#else

LSYSTEMINFO_FUNCTION_UNSUPP(LString LSystemInfo::getMachine () const)
LSYSTEMINFO_FUNCTION_UNSUPP(LString LSystemInfo::getOSRelease () const)
LSYSTEMINFO_FUNCTION_UNSUPP(LString LSystemInfo::getOSVersion () const)
LSYSTEMINFO_FUNCTION_UNSUPP(LString LSystemInfo::getOS () const)

#endif
