/* ****************************************************************************
   |      File: lthread.cc
   |      Contains: class LThread
   |      Created: 17.03.1998
   |      Last Changed: 11.04.1998
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |
   ************************************************************************* */

/* ************************************************************************* *
 * Change log:                                                               *
 * -------------                                                             *
 *                                                                           *
 *   -> [v0.00] - 05.03.1998 - Created file and started coding               * 
 *   -> [v0.00] - 10.04.1998 - Moved content to lthread.h coz templated      *
 *                             thread                                        *
 *   -> [v0.00] - 11.04.1998 - Moved back to lthread.cc :)                   *
 *   -> [v0.00] - 22.11.1998 - Added copy ctor for LThread                   *
 *   -> [v0.00] - 10.04.2000 - WIN32 thread control                          *
 *   -> [v0.00] - 21.04.2000 - Added LVCS and more cast LThreadFNs           *
 * ************************************************************************* */

#include <lcl.h>

// **** Prototypes

void * LThreadStarter ( void * ptr );

// **** LCL Includes

LCL_EXPORT int countLThreadFn = 0;
LCL_EXPORT int countLThread = 0;

#include <lcl/lthread.h>
#include <lcl/lstreamproxy.h>
#include <lcl/lvcs.h>
#include <lcl/ltrace.h>
#include <lcl/lxcpt.h>
#include <lcl/lrevision.h>

#include <signal.h>
#include <stdio.h>

#ifndef PURE_WIN32

LFEATURE("LCL", __lcl_f_threads_posix, "POSIX Threading" );

LVCS_INIT_D(LThread, " Posix" );
LVCS_INIT_D(LThreadFn, " Posix" );
LVCS_INIT(LFuncThreadFn);
template <class M> LVCS_INIT_D(LCastThreadFn<M>, " Posix" );
template <class Object> LVCS_INIT_D(LMethodThreadFn<Object>, " Posix" );
//template <class Object, class P1> LVCS_INIT_D(LMethodCastThreadFn<Object>, " Posix" );

bool thread_attr_init = false;
pthread_attr_t lcl_thread_attrs_detached;
pthread_attr_t lcl_thread_attrs_joinable;

void * LThreadStarter ( void * ptr )
{
  LTRACE ( "LThreadStarter", "Starting new thread" );

  LThreadFn * threadFn = (LThreadFn*)ptr;

  try {
    threadFn->run();
  }
  catch ( LBasicException & e ) {
    e.log ();
  }
  catch ( int i )
    {
      lerr << "Caught integer exception: " << i << endl << flush;
    }
  catch ( const char * str ) {
    lerr << "Cought: " << str << endl << flush;
  }
  catch ( ... ) {
    lerr << "Unkown exception cought in LThreadStarter!!" << endl << flush;
  }
  if ( threadFn->deleteMe )
    delete threadFn;

  LTRACE ( "LThreadStarter", "Finished!" );

  return NULL;
}

LThread::LThread ( LThreadFn * func )
{
  threadFunc = func;
  countLThread++;
}

LThread::LThread ( LThread const & p1 )
{
  threadFunc = p1.threadFunc;
  countLThread++;
  tid = p1.tid;
}

/*
LThread::LThread ( pthread_t _tid )
{
  threadFunc = NULL;
  countLThread++;
  tid = _tid;
}
*/

LThread::~LThread ()
{
  countLThread--;
}

/*LThread::run ()
{
  if ( threadFunc != NULL )
    threadFunc->run( );
}*/

/*!
 * Starts a new instance of the thread function
 * \post The new thread is started
 */

void LThread::start ( bool joinable )
{
  LTRACE_FUNC ( "LThread", "start" );

  if ( !thread_attr_init ) {
    LTRACE ( "LThread", "Initializing attributes for LThread framework" );
    pthread_attr_init ( &lcl_thread_attrs_detached );
    pthread_attr_setdetachstate ( &lcl_thread_attrs_detached, PTHREAD_CREATE_DETACHED );
    pthread_attr_init ( &lcl_thread_attrs_joinable );
    pthread_attr_setdetachstate ( &lcl_thread_attrs_joinable, PTHREAD_CREATE_JOINABLE );
    thread_attr_init = true;
  }

  int rc;
  if ( joinable )
    {
      rc = pthread_create ( &tid, &lcl_thread_attrs_joinable, LThreadStarter, (void*)threadFunc );
    }
  else
    {
      rc = pthread_create ( &tid, &lcl_thread_attrs_detached, LThreadStarter, (void*)threadFunc );
    }
  if ( rc ) {
    LTRACE ( "LThread", "Failed to create new thread: " + LString(rc) + " -> " + LString::fromLastError(rc) );
  }
}

/*!
 * Cancel a thread
 *
 * \note Under POSIX the thread will not be aborted. It will just the set to cancelled state. The POSIX thread environment will decide when
 * the thread will be stopped!
 *
 * \post the thread is cancelled
 */
void LThread::stop ()
{
  pthread_cancel ( tid );
}

/*!
 * \bug this is not working!
 */

void LThread::kill ()
{
  // pthread_kill ( tid, SIGKILL );
  NAMESPACE_STD(cout) << " -> Function not implemented\n";
}

/*!
 * \bug this is not working (at last it is only for joinable threads)!!!
 */

void * LThread::waitFor ()
{
  LTRACE_FUNC ( "LThread", "void * LThread::waitFor ( )" );

  void * ptr = NULL;
  int rc = pthread_join ( tid, &ptr );

  if ( rc == EINVAL )
    {
      throw NotJoinableException ( __FILE__, __LINE__, tid );
    }

  return ptr;
}


#else 

LFEATURE("LCL", __lcl_f_thread_win32, "Win32 Threading" );

#ifdef _LCL_ENV_NUTCRACKER_

#include <nutc.h>
#include <winnutc.h>

LFEATURE("LCL", __lcl_f_thread_nutc, "NuTCRACKER Threading Option" );
#else

#endif

LVCS_INIT_D(LThread, " WIN32" );
template <class M> LVCS_INIT_D(LCastThreadFn<M>, "WIN32" );
template <class Object> LVCS_INIT_D(LMethodThreadFn<Object>, "WIN32" );
template <class Object, class P1> LVersionControlObject LMethodCastThreadFn<Object,P1>::vcsObject = LVersionControlObject ( "LMethodCastThreadFn<Object,P1> WIN32" , __TIMESTAMP__, __FILE__ );

bool lcl_thread_win32_init = false;

DWORD WINAPI Win32WrapperThreadFunc( LPVOID ptr )
{
  LThreadFn * threadFn = (LThreadFn*)ptr;

#ifdef _LCL_ENV_NUTCRACKER_
  _NutEnableNuTC(NULL);
#endif

  try {
    if ( ptr )
      threadFn->run();
    else
      lerr << "PANIC -- Win32WrapperThreadFunc called with NULL!" << endl;
  }

  catch ( LBasicException & e ) {
    e.log ();
  }
  catch ( const char * str ) {
    lerr << "Cought: " << str << endl << flush;
  }
  catch ( ... ) {
    lerr << "Unkown exception cought in LThreadStarter!!" << endl << flush;
  }

  if ( threadFn->deleteMe )
    delete threadFn;

#ifdef _LCL_ENV_NUTCRACKER_
  _NutDisableNuTC();
#endif

  return NULL;
}

LThread::LThread ( LThreadFn * func )
{
  threadFunc = func;
  countLThread++;
}

LThread::LThread ( LThread const & p1 )
{
  threadFunc = p1.threadFunc;
  countLThread++;
  tid = p1.tid;
}

LThread::~LThread ()
{
  countLThread--;
}

void LThread::start ( bool joinable )
{
  DWORD _tid;
  tid = CreateThread ( NULL, 0L, Win32WrapperThreadFunc, threadFunc, 0L, &_tid ); 
}

void * LThread::waitFor ()
{  
  DWORD ptr;
  WaitForSingleObject ( tid, INFINITE );
  GetExitCodeThread ( tid, &ptr );
  return (void*)ptr;
}

void LThread::stop()
{
  TerminateThread ( tid, 0 );
}

void LThread::kill ()
{
  stop();
}

#endif

LThreadID LThread::currentID()
{
  return LGetCurrentThreadID ();
}

LThread::NotJoinableException::NotJoinableException ( char const * _file, luint32 _line, LThreadID _threadID ) :
  LBasicException ( _file, _line, "Thread is not joinable" ),
  threadID ( _threadID )
{
}

LThreadID LThread::NotJoinableException::getThreadID () const
{
  return threadID;
}

void LThread::NotJoinableException::log ( void ) const
{
  LBasicException::log ();
  lerr << " >> LThread::NotJoinable Exception (" + LString((luint64)threadID) + ")" << endl << flush;
}
