#include <lcl/lticker.h>


LTicker::LTicker ( LString const & name ) :
  m_name ( name )
{
}

LTicker::~LTicker ()
{
}

void LTicker::tick ( void )
{
  tickerNamed.emit ( m_name );
  ticker.emit ();
}

/*!
 * \param timeout Timeout period in seconds
 * \param name The name of this ticker. Used to identify the ticker in an event
 */

LAsyncTicker::LAsyncTicker ( unsigned long timeout, LString const & name ) :
  LTicker ( name ),
  runnerFn ( &LAsyncTicker::runner, *this ),
  running ( false ),
  exited ( false ),
  m_timeout ( timeout )
{
  LThread tco ( &runnerFn );
  tco.start ();
}

LAsyncTicker::~LAsyncTicker ()
{
  running = false;
  while ( !exited ) lcl_yield ();
}

void LAsyncTicker::runner ( void )
{
  unsigned long last = 0;

  running = true;
  while ( running )
    {
      if ( difftime ( time(0), last ) >= m_timeout )
	{
	  last = time(0);
	  LTicker::tick ();
	}
      lcl_usleep ( 100*1000 );
    }
  exited = true;
}


void LAsyncTicker::tick ( void )
{
}
