
#include <lcl.h>

#include <lcl/ltrace.h>
#include <lcl/lstreamproxy.h>
#include <lcl/ldatetime.h>
#include <lcl/ltracechannel.h>
#include <lcl/lregex.h>

#include <string.h>
#include <time.h>

#include <lcl/ltrace_file.h>
#include <lcl/lapp.h>
#include <lcl/llogsystem.h>


// LTraceOutput_File

LGuardMutex LTraceOutput_File::syncMutex;

LTraceOutput_File::LTraceOutput_File () :
  fileOutputFD ( NULL ),
  fileOutput ( NULL ),
  m_maxSize ( 5*1024*1024 ),
  m_maxFiles ( 3 ),
  m_size ( 0 ),
  m_fileNr ( 0 ),
  m_use_tpl ( false )
{
}

LTraceOutput_File::~LTraceOutput_File ()
{
  closeOutput ();
}

void LTraceOutput_File::closeOutput ( void )
{
  LAutoLock _lock ( syncMutex );

  if ( fileOutput )
    {
      delete fileOutput;
      fileOutput = NULL;
    }
  if ( fileOutputFD )
    {
      delete fileOutputFD;
      fileOutputFD = NULL;
    }
}

void LTraceOutput_File::setFile ( LString const & fname )
{
  LAutoLock _lock ( syncMutex );
  closeOutput ();

  fileOutputFD = new LFile ( fname, LFDManager::write | LFDManager::create | LFDManager::truncate );
  if ( !fileOutputFD->isValid() )
    lerr << "Could not open file: " << fname << " for writing (" << LString::fromLastError() << ")!" << endl << flush;
  else
    fileOutput = new LFOStream ( *fileOutputFD );
}

void LTraceOutput_File::setOutput ( LString const & location )
{
  LAutoLock _lock ( syncMutex );

  m_use_tpl = false;
  m_location = location;

  setFile ( m_location );
}

LString LTraceOutput_File::getFileName () const
{
  LHash<LString,LString> hash;
  hash.set ( "NR", m_fileNr );  
  return m_template.replaceVars(hash);
}

void LTraceOutput_File::setTemplate ( LString const & tpl )
{
  LAutoLock _lock ( syncMutex );
  
  m_use_tpl = true;
  m_template = tpl;

  setFile ( getFileName() );
}

void LTraceOutput_File::setOptions ( LHash<LString,LString> const & vars )
{
  if ( vars.isSet ( "output" ) )
    {
      setOutput ( vars.get ( "output" ) );
    }
  if ( vars.isSet ( "template" ) )
    {
      setTemplate ( vars.get ( "template" ) );
    }
  if ( vars.isSet ( "max_files" ) )
    {
      m_maxFiles = vars.get( "max_files" );
    }
  if ( vars.isSet ( "max_size" ) )
    {
      m_maxSize = vars.get ( "max_size" );
    }
}


void LTraceOutput_File::trace ( LTraceMessage const & msg )
{
  char * buffer = (char*)malloc ( strlen ( msg.getChannel() ) + strlen(msg.getMessage()) + strlen(msg.getFile() ) + 1024 );

  sprintf ( buffer, "[%ld.%ld] [%ld.%ld] [%s:%ld] %s -> %s", msg.getTime(), msg.getUTime(), msg.getPID(), msg.getTID(), msg.getFile(), msg.getLine(), msg.getChannel(), msg.getMessage() );

  LAutoLock _lock ( syncMutex );
  if ( fileOutput )
    {
      *fileOutput << buffer << endl << flush;
    }

  // Now check if we need to change the file...
  if ( m_use_tpl )
    {
      m_size += strlen ( buffer ) + 2;
      if ( m_size >= m_maxSize )
	{
	  m_fileNr++;
	  if ( m_fileNr >= m_maxFiles )
	    m_fileNr=0;
	  m_size = 0;
	  setFile ( getFileName() );
	}
    }

  free ( buffer );
}
