#include <lcl/lutils.h>
#include <lcl/lhash.h>
#include <lcl/lvcs.h>
#include <lcl/lstringtokenizer.h>
#include <lcl/lrevision.h>
#include <lcl/ltrace.h>

#include <string.h>
#include <stdlib.h>

#include <lcl.h>

LTRACE_CHANNEL_STATIC("perror", __lcl_trace_perror_1, "Operating system errors" );

#ifndef HAVE_USLEEP
LFEATURE("LCL Utils", lcl_u__f1, "Emulated usleep()" );
void usleep ( unsigned int microsecs )
{
  struct timeval tv;
  tv.tv_sec = microsecs / 1000000;
  tv.tv_usec = microsecs % 1000000;
  select ( 0, NULL, NULL, NULL, &tv );
}
#endif

#ifdef _LCL_SYS_WIN32_

LCL_EXPORT LHash<LString,LString> getEnvVars ()
{
  LHash<LString,LString> hash ( 10 );
  LString str, key, value;
  int i = 0;

  char * myenvs = GetEnvironmentStringsA();

  bool inkey = true;
  bool lastNull = false;
  bool test = (!lastNull) || (myenvs[i]!=0);
  while ( test )
    {
      lastNull = ( myenvs[i] == 0 );
      if ( lastNull )
	{
	  hash.set ( key, value );
	  inkey = true;
	  key = value = "";
	  i++;
	  test = (!lastNull) || (myenvs[i]!=0);
	  continue;
	}
	
      if ( inkey )
	{
	  if ( myenvs[i] != '=' )	    
	    key += myenvs[i];
	  else
	    inkey = false;
	}
      else
	{
	  value+= myenvs[i];
	}
      i++;
      test = (!lastNull) || (myenvs[i]!=0);
    }

  FreeEnvironmentStringsA ( myenvs );

  return hash;
}

#else

extern char ** environ;

LCL_EXPORT LHash<LString,LString> getEnvVars ()
{
  LHash<LString,LString> hash ( 10 );
  LString str, key, value;
  int i = 0;

  char ** lenviron_ptr;

  lenviron_ptr = environ;

  while ( (lenviron_ptr)[i] ) {
    i++;
    str = (lenviron_ptr)[i];
    str.setDelims("=");
    key = str.word(0);
    value = str.subString ( key.length()+1 );
    //    cout << key << "<=>" << value << "\n";
    hash.set ( key, value );
  }

  return hash;
}


#endif

#ifdef _LCL_SYS_WIN32_

LVersionControlObject vcsObject_lutils ( "LUtils WIN32", __LCL_TIMESTAMP__, __FILE__ );

/*
#ifdef BUILD_LCL_DLL
_declspec(dllimport)
#endif
extern char ** _environ;

char *** lenviron_ptr = &_environ;
*/

LCL_EXPORT unsigned long getLastError ()
{
  return GetLastError ();
}

LCL_EXPORT unsigned long LGetProcessID ()
{
  return GetCurrentProcessId();
}

LCL_EXPORT LThreadID LGetCurrentThreadID ()
{
  return GetCurrentThreadId ();
}

LCL_EXPORT void lcl_sleep ( unsigned long thetime )
{
  Sleep ( thetime * 1000 );
}

LCL_EXPORT void lcl_usleep ( unsigned long usecs )
{
  Sleep ( usecs / 1000 );
}

LCL_EXPORT void lcl_yield ( void )
{
  Sleep ( 0 );
}


#else /* end of WIN32 only section */


#ifdef HAVE_NANOSLEEP
LFEATURE("LCL Utils", lcl_nanosleep_f1, "Emulated lcl_usleep() with nanosleep()" );
LCL_EXPORT void lcl_usleep ( unsigned long usecs )
{
  struct timespec ts_in, ts_out;
  ts_in.tv_sec = usecs / 1000000;
  ts_in.tv_nsec = (usecs % 1000000) * 1000;
  while ( (nanosleep ( &ts_in, &ts_out ) == -1) && (errno==EINTR) )
    {
      ts_in = ts_out;
    }  
}

#else

LCL_EXPORT void lcl_usleep ( unsigned long usecs )
{
  usleep ( usecs );
}

#endif

LCL_EXPORT void lcl_yield ( void )
{
  sched_yield ();
}

LVersionControlObject vcsObject_lutils ( "LUtils GNU/Posix/Unix/NutCracker", __LCL_TIMESTAMP__, __FILE__ );
/*
extern char ** environ;
char *** lenviron_ptr = &environ;
*/

LCL_EXPORT unsigned long LGetProcessID ()
{
  return getpid();
}

LThreadID LGetCurrentThreadID ()
{
  return pthread_self ();
}

LCL_EXPORT unsigned long getLastError ()
{
  return errno;
}

LCL_EXPORT void lcl_sleep ( unsigned long thetime )
{
  sleep ( thetime );
}

#endif /* end of posix section */


LCL_EXPORT unsigned long hash ( char const * p1, unsigned long size, unsigned long hash_size )
{
  unsigned long i, hash;
  unsigned long hash_key=0, hk2=0;

  for ( i=0; i<size; i++) {
    hash_key += p1[i]*i;
  }
  for ( i=0; i<size; i+=2) {
    hk2 += p1[i];
  }

  hash_key = hash_key*hk2;

  hash = hash_key % hash_size;

  //    cout << "Hash Key " << hash << "\n"; cout.flush();

  return hash;
}

LCL_EXPORT unsigned long hash ( char const * p1, unsigned long hash_size )
{
  return hash ( p1, strlen ( p1 ), hash_size );
}

int GetProtocolByName ( char * name )
{
  struct protoent * pi;
  pi = getprotobyname ( name );

  if ( pi == NULL ) {
    return -1;
  } else {
    return pi->p_proto;
  }
}

#ifdef __NUTC__

#include <nutc.h>

LCL_EXPORT LString Lnutc2win32 ( LString path )
{
  char buffer_in[PATH_MAX+1];
  char buffer_out[PATH_MAX+1];

  strcpy ( buffer_in, path );

  _NutPathToWin32( buffer_in, buffer_out, TRUE );

  return buffer_out;
}

#endif
