#include <lcl/lapp.h>
#include <lcl/lstreamproxy.h>

/*
 * Basic application showing the use of LApplication
 */

/*
 * How will LApplication::init parse command line parameters?
 *
 * assume the application name is "app" and you execute:
 *   ./app -abc --debug --os linux -l 3
 *
 * This will split -abc up to 'a', 'b' and 'c'.
 * --debug will become 'debug'. '--os linux' will become 'os' with value 'linux' assigned.
 * 'l' will go to 'l' with value '3'.
 *
 * Easy, isn't it?
 */

void main ( int argc, char * argv[] )
{
  // theApp is globally defined in lapp.c/lapp.h

  // Init the main instance
  theApp.init ( argc, argv );

  /*
   * Parse LCL parameters
   * Only needed when you want to used tracing, logging and default LCL parameters.
   * Default parameters are desribed in LApplication::processLCLParameters
   */
  theApp.processLCLParameters ();

  // Test for 'a' which is set
  if ( theApp.isParam ( "a" ) )
    lout << "Option a is set!" << endl;

  // Test for 'd' which is not set
  if ( !theApp.isParam ( "d" ) )
    lout << "Option d is not set!" << endl;

  // Test for 'debug' which is set
  if ( theApp.isParam ( "debug" ) )
    lout << "Option debug is set!" << endl;

  // Test if 'l' is set and has a value assigned
  if ( theApp.hasParam ( "l" ) )
    lout << "'l' has a value!" << endl;

  // Test if 'debug' is set and has a value assigned
  if ( !theApp.hasParam  ( "debug" ) )
    lout << "'debug' has no value!" << endl;

  // Assume you want to read in the value of 'os'
  LString os = theApp.getParam ( "os" );
  // If 'os' is not set ... it will be an empty string

  // Get 'target'. If not set, use default value 'mars'
  LString target = theApp.getParam ( "target", "mars" );

  // Now get the value of 'l' which might also be set using 'level'
  int level = theApp.getParam ( "l", theApp.getParam ( "level" ) );
  // note: LString will convert to int automagically ;-)

  // Do the last step again but use a default value
  level = theApp.getParam ( "l", theApp.getParam ( "level", "1" ) );
}
