#include <lcl/lasciiproto.h>
#include <lcl/lsserver.h>

class MyProtocolHandler : public LASCIIProtocolHandler {
protected:
public:
  virtual void gotLine ( LString const & line );
  virtual luint32 maxMessageSize () const
  {
    return 4096;
  }
};

void MyProtocolHandler::gotLine ( LString const & line )
{
  if ( line == "QUIT" )
    {
      m_connection->tickLastData ();
      m_connection->sendLine ( "GOODBYE" );
      m_connection->requestClose ();
    }
  else if ( line.word(0) == "PING" )
    {
      m_connection->tickLastData();
      m_connection->sendLine ( "100 " + line.word(1) );
    }
  else
    {
      m_connection->tickLastData ();
      m_connection->sendLine ( "GOT: " + line );
    }
}

class MyProtocolFactory : public LASCIIProtocolFactory {
protected:
public:
  virtual LASCIIProtocolHandler * createHandler ( void )
  {
    return new MyProtocolHandler;
  }
};

int main ( int argc, char * argv[] )
{
  theApp.init ( argc, argv );
  theApp.processLCLParameters ();

  LSocketServer sserver;
  LSocketServer sserver2;
  LASCIIProtocolTCPServer pserver ( new MyProtocolFactory );

  LSockAddr4Inet addr = LSockAddr4Inet::any();
  addr.setPort ( 1202 );

  sserver.createSocket ();
  sserver.bind ( addr );
  sserver.listen ();
  sserver.start ();
  sserver.setAsync ( false );

  connect_emitter_1 ( &LASCIIProtocolTCPServer::newConnection, pserver, sserver.connection );

  LSockAddr4Inet addr2 = LSockAddr4Inet::any();
  addr2.setPort ( 6666 );

  sserver2.createSocket ();
  sserver2.bind ( addr2 );
  sserver2.listen ();
  sserver2.start ();
  sserver2.setAsync ( false );

  connect_emitter_1 ( &LASCIIProtocolTCPServer::newConnection, pserver, sserver2.connection );

  LSelectLoop::getInstance()->loop ();

  return 0;
}
